<copyright> BIT tag.Window extensions
    Written by <a href="mailto:tiggr@ics.ele.tue.nl">Pieter J. Schoenmakers</a>

    Copyright &copy; 1997-1998 Pieter J. Schoenmakers.
    All rights reserved.

    This file is part of TAG.  TAG is distributed under the terms of the
    TAG License, a copy of which can be found in the TAG distribution; see
    the file LICENSE.

    <id>$Id: Window.t,v 1.7 1998/07/21 15:06:24 tiggr Exp $</id>
    </copyright>

implementation class
Window: posing tag.Window, bit

end;

implementation instance
Window
{
  <doc> The selected views in this window.  This is the group of Views
      that are selected, within the bottom-most selected View that is
      their superview.  </doc>
  MutableArray selected_views;

  <doc> The previously recorded mouse location while {dragging}, or the
      original location of the mouse down when {selecting} or {linking}.
      </doc>
  float mouse_x, mouse_y;

  <doc> Operating mode of this window.  Iff {TRUE}, views in this window
      are manipulated instead of them responding in a normal way.  </doc>
  mutable boolean manipulating;

  <doc> Iff {TRUE}, we're dragging the selected views.  </doc>
  boolean dragging;

  <doc> Iff {TRUE}, we're dragging for making a selection.  </doc>
  boolean selecting;

  <doc> Iff {TRUE}, we're dragging for making a link.  </doc>
  boolean linking;
}

<doc> Overridden designated initializer.  </doc>
id
  initWithContentRect (float, float, float, float) (x, y, w, h)
	    styleMask int style
	      backing int backing
	       screen tag.Screen screen
{
  selected_views = [MutableObjectArray new];

  = [super initWithContentRect (x, y, w, h) styleMask style
	   backing backing screen screen];
}

boolean (changes)
  adjustSelection View responder
     incremental: boolean incr = NO
	  select: boolean do_select = YES
{
  int n = [selected_views length];

  if (responder == content_view)
    {
      if (!incr && n > 0)
	{
	  [selected_views makeElementsPerform selector (void bitDeselect)];
	  [selected_views truncate 0];
	  changes = YES;
	}
    }
  else
    {
      int j, i = [selected_views indexOfIdentical responder];

      if (!incr)
	{
	  if (i == -1)
	    {
	      [selected_views makeElementsPerform selector (void bitDeselect)];
	      [selected_views truncate 0];
	      if (do_select)
		{
		  [selected_views add responder];
		  [responder bitSelect];
		}
	      changes = YES;
	    }
	}
      else if (i == -1)
	{
	  [selected_views add responder];
	  [responder bitSelect];
	  changes = YES;
	}
      else
	{
	  [selected_views removeElementAt i];
	  [responder bitDeselect];
	  changes = YES;
	}
    }

  if (changes)
    [self display];
}

<doc> Update the position of the {link_hor} and {link_ver} windows.  </doc>
void
  updateLink (float, float) (cur_x, cur_y)
{
  [bit updateLinkFrom [self convertBaseToScreen (mouse_x, mouse_y)]
       to [self convertBaseToScreen (cur_x, cur_y)]];
}

<doc> If we are manipulating, manipulate the selected view or views.
    </doc>
void
  sendEvent MouseEvent event
	 to View responder
      using selector sel
{
  if (!manipulating)
    {
      [super sendEvent event to responder using sel];
      return;
    }

  if ([Runtime selector sel equals selector (void mouseDown Event)])
    {
      if (!!link_origin)
	[bit displayLink (nil, nil)];

      (mouse_x, mouse_y) = [event location];
      int mods = [event keyModifiers];
      if (!mods || mods == EVENT_MOD_SHIFT)
	{
	  /* Simple, possibly incremental, selection.  */
	  [self adjustSelection responder incremental: !!mods];
	}
      else if (mods == EVENT_MOD_CONTROL)
	{
	  /* Start off dragging the link from the currently targeted view.  */
	  [self adjustSelection responder select: NO];
	  if (responder != content_view)
	    {
	      float m_x, m_y, w, h;

	      (m_x, m_y, w, h) = [responder bounds];
	      (m_x, m_y) = [responder
			     convertPoint (m_x + w / 2.0, m_y + h / 2.0)
			     toView nil];

	      [bit displayLink (responder, responder)];

	      (mouse_x, mouse_y, m_x, m_y) = (m_x, m_y, mouse_x, mouse_y);
	      [self updateLink (m_x, m_y)];
	      linking = TRUE;
	    }
	}
      else
	[[[stdio err] print ("unhandled mods ", mods)] nl];
    }
  else if (!dragging && !linking
	   && [Runtime selector sel equals selector (void mouseDragged Event)]
	   && [selected_views length] > 0)
      {
	/* Mark those views being dragged as not being selected.  */
	[selected_views makeElementsPerform selector (void bitDeselect)];
	dragging = TRUE;
      }

  if (linking)
    if ([Runtime selector sel equals selector (void mouseDragged Event)])
      {
	float m_x, m_y;
	(m_x, m_y) = [event location];
	[self updateLink (m_x, m_y)];

	View v = [content_view hitTest (m_x, m_y)];
	View current = ![selected_views length] ? nil : selected_views[0];

	if (v == content_view)
	  v = nil;

	if (v != current)
	  if (!v)
	    [self adjustSelection content_view];
	  else
	    [self adjustSelection v];
      }
    else if ([Runtime selector sel equals selector (void mouseUp Event)])
      {
	int n = [selected_views length];

	if (n == 1)
	  [bit displayLink (link_origin, selected_views[0])];
	else
	  [bit displayLink (nil, nil)];
	linking = FALSE;
      }

  if (dragging)
    if ([selected_views length] > 0
	&& [Runtime selector sel equals selector (void mouseDragged Event)])
       {
	 float x, y;

	 (x, y) = [tag clipPoint [event location] to (0.0, 0.0, f_w, f_h)];

	 if (mouse_x != x || mouse_y != y)
	   {
	     float dx = x - mouse_x, dy = y - mouse_y;

	     // CCC This should be done with an invocation, but the resolver
	     // doesn't do that properly...
	     // Fri Jul 11 00:28:33 1997, tiggr@tricky.es.ele.tue.nl
	     // Invocation inv = [Invocation of selector ("v_moveBy__ff_")
	     // 			       with (dx, dy)];
	     // [selected_views makeElementsPerform inv];

	     int i, n = [selected_views length];
	     while (i < n)
	       [selected_views[i++] moveBy (dx, dy)];

	     // Very crude...
	     // Fri Jul 11 00:23:28 1997, tiggr@tricky.es.ele.tue.nl
	     [self display];

	     (mouse_x, mouse_y) = (x, y);
	   }
       }
     else if ([Runtime selector sel equals selector (void mouseUp Event)])
       {
	 dragging = FALSE;
	 [selected_views makeElementsPerform selector (void bitSelect)];
	 [self display];
       }
}

end;
