<copyright> Build Interfaces for TAG.
    Written by <a href="mailto:tiggr@ics.ele.tue.nl">Pieter J. Schoenmakers</a>

    Copyright &copy; 1997-1998 Pieter J. Schoenmakers.
    All rights reserved.

    This file is part of TAG.  TAG is distributed under the terms of the
    TAG License, a copy of which can be found in the TAG distribution; see
    the file LICENSE.

    <id>$Id: bit.t,v 1.10 1998/07/21 15:06:25 tiggr Exp $</id>
    </copyright>

implementation class
bit: stdio, tag, tag.Constants, Conditions
{
  <doc> The source of the current link, if any.  </doc>
  static View link_origin;

  <doc> For the active link (determined by {link_origin}, the destination.
      </doc>
  static View link_destination;

  <doc> The window (sort-of) we use for drawing the horizontal part of the
      link we are showing of .  </doc>
  static Window link_hor;

  <doc> Likewise, for the vertial part.  </doc>
  static Window link_ver;

  <doc> Width of the links.  2 is appropriate when no ornaments are being
      added by a window manager.  </doc>
  const LINK_WIDTH = 1.0;

  <doc> The palette window.  </doc>
  static Window palette_window;

  <doc> The palette view, currently the context view of the palette
      window.  </doc>
  static View palette_view;

  <doc> The window currently being manipulated.  </doc>
  static Window doc_window;

  <doc> The window containing the menu buttons.  </doc>
  static Window menu_window;
}

<doc> Update the link to the indicated (screen) coordinates.  </doc>
void
  updateLinkFrom (float, float) (f_x, f_y)
	      to (float, float) (t_x, t_y)
{
  float dif_x = t_x - f_x, dif_y = t_y - f_y;

  if (!dif_x)
    dif_x = 1.0;
  t_x = f_x + dif_x;

  if (!dif_y)
    dif_y = 1.0;
  t_y = f_y + dif_y;

  if (dif_x > 0.0)
    [link_hor setFrame (f_x, f_y, dif_x, LINK_WIDTH)];
  else
    [link_hor setFrame (t_x, f_y, -dif_x, LINK_WIDTH)];

  if (dif_y > 0.0)
    [link_ver setFrame (t_x, f_y, LINK_WIDTH, dif_y)];
  else
    [link_ver setFrame (t_x, t_y, LINK_WIDTH, -dif_y + LINK_WIDTH)];
}

<doc> Display the link from the {from} View to the {to} View.  Iff {from}
    is {nil}, the link is removed from the screen.  </doc>
void
  displayLink (View, View) (from, to)
pre
  !!from -> !!to
{
  if (!from)
    {
      if (!!link_origin)
	{
	  link_origin = link_destination = nil;
	  [link_hor orderOut self];
	  [link_ver orderOut self];
	}
    }
  else
    {
      link_origin = from;
      link_destination = to;

      float m_x, m_y, e_x, e_y, w, h;

      (m_x, m_y, w, h) = [link_origin bounds];
      (m_x, m_y) = [link_origin convertPoint (m_x + w / 2.0, m_y + h / 2.0)
				toView nil];
      (e_x, e_y, w, h) = [link_destination bounds];
      (e_x, e_y) = [link_destination convertPoint (e_x + w / 2.0, e_y + h / 2.0)
				     toView nil];

      [self updateLinkFrom [[link_origin window] convertBaseToScreen (m_x, m_y)]
	    to [[link_destination window] convertBaseToScreen (e_x, e_y)]];

      [link_hor orderFront self];
      [link_ver orderFront self];
    }
}

int
  main Array args
{
  ConditionClass cc = condition;

  bind ((cc,
	 {
	   [[err print ("signal: ", condition)] nl];
	   condition;
	 }))
    ({
      Application app = [Application shared];

      if (!app)
	= 1;
      else
	{
	  [self startup];

	  [Application run];

	  = 0;
	}
    });
}

void
  startup
{
  system_display = [tag.Display default];

  tag.Screen s = [system_display defaultScreen];
  palette_window = [s windowWithContentRect (100.0, 100.0, 300.0, 200.0)
		      styleMask WINDOW_STYLE_TITLED
		      backing BACKING_STORE_RETAINED];
  palette_view = [palette_window content_view];
  [palette_window set_manipulating TRUE];

  Box b = [[Box alloc] initFrame (10.0, 10.0, 60.0, 50.0)];
  [b set_title "Box"];
  [palette_view addSubview Any (b)];

  Slider sl = [[Slider alloc] initFrame (80.0, 10.0, 70.0, 30.0)];
  // Cast not necessary...
  // Mon Jul  7 21:15:35 1997, tiggr@tricky.es.ele.tue.nl
  SliderCell sc = SliderCell ([sl cell]);
  [sc set_max_value 2.0];
  [sc set_continuous YES];
  // CCC This cast should not be necessary...
  // Mon Jul  7 22:28:16 1997, tiggr@tricky.es.ele.tue.nl
  [palette_view addSubview Any (sl)];

  Slider sl2 = [[Slider alloc] initFrame (80.0, 70.0, 70.0, 30.0)];
  sc = SliderCell ([sl2 cell]);
  [sc set_max_value 1.0];
  [palette_view addSubview Any (sl2)];

  [sl setTarget sl2];
  [sl setAction selector (void takeDoubleValueFrom Control)];

  [palette_window orderFront self];

  menu_window = [s windowWithContentRect (500.0, 200.0, 300.0, 200.0)
		   styleMask WINDOW_STYLE_TITLED
		   backing BACKING_STORE_RETAINED];
  tag.View v = [menu_window content_view];
  tag.Button button = [[Button alloc] initFrame (10.0, 10.0, 125.0, 24.0)];
  [button set_auto_size VIEW_SIZE_WIDTH | VIEW_SIZE_MAXX];
  [button setTitle "Test Interface"];
  [[button cell] set_bordered YES];

  [button setTarget self];
  [button setAction selector (void testInterface All)];

  [v addSubview button];

  [menu_window orderFront self];

  link_hor = [s windowWithContentRect (10.0, 10.0, 1.0, 1.0)
		styleMask WINDOW_STYLE_PLAIN backing BACKING_STORE_NONRETAINED];
  [link_hor setBackgroundColor [Color black]];
  link_ver = [s windowWithContentRect (10.0, 10.0, 1.0, 1.0)
		styleMask WINDOW_STYLE_PLAIN backing BACKING_STORE_NONRETAINED];
  [link_ver setBackgroundColor [Color black]];
}

void
  testInterface Button sender
{
  class (StreamEncoder) encoding_class = [BinaryStreamEncoder self];
  MutableByteString data = [MutableByteString withCapacity 1024];
  StreamEncoder encoder = [[encoding_class alloc] initWithStream data];

  [encoder encodeRoot palette_window];

  StreamDecoder decoder = [[BinaryStreamDecoder alloc]
			    initWithStream [StringStream with data]];
  tag.Window w = [decoder decodeRoot];

  [w orderFront self];

//  [menu_window orderOut self];
//  [[Timer withInterval 5.0 invocation
//	  [Invocation of selector ("v_orderFront_r")
//		      to: menu_window with self]] schedule];
}

end;

implementation instance bit end;
