<copyright> MultiAction example and multithreaded TAG testcase.
    Written by <a href="mailto:tiggr@ics.ele.tue.nl">Pieter J. Schoenmakers</a>

    Copyright &copy; 1997 Pieter J. Schoenmakers.
    All rights reserved.

    This file is part of TAG.  TAG is distributed under the terms of the
    TAG License, a copy of which can be found in the TAG distribution; see
    the file LICENSE.

    <id>$Id: MultiAction.t,v 1.4 1998/03/23 17:46:47 tiggr Exp $</id>
    </copyright>

implementation class
MultiAction: tag, Constants, Conditions, stdio

int
  main Indexed arguments
{
  ConditionClass cc = condition;

  bind ((cc,
	 {
	   /* Report the condition and return it (to indicate we didn't
              handle it).  */
	   [[err print ("signal: ", condition)] nl];
	   condition;
	 }))
    ({
      [tag.Display set_locks_needed YES];

      /* Let TAG initialize itself, open displays, etc.  */
      Application app = [Application shared];
      Indexed active_displays = [tag.Display active_displays];
      int i, n = [active_displays length];

      for (i = 0; i < n; i++)
	{
	  tag.Display d = system_display = active_displays[i];
	  tag.Screen s = [d defaultScreen];

	  tag.Window w = [s windowWithContentRect (100.0, 100.0, 410.0, 260.0)
			    styleMask WINDOW_STYLE_TITLED
			    backing BACKING_STORE_RETAINED];
	  View content_view = [w content_view];

	  Slider slider1 = [[Slider alloc] initFrame (10.0, 10.0, 100.0, 10.0)];
	  SliderCell sc = SliderCell ([slider1 cell]);
	  [sc set_max_value 100.0];
	  [sc set_min_value 0.0];

	  Slider slider2 = [[Slider alloc] initFrame (10.0, 30.0, 100.0, 10.0)];
	  sc = SliderCell ([slider2 cell]);
	  [sc set_max_value 100.0];
	  [sc set_min_value 0.0];

	  [[w content_view] addSubview slider1];
	  [[w content_view] addSubview slider2];

	  [[self new] performInThread selector ("v_play_d_with_r")
		      with (0.5d0, slider1)];
	  [[self new] performInThread selector ("v_play_d_with_r")
		      with (0.75d0, slider2)];

	  [w orderFront self];
	}

      if (!n)
	[[err print "no active displays"] nl];
      else
	[Application run];
    });
}

end;

implementation instance
MultiAction
{
  double delta;
}

<doc> Designated initializer.  Set {delta} to 1.0.  </doc>
id
  init
{
  delta = 1.0;
  = [super (tag) init];
}

<doc> Set up the timer to repeatedly {bump}, and run the {RunLoop} of this
    thread.  </doc>
void
  play double interval
  with Slider slider
{
  Invocation i = [Invocation of selector ("v_bump_r")
			     to: self with slider];
  [[Timer withInterval interval invocation i repeats: YES] schedule];
  [[RunLoop current] run];
}

<doc> Position the slider {s} one step further, in the direction of
    {delta}.  </doc>
void
  bump Slider s
{
  SliderCell sc = [s cell];
  double min, max, v;

  min = [sc min_value];
  max = [sc max_value];
  v = [sc value];

  v += delta;
  if (v > max)
    (v, delta) = (max - delta, -delta);
  else if (v < min)
    (v, delta) = (min - delta, -delta);

  [s setDoubleValue v];
  [[[s window] display] flushOutput];
}

end;
