<copyright> PS TAG Display class.
    Written by <a href="mailto:tiggr@es.ele.tue.nl">Pieter J. Schoenmakers</a>

    Copyright &copy; 1996 Pieter J. Schoenmakers.
    All rights reserved.

    This file is part of TAG.  TAG is distributed under the terms of the
    TAG License, a copy of which can be found in the TAG distribution; see
    the file LICENSE.

    <id>$Id: Display.t,v 1.6 1998/03/16 01:33:45 tiggr Exp $</id>
    </copyright>

implementation class
Display: tag.Display, tom.Constants

<doc> Register this CIT to TAG.  </doc>
void
  load MutableArray arguments
{
  int i, n = [arguments length], in_n = n;
  String s;

  [self registerCIT self withName "ps"];
}

<doc> Return a new display that outputs the postscript code to a file with
    the {name}.  </doc>
instance (id) (display)
  openDisplay String name
{
  display = [[self alloc] initToFileNamed name];
}

end;

implementation instance
Display
{
  <doc> The file to which we output our code.  </doc>
  public File output;

  <doc> Our screen (sort of).  </doc>
  Screen screen;
}

<doc> Close this display.  </doc>
void
  close
{
  [output close];
}

Screen
  defaultScreen
{
  = screen;
}

<doc> Designated initializer.  Return {nil} if opening the display failed.
    </doc>
id
  initToFileNamed String file_name
{
  if (!catch (self)
         bind ((file-error,
		{
		  [[self error] print ("opening ", file_name, ": ", condition)];
		  [self throw NO];
		  nil;
		}))
	   ({
	     output = [File open file_name output: YES
			    flags: FILE_EXIST_TRUNCATE | FILE_NOT_EXIST_CREATE];
	     YES;
	   }))
    return nil;

  screen = [[Screen alloc] initOnDisplay self];

  // Not true, of course...
  // Thu Mar 12 22:52:59 1998, tiggr@gerbil.org
  [[output print "%!PS-Adobe-3.0 EPSF-3.0"] nl];

  = self;
}

<doc> Return the {CachedImageRep} class to be used to display on this
    {Display}.  </doc>
class (CachedImageRep)
  cachedImageRepClass
{
  = [CachedImageRep self];
}

<doc> Start clipping to the indicated rectangle on the currently
    {locked_view}.  </doc>
void
  clipToRect (float, float, float, float) (x, y, w, h)
{
  // Thu Mar 12 22:31:20 1998, tiggr@gerbil.org
}

<doc> Cancel a previous {clipToRect}.  </doc>
void
  unclip
{
  // Thu Mar 12 22:31:27 1998, tiggr@gerbil.org
}

void
  doLockView View v
{
  float x, y, w, h;

  [super doLockView v];

  (x, y, w, h) = [v frame];
  // Sun Mar 15 02:53:22 1998, tiggr@gerbil.org
  // [output print ("gsave ", x, " ", y, " translate\n")];
  [output print "gsave\n"];
}

void
  doUnlockView View v
{
  [super doUnlockView v];
  [output print "grestore\n"];
}

void
  drawLine (float, float, float, float) (x1, y1, x2, y2)
{
  [[output print ("newpath ", x1, " ", y1, " moveto ",
		  x2, " ", y2, " lineto stroke")] nl];
}

void
  fillRect (float, float, float, float) (x, y, w, h)
{
  [[[output print ("newpath ", x, " ", y, " moveto 0 ", h, " rlineto ")]
     print (w, " 0 rlineto 0 ", -h, " rlineto ", -w, " 0 rlineto fill")] nl];
}

void
  setColor Color c
{
  [c setPSColor self];
}

// CCC Why is this, unqualified, an ambiguous reference when compiling Test?
// Fri Nov 22 18:12:19 1996, tiggr@jaguar.ics.ele.tue.nl
void
  setFont tag.Font f
{
  [Font (f) setPSFont self];
}

void
  show String text
    at (float, float) (x, y)
{
  // 
}

Font
  fontNamed String name
       size int size
{
  = [self systemFont];
}

Font
  systemFont
{
  = [Font new];
}

<doc> Close the current path.  </doc>
void
  closepath
{
  [output print "closepath\n"];
}

void
  curveto (float, float, float, float, float, float)
	  (x1, y1, x2, y2, x3, y3)
{
  [output print (x1, " ", y1, " ", x2, " ", y2,
		 " ", x3, " ", y3, " curveto\n")];
}

void
  eofill
{
  [output print "eofill\n"];
}

void
  fill
{
  [output print "fill\n"];
}

void
  lineto (float, float) (x, y)
{
  [output print (x, " ", y, " lineto\n")];
}

void
  newpath
{
  [output print "newpath\n"];
}

void
  rlineto (float, float) (x, y)
{
  [output print (x, " ", y, " rlineto\n")];
}

void
  moveto (float, float) (x, y)
{
  [output print (x, " ", y, " moveto\n")];
}

void
  rmoveto (float, float) (x, y)
{
  [output print (x, " ", y, " rmoveto\n")];
}

void
  stroke
{
  [output print "stroke\n"];
}

end;
