<copyright> TAG Application class.
    Written by <a href="mailto:tiggr@es.ele.tue.nl">Pieter J. Schoenmakers</a>

    Copyright &copy; 1996, 1997 Pieter J. Schoenmakers.
    All rights reserved.

    This file is part of TAG.  TAG is distributed under the terms of the
    TAG License, a copy of which can be found in the TAG distribution; see
    the file LICENSE.

    <id>$Id: Application.t,v 1.14 1998/07/21 15:07:45 tiggr Exp $</id>
    </copyright>

/******************** ApplicationDelegate ********************/

implementation class
ApplicationDelegate: instance (All)

end;

implementation instance
ApplicationDelegate: instance (All)

<doc> Return {FALSE} if the delegate does not allow the application to
    terminate.  The default implementation returns {TRUE}.  </doc>
boolean
  applicationWillTerminate Application sender
{
  = TRUE;
}

end;

/******************** Application ********************/

implementation class
Application: Responder,
	RunLoopDelegate
{
  <doc> The shared {Application} object.  </doc>
  static instance (id) shared;
}

<doc> {run} the {shared} application object.  </doc>
void
  run
{
  if ([self shared] != nil)
    [shared run];
}

<doc> If necessary, create a new instance of the receiving class, to
    become {shared}.  Then return it.  </doc>
instance (id)
  shared
{
  if (!shared)
    shared = [self new];

  = shared;
}

end;

implementation instance
Application
{
  <doc> The key window.  </doc>
  public Window key_window;

  <doc> The main window.  </doc>
  public Window main_window;

  <doc> The main menu.  </doc>
  public Menu main_menu;

  <doc> The delegate.  </doc>
  public mutable ApplicationDelegate delegate;
}

<doc> Designated initializer.  </doc>
id (self)
  init
{
  if (![Display openDisplays])
    return nil;

  main_menu = [[Menu alloc] initWithTitle [Runtime program_name]];
}

<doc> Tell the previous key window to {makeMainWindow}, and tell the new
    key window to {becomeKeyWindow}.  </doc>
void
  set_key_window Window window
{
  if (key_window == window)
    return;

  if (key_window != nil)
    [key_window makeMainWindow];

  key_window = window;
  [key_window becomeKeyWindow];

  [[[stdio err] print ("key-window: ", key_window)] nl];
}

<doc> Tell the new main window to {becomeMainWindow}.  </doc>
void
  set_main_window Window window
{
  if (main_window == window)
    return;

  main_window = window;
  [main_window becomeMainWindow];

  [[[stdio err] print ("main-window: ", main_window)] nl];
}

<doc> Run the {RunLoop}.  </doc>
void
  run
{
  [main_menu addItem "Quit" action selector (void terminate All) target: self];
  [[main_menu window] orderFront self];

  RunLoop rl = [RunLoop current];
  [rl set_delegate self];
  [rl run];
}

<doc> Send the {action} to the {target} from the {sender}.  If the
    {target} is nil, the action is sent into the responder chain of the
    key window or, failing that, the main window.  </doc>
void
  sendAction selector action
	  to All target
	from View sender
pre
  !!action
{
  if (target != nil)
    catch (self)
      bind ((unrecognized-selector,
	     {
	       [[self error] print (cmd, ": ", condition)];
	       [self throw void];
	       condition;
	     }))
      [target perform action with sender];
  else if (key_window == nil
	   || ![key_window tryToDispatch action from sender])
    if (main_window == nil
	|| (main_window != key_window
	    && ![main_window tryToDispatch action from sender]))
      if ([self respondsTo action])
	[self perform action with sender];
      else if (delegate != nil && [delegate respondsTo action])
	[delegate perform action with sender];
}

<doc> If the {delegate} allows it, terminate the application (and exit the
    program).  </doc>
void
  terminate All sender
{
  if (!delegate || [delegate applicationWillTerminate self])
    [Runtime exit 0];
}

/******************** RunLoopDelegate ********************/
<doc> <h4>RunLoopDelegate</h4> </doc>

<doc> Give the active display a chance to flush any pending information.
    </doc>
void
  runLoopWillSelect RunLoop loop
{
  [Display flushOutput];
}

end;
