<copyright> TAG Box class.
    Written by <a href="mailto:tiggr@es.ele.tue.nl">Pieter J. Schoenmakers</a>

    Copyright &copy; 1996, 1997 Pieter J. Schoenmakers.
    All rights reserved.

    This file is part of TAG.  TAG is distributed under the terms of the
    TAG License, a copy of which can be found in the TAG distribution; see
    the file LICENSE.

    <id>$Id: Box.t,v 1.9 1997/12/08 23:57:03 tiggr Exp $</id>
    </copyright>

implementation class
Box: View

end;

implementation instance
Box
{
  <doc> The {Cell} displaying our title.  </doc>
  public Cell title_cell;

  <doc> The {View} we use for containing our subviews.  </doc>
  public View content_view;

  <doc> Horizontal space maintained around the contents.  </doc>
  float h_spacing;

  <doc> Vertical space maintained around the contents.  </doc>
  float v_spacing;

  <doc> The title position.  </doc>
  int title_position;

  <doc> The border type.  </doc>
  int border_type;

  <doc> The title rectangle.  </doc>
  float tr_x, tr_y, tr_w, tr_h;

  <doc> The contents box rectangle.  </doc>
  float cb_x, cb_y, cb_w, cb_h;
}

<doc> <h4>{View} methods</h4> </doc>

<doc> Designated initializer.  </doc>
id
  initFrame (float, float, float, float) (x, y, w, h)
{
  if (![super initFrame (x, y, w, h)])
    return nil;

  h_spacing = v_spacing = 5.0;

  title_cell = [[Cell alloc] initTextCell "Box"];
  content_view = [View new];
  [self addSubview content_view];

  [self set_title_position BOX_TITLE_TOP_AT];

  = self;
}

<doc> Redraw (part of) the box.  </doc>
void
  drawSelf (float, float, float, float) (x, y, w, h)
{
  Display d = [self currentDisplay];

  (x, y, w, h) = [d drawGroove (b_x, b_y, b_w, b_h)];
  [d setColor [Color light]];
  [d fillRect (x, y, w, h)];

  if (title_position != BOX_TITLE_NONE)
    [title_cell drawSelf (tr_x, tr_y, tr_w, tr_h) inView self];
}

<doc> Update our idea of the rectangles.  </doc>
void
  sizeTo (float, float) (w, h)
{
  [super sizeTo (w, h)];
  [self set_title_position title_position];
}

<doc> <h4>Box methods</h4> </doc>

(float, float)
  contentViewMargins
{
  = (h_spacing, v_spacing);
}

void
  set_title String title
{
  [title_cell setStringValue title];
  [self set_title_position title_position];
}

void
  set_title_position int pos
{
  float x, y, w, h, c_x, c_y, c_w, c_h;

  title_position = pos;
  needs_display = TRUE;

  (cb_x, cb_y, cb_w, cb_h) = (x, y, w, h) = [self bounds];
  (c_x, c_y, c_w, c_h) = [system_display innerGroove (x, y, w, h)];
  (tr_w, tr_h) = [title_cell cellSize];

  tr_x = (w - tr_w) / 2.0;
  if (pos == BOX_TITLE_TOP_AT)
    {
      tr_y = cb_y + cb_h - tr_h;
      cb_h -= tr_h / 2.0;
      c_y += v_spacing;
      c_h -= tr_h + v_spacing;
    }

  if (c_h < 0.0)
    c_h = 0.0;

  [content_view setFrame (c_x + h_spacing, c_y, c_w - 2.0 * h_spacing, c_h)];
}

/******************** Archiving ********************/
<doc> <h4>Archiving</h4> </doc>

void
  encodeUsingCoder Encoder coder
{
  if (![coder hasBeenCodedFor [Box self]])
    {
      [super encodeUsingCoder coder];
      [coder encode title_cell];
      [coder encode content_view];
      [coder encode h_spacing];
      [coder encode v_spacing];
      [coder encode title_position];
      [coder encode border_type];
      [coder encode tr_x];
      [coder encode tr_y];
      [coder encode tr_w];
      [coder encode tr_h];
      [coder encode cb_x];
      [coder encode cb_y];
      [coder encode cb_w];
      [coder encode cb_h];
    }
}

void
  initWithCoder Decoder coder
{
  if (![coder hasBeenCodedFor [Box self]])
    {
      [super initWithCoder coder];
      title_cell = [coder decode];
      content_view = [coder decode];
      h_spacing = [coder decode];
      v_spacing = [coder decode];
      title_position = [coder decode];
      border_type = [coder decode];
      tr_x = [coder decode];
      tr_y = [coder decode];
      tr_w = [coder decode];
      tr_h = [coder decode];
      cb_x = [coder decode];
      cb_y = [coder decode];
      cb_w = [coder decode];
      cb_h = [coder decode];
    }
}

end;
