<copyright> TAG Browser and BrowserDelegate class.
    Written by <a href="mailto:tiggr@es.ele.tue.nl">Pieter J. Schoenmakers</a>

    Copyright &copy; 1998 Pieter J. Schoenmakers.
    All rights reserved.

    This file is part of TAG.  TAG is distributed under the terms of the
    TAG License, a copy of which can be found in the TAG distribution; see
    the file LICENSE.

    <id>$Id: Browser.t,v 1.10 1998/07/21 15:07:47 tiggr Exp $</id>
    </copyright>

implementation class
BrowserDelegate: instance (All)

end;

implementation instance
BrowserDelegate: instance (All)

<doc> Either fill the {column} of the {browser} in the {matrix} and return
    {TRUE}, or return {FALSE}.  The default implementation returns false.
    </doc>
boolean
	      browser Browser browser
  createRowsForColumn int column
	     inMatrix Matrix matrix
{
}

<doc> Return {TRUE} if the {column} of the {browser} is valid.  Queried in
    response to a {validateVisibleColumns}.  The default implementation
    returns {TRUE}.  </doc>
boolean
	browser Browser browser
  isColumnValid int column
{
  = TRUE;
}

<doc> Return the number of rows in the {column} of the {browser}.  This
    method is invoked is {browser createRowsForColumn inMatrix} returned
    {FALSE} (and did not fill the matrix.  The default implementation
    returns 0.  </doc>
int
	       browser Browser browser
  numberOfRowsInColumn int column
{
}

<doc> Return the title to display above the {column}.  The default
    implementation returns {nil}, which is suitable only for untitled
    browsers.  </doc>
String
	browser Browser browser
  titleOfColumn int column
{
}

<doc> Be informed that the {browser} will display the {cell} at {row} in
    the {column}.  If the delegate did not preload all cells in the
    column, now is the time to load this {cell}.  </doc>
void
	  browser Browser browser
  willDisplayCell BrowserCell cell
	       at (int, int) (row, column)
{
}

end;

/******************** ********************/

implementation class
Browser: Control, MatrixDelegate
{
  <doc> Our {cell_class} should resemble a {BrowserCell}.  </doc>
  redeclare class (BrowserCell) cell_class;

  <doc> The default distance between all elements of a Browser.  </doc>
  const BOUNDARIES = 2.0;
}

<doc> Set the {cell_class} to {BrowserCell}.  </doc>
void
  load Array arguments
{
  cell_class = [BrowserCell self];
}

end;

implementation instance
Browser
{
  <doc> The titles of the loaded columns.  If this is {nil}, no titles are
      displayed.  </doc>
  MutableIndexed titles;

  <doc> The matrices (instances of the {Matrix} class) in the loaded
      columns.  </doc>
  MutableIndexed matrices;

  <doc> The scrollviews used to show the visible columns.  </doc>
  MutableIndexed scroll_views;

  <doc> The horizontal scroller employed to change the visible columns.
      </doc>
  Scroller scroller;

  <doc> The number of visible columns.  </doc>
  int num_visible_columns;

  <doc> The index of the first visible column.  </doc>
  int first_column;

  <doc> The index of the last column.  The matrix in this column is the
      one (most often) manipulated by the delegate.  </doc>
  int last_column;

  <doc> The minimum column width.  When resizing, the number of columns is
      changed such that each of the columns is not narrower than this.  If
      this is 0, the number of visible columns will not change.  </doc>
  public mutable float min_column_width;

  <doc> Iff {TRUE} the {last_column} has been loaded.  </doc>
  boolean fully_loaded;

  <doc> The delegate of this browser.  </doc>
  public mutable BrowserDelegate delegate;
}

<doc> Designnated initializer.  </doc>
id
  initFrame (float, float, float, float) (x, y, w, h)
{
  = [super initFrame (x, y, w, h)];

  // Drop the cell created by Control.
  // Fri Jan 30 13:06:07 1998, tiggr@akebono.ics.ele.tue.nl
  cell = nil;

  scroller = [[Scroller alloc] initFrame (0.0, 0.0, w, SCROLLER_WIDTH)];
  [scroller set_auto_size VIEW_SIZE_WIDTH | VIEW_SIZE_MAXY];
  [self addSubview scroller];

  scroll_views = [MutableObjectArray new];
  matrices = [MutableObjectArray new];
  num_visible_columns = 2;
  [self reconfigure];
}

void
  display (float, float, float, float) (x, y, w, h)
{
  if (!fully_loaded)
    {
      if ([matrices length] != last_column + 1)
	{
	  Matrix m = [[Matrix alloc]
		       initFrame (0.0, 0.0, 0.0, 0.0) mode MATRIX_MODE_HIGHLIGHT
		       prototype [[BrowserCell alloc]
				   initTextCell "browser-cell"]
		       dimensions (0, 1)];
	  [m set_delegate self];
	  [m setTarget self];
	  [m setAction selector (void noteChange All)];
	  [matrices add m];
	}

      Matrix m = matrices[last_column];
      ScrollView sv = scroll_views[last_column - first_column];
      [sv set_document_view m];
      if (delegate != nil)
	if (![delegate browser self createRowsForColumn last_column
		       inMatrix m])
	  [self unimplemented cmd message: "non-lazy delegate"];
      [m sizeToCells];
      [self reconfigure];
      fully_loaded = YES;
    }

  [super display (x, y, w, h)];
}

<doc> Do nothing.  </doc>
void
  drawSelf (float, float, float, float) (x, y, w, h)
{
}

void
  noteChange All sender
{
  [self unimplemented cmd];
}

<doc> Handle a change in the number of visible columns.  </doc>
protected void
  reconfigure
{
  int i, nsv = [scroll_views length];

  [[err print ("Browser: reconfigure: ", nsv, " -> ", num_visible_columns)] nl];

  while (nsv < num_visible_columns)
    {
      ScrollView sv = [[ScrollView alloc] initFrame (0.0, 0.0, 400.0, 400.0)];
      ClipView cp = [[ClipView alloc] initFrame (0.0, 0.0, 400.0, 400.0)];

      [sv set_content_view cp];
      [cp set_background_color [Color light]];
      [sv set_auto_size VIEW_SIZE_WIDTH | VIEW_SIZE_HEIGHT];
      [sv set_vertical_scroller_required YES];
      [[sv scroller_v] set_continuous YES];
      [self addSubview sv];

      scroll_views[nsv++] = sv;
    }

  // CCC Tue Jan 27 21:35:29 1998, tiggr@tnt.ics.ele.tue.nl
  int iwidth = int (b_w);
  int icsp = int (BOUNDARIES);
  int ticsp = icsp * (num_visible_columns - 1);
  iwidth = (iwidth - ticsp) / num_visible_columns;
  float width = float (iwidth - 1);

  for (i = 0; i < nsv; i++)
    {
      ScrollView sv = scroll_views[i];
      float offset = !scroller ? 0.0 : BOUNDARIES + SCROLLER_WIDTH;

      [sv moveTo (float (i) * (width + float (icsp)) + float (i), offset)];
      [sv sizeTo (width, b_h - offset)];
    }


  int nm = [matrices length];
  for (i = first_column; i < nm && i < first_column + num_visible_columns; i++)
    {
      Matrix m = matrices[i];
      float mw, mh, cw, ch;
      (, , mw, mh) = [m frame];
      (, , cw, ch) = [[m superview] frame];

      if (mw != cw)
	[m sizeTo (cw, mh)];
    }
}

// Duplication :-(
// Fri Jan 30 14:45:34 1998, tiggr@akebono.ics.ele.tue.nl
<doc> Rearrange the columns, possibly removing or creating some.  </doc>
void
  setFrame (float, float, float, float) (x, y, w, h)
{
  [super setFrame (x, y, w, h)];

  if (min_column_width > 0.0)
    {
      num_visible_columns = int (b_w / min_column_width);
      if (num_visible_columns < 1)
	num_visible_columns = 1;
    }

  [self reconfigure];
}

<doc> Rearrange the columns, possibly removing or creating some.  </doc>
void
  sizeTo (float, float) (x, y)
{
  [super sizeTo (x, y)];

  if (min_column_width > 0.0)
    {
      num_visible_columns = int (b_w / min_column_width);
      if (num_visible_columns < 1)
	num_visible_columns = 1;
    }

  [self reconfigure];
}

void
  superviewSizeChanged (float, float) (so_w, so_h)
{
  [super superviewSizeChanged (so_w, so_h)];
  [self reconfigure];
}

/******************** MatrixDelegate ********************/
<doc><h4>MatrixDelegate</h4></doc>

<doc> Ensure the {cell} has been loaded.  </doc>
void
	   matrix Matrix matrix
  willDisplayCell BrowserCell cell
	       at (int, int) (row, column)
{
  int i, n = [matrices length];

  for (i = n - 1; i >= 0; i--)
    if (matrix == matrices[i])
      {
	if (![cell loaded] && delegate != nil)
	  [delegate browser self willDisplayCell cell at (row, i)];
	break;
      }
}

/******************** Archiving ********************/
<doc><h4>Archiving</h4></doc>

void
  encodeUsingCoder Encoder coder
{
  if (![coder hasBeenCodedFor [Browser self]])
    {
      [super encodeUsingCoder coder];
      [self unimplemented cmd];
    }
}

void
  initWithCoder Decoder coder
{
  if (![coder hasBeenCodedFor [Browser self]])
    {
      [super initWithCoder coder];
      [self unimplemented cmd];
    }
}

end;
