<copyright> TAG BrowserCell class.
    Written by <a href="mailto:tiggr@es.ele.tue.nl">Pieter J. Schoenmakers</a>

    Copyright &copy; 1998 Pieter J. Schoenmakers.
    All rights reserved.

    This file is part of TAG.  TAG is distributed under the terms of the
    TAG License, a copy of which can be found in the TAG distribution; see
    the file LICENSE.

    <id>$Id: BrowserCell.t,v 1.2 1998/02/03 22:24:12 tiggr Exp $</id>
    </copyright>

implementation class
BrowserCell: Cell
{
  <doc> The browser right-arrow images.  </doc>
  static Image arrow_right, arrow_right_h;
}

<doc> Load the images.  </doc>
private void
  load_images
{
  arrow_right = [Image named "browse-right"];
  arrow_right_h = [Image named "browse-right-h"];
}

end;

implementation instance
BrowserCell
{
  <doc> Iff {TRUE}, this cell has been loaded.  </doc>
  public mutable boolean loaded;

  <doc> Iff {TRUE}, this is a leaf cell.  </doc>
  public mutable boolean is_leaf;
}

<doc> Just-to-be-sure initializer.  </doc>
id
  init
{
  = [self initTextCell "Button"];
}

<doc> Designated initializer.  </doc>
id
  initTextCell String value
{
  if (!arrow_right)
    [isa load_images];

  = [super initTextCell value];
}

<doc> Return the minimum acceptable size for this cell.  This attempts to
    positio the image next to the text.  </doc>
(float, float)
  cellSize
{
  float x, y, ix, iy;

  (x, y) = [super cellSize];
  (ix, iy) = [arrow_right size];

  x += ix + 4.0;
  if (y < iy + 4.0)
    y = iy + 4.0;

  = (x, y);
}

void
  drawInside (float, float, float, float) (x, y, w, h)
      inView View view
{
  boolean high = highlighted ^ state;

  float iw, ih;
  (iw, ih) = [arrow_right size];

  Display d = [view currentDisplay];
  [d setColor high ? [Color white] : [Color light]];
  [d fillRect (x + w - iw - 4.0, y, 4.0 + iw, h)];

  [super drawInside (x, y, w - 4.0 - iw, h) inView view];

  if (!is_leaf)
    {
      Image im = high ? arrow_right_h : arrow_right;
      [im compositeToPoint (x + w - iw - 2.0, y + (h - ih) / 2.0)
	  operation COMPOSITE_COPY];
    }
}

/******************** Archiving ********************/
<doc><h4>Archiving</h4></doc>

void
  encodeUsingCoder Encoder coder
{
  if (![coder hasBeenCodedFor [BrowserCell self]])
    {
      [super encodeUsingCoder coder];
      [coder encode loaded];
      [coder encode is_leaf];
    }
}

void
  initWithCoder Decoder coder
{
  if (![coder hasBeenCodedFor [BrowserCell self]])
    {
      [super initWithCoder coder];
      loaded = [coder decode];
      is_leaf = [coder decode];
    }
}

end;
