<copyright> TAG ButtonCell class.
    Written by <a href="mailto:tiggr@es.ele.tue.nl">Pieter J. Schoenmakers</a>

    Copyright &copy; 1996-1998 Pieter J. Schoenmakers.
    All rights reserved.

    This file is part of TAG.  TAG is distributed under the terms of the
    TAG License, a copy of which can be found in the TAG distribution; see
    the file LICENSE.

    <id>$Id: ButtonCell.t,v 1.13 1998/02/24 22:52:21 tiggr Exp $</id>
    </copyright>

<doc> A ButtonCell is an ActionCell.

    For the display of the text or alternate text, the ButtonCell depends
    on functionality inherited from the {Cell}.  </doc>
implementation class
ButtonCell: ActionCell
{
  <doc> The horizontal space between the image and its surroundings.
      </doc>
  const IMG_HOR_SPACE = 5.0;

  <doc> Definition of how the reflection constants are stuffed in the
      {reflection} ivar.  </doc>
  const STATE_SHIFT = 0;
  const HIGHLIGHT_SHIFT = 4;
}

end;

implementation instance
ButtonCell
{
  <doc> The image we display in the normal state.  </doc>
  Image image;

  <doc> The alternative image employed by this ButtonCell.  </doc>
  public mutable Image alternate_image;

  <doc> The alternative text.  </doc>
  public mutable String alternate_title;

  <doc> Stuffed information on how to highlight and how to reflect state.
      </doc>
  byte reflection;

  <doc> Where the image is displayed.  </doc>
  byte image_position;
}

<doc> Designated initializer.  </doc>
id
  init
{
  = [super initTextCell "Button"];
}

/********** **********/

int
  highlightsBy
{
  = (int (reflection) >> HIGHLIGHT_SHIFT) & CELL_MASK_ALL;
}


<doc> Overridden to return the {image}.  </doc>
Image
  image
{
  = image;
}

<doc> Overridden to manipulate the {image}.  </doc>
void
  set_image Image im
{
  image = im;
}

<doc> Set the kind of button, by setting the {reflection} according to the
    {button_type}.  Directly modifies {reflection}; does not call
    {setHighlightsBy} or {setShowsStateBy}.  </doc>
void
  setButtonType int button_type
{
  if (button_type == BUTTON_MOMENTARY_LIGHT)
    reflection = byte ((CELL_MASK_CHANGE_GRAY | CELL_MASK_CHANGE_BACKGROUND)
		       << HIGHLIGHT_SHIFT);
  else if (button_type == BUTTON_MOMENTARY_PUSH)
    reflection = byte ((CELL_MASK_CHANGE_GRAY | CELL_MASK_CHANGE_BACKGROUND
			| CELL_MASK_PUSH) << HIGHLIGHT_SHIFT);
  else if (button_type == BUTTON_MOMENTARY_CHANGE)
    reflection = byte (CELL_MASK_CONTENTS << HIGHLIGHT_SHIFT);
  else if (button_type == BUTTON_PUSH_ON_PUSH_OFF)
    reflection = byte (((CELL_MASK_CHANGE_GRAY | CELL_MASK_CHANGE_BACKGROUND
			 | CELL_MASK_PUSH) << HIGHLIGHT_SHIFT)
		       | ((CELL_MASK_CHANGE_GRAY | CELL_MASK_CHANGE_BACKGROUND
			   | CELL_MASK_PUSH) << STATE_SHIFT));
  else if (button_type == BUTTON_ON_OFF)
    reflection = byte (((CELL_MASK_CHANGE_GRAY | CELL_MASK_CHANGE_BACKGROUND)
			<< HIGHLIGHT_SHIFT)
		       | ((CELL_MASK_CHANGE_GRAY | CELL_MASK_CHANGE_BACKGROUND)
			  << STATE_SHIFT));
  else if (button_type == BUTTON_TOGGLE
	    || button_type == BUTTON_SWITCH
	    || button_type == BUTTON_RADIO)
    {
      reflection = byte ((CELL_MASK_CONTENTS << HIGHLIGHT_SHIFT)
			 | (CELL_MASK_CONTENTS << STATE_SHIFT));
      if (button_type == BUTTON_SWITCH)
	{
	  // Mon Jan 26 21:10:10 1998, tiggr@tnt.ics.ele.tue.nl
	  // [self set_image xxx];
	  // [self set_alternate_image xxx];
	}
      if (button_type == BUTTON_RADIO)
	{
	  // Mon Jan 26 21:10:13 1998, tiggr@tnt.ics.ele.tue.nl
	  // [self set_image xxx];
	  // [self set_alternate_image xxx];
	}
    }
}

void
  setHighlightsBy int h
{
  reflection = byte (reflection & ~(CELL_MASK_ALL << HIGHLIGHT_SHIFT)
		     | ((h & CELL_MASK_ALL) << HIGHLIGHT_SHIFT));
}

void
  setImagePosition int position
pre
  position >= BUTTONCELL_IMAGE_NONE && position <= BUTTONCELL_IMAGE_OVERLAP
{
  image_position = byte (position);
}

void
  setShowsStateBy int h
{
  reflection = byte (reflection & ~(CELL_MASK_ALL << STATE_SHIFT)
		     | ((h & CELL_MASK_ALL) << STATE_SHIFT));
}

int
  showsStateBy
{
  = (int (reflection) >> STATE_SHIFT) & CELL_MASK_ALL;
}

/********** **********/

<doc> Return the natural size for this cell.  </doc>
(float, float) (res_x, res_y)
  cellSize
{
  float iw, ih, mw, mh;

  if (image_position != BUTTONCELL_IMAGE_NONE)
    {
      if (image != nil)
	(mw, mh) = [image size];
      if (alternate_image != nil)
	{
	  (iw, ih) = [alternate_image size];
	  if (mw < iw)
	    mw = iw;
	  if (mh < ih)
	    mh = ih;
	}
    }

  if (image_position == BUTTONCELL_IMAGE_ONLY)
    = (mw, mh);
  else
    {
      float x, y;

      (x, y) = [super cellSize];
      if (image_position == BUTTONCELL_IMAGE_LEFT
	  || image_position == BUTTONCELL_IMAGE_RIGHT)
	{
	  if (mw != 0.0)
	    x += mw + IMG_HOR_SPACE;
	  if (mh > y)
	    y = mh;
	}
      else if (image_position == BUTTONCELL_IMAGE_BELOW
	  || image_position == BUTTONCELL_IMAGE_ABOVE)
	{
	  if (mw != 0.0)
	    y += mh + IMG_HOR_SPACE;
	  if (mw > x)
	    x = mw;
	}
      else if (image_position == BUTTONCELL_IMAGE_OVERLAP)
	{
	  if (mw > x)
	    x = mw;
	  if (mh > y)
	    y = mh;
	}

      = (x, y);
    }

  // Hack!
  // Tue Feb 24 23:37:23 1998, tiggr@gerbil.org
  if (bordered)
    {
      res_x += 4.0;
      res_y += 4.0;
    }
}

void
  drawInside (float, float, float, float) (x, y, w, h)
      inView View view
{
  Display d = [view currentDisplay];

  int high = state ? [self showsStateBy] : 0;
  if (highlighted)
    high = high ^ [self highlightsBy];

  Image im = image;
  if (alternate_image != nil && high & CELL_MASK_CONTENTS != 0)
    im = alternate_image;

  if (bordered)
    (x, y, w, h) = [d innerButton (x, y, w, h)
		      highlighted: high & CELL_MASK_PUSH != 0];

  [d setColor (high & (CELL_MASK_CHANGE_GRAY | CELL_MASK_CHANGE_BACKGROUND)
	       != 0) ? [Color white] : [Color light]];
  [d fillRect (x, y, w, h)];

  if (high & CELL_MASK_PUSH != 0)
    y -= 1.0;

  if (im != nil)
    {
      float ix, iy, iw, ih, ixm = IMG_HOR_SPACE;

      (iw, ih) = [im size];

      if (image_position == BUTTONCELL_IMAGE_ONLY)
	{
	  [im compositeToPoint (x + (w - iw) / 2.0, y + (h - ih) / 2.0)
	      operation COMPOSITE_COPY];
	  return;
	}

      // Forgot some...
      // Tue Feb 24 22:39:20 1998, tiggr@gerbil.org
      [im compositeToPoint (x + w - ixm - iw, y + (h - ih) / 2.0)
	  operation COMPOSITE_COPY];

      w -= iw + 2.0 * ixm;
    }

  Font f = [d systemFont];
  String what = !contents ? "Cell" : contents;
  if (alternate_title != nil && high & CELL_MASK_CONTENTS != 0)
    what = alternate_title;

  float sw = [f widthOf what];

  [d setFont f];
  [d setColor enabled ? [Color black] : [Color dark]];
      
  x = x + (w - sw) / 2.0;
  y = y + (h - [f height] - [f descent]) / 2.0;

  [d show what at (x, y)];
}

void
  drawSelf (float, float, float, float) (x, y, w, h)
    inView View view
{
  int high = state ? [self showsStateBy] : 0;
  if (highlighted)
    high = high ^ [self highlightsBy];

  if (bordered)
    [[view currentDisplay] drawButton (x, y, w, h)
			   highlighted: high & CELL_MASK_PUSH != 0];

  [self drawInside (x, y, w, h) inView view];
}

/******************** Archiving ********************/
<doc> <h4>Archiving</h4> </doc>

void
  encodeUsingCoder Encoder coder
{
  if (![coder hasBeenCodedFor [ButtonCell self]])
    {
      [super encodeUsingCoder coder];
      [coder encode image];
      [coder encode alternate_image];
      [coder encode alternate_title];
      [coder encode reflection];
    }
}

void
  initWithCoder Decoder coder
{
  if (![coder hasBeenCodedFor [ButtonCell self]])
    {
      [super initWithCoder coder];
      image = [coder decode];
      alternate_image = [coder decode];
      alternate_title = [coder decode];
      reflection = [coder decode];
    }
}

end;
