<copyright> TAG Cell class.
    Written by <a href="mailto:tiggr@es.ele.tue.nl">Pieter J. Schoenmakers</a>

    Copyright &copy; 1996-1998 Pieter J. Schoenmakers.
    All rights reserved.

    This file is part of TAG.  TAG is distributed under the terms of the
    TAG License, a copy of which can be found in the TAG distribution; see
    the file LICENSE.

    <id>$Id: Cell.t,v 1.23 1998/02/03 22:24:13 tiggr Exp $</id>
    </copyright>

implementation class
Cell: tag, Copying
{
  // Not yet...
  // Mon Jan 26 21:14:44 1998, tiggr@tnt.ics.ele.tue.nl
  <doc> Meaning of the bits in the {flags} of a {Cell}.  </doc>
  const CF_STATE	= 0x01;
  const CF_HIGHLIGHTED	= 0x02;
  const CF_DISABLED	= 0x04;
  const CF_TYPE		= 0x08;
  const CF_SELECTABLE	= 0x10;
  const CF_EDITABLE	= 0x20;
  const CF_ALIGN_LEFT	= 0x40;
  const CF_ALIGN_RIGHT	= 0x80;
  const CF_ALIGN_FULL	= 0xc0;
  const CF_BORDER_MASK	= 0;
}

end;

implementation instance
Cell
{
  <doc> The textual contents, or the contents' textual representation.
      </doc>
  String contents;

  <doc> The image if we hold an image; the {Font} if we hold text.  </doc>
  Any info;

  <doc> This cell's state.  </doc>
  public mutable boolean state;

  <doc> This cell's highlighting.  </doc>
  public mutable boolean highlighted;

  <doc> Iff {TRUE}, we're an image cell.  Otherwise, we're a text cell.
      </doc>
  boolean is_image;

  <doc> Iff {TRUE}, we're bordered.  </doc>
  public mutable boolean bordered;

  <doc> Iff {TRUE}, we're bezeled.  </doc>
  public mutable boolean bezeled;

  <doc> Iff {TRUE}, we're enabled.  Otherwise we ignore mouse buttons.  </doc>
  public mutable boolean enabled;
}

/******************** manipulation ********************/

id
  initTextCell String title
pre
  system_display != nil
{
  (info, contents) = (Any ([system_display systemFont]), title);
  enabled = YES;

  = self;
}

<doc> Return the minimum acceptable size for this cell.  </doc>
(float, float)
  cellSize
{
  float x, y;

  if (info != nil)
    if (is_image)
      (x, y) = [Image (info) size];
    else
      (x, y) = ([Font (info) widthOf contents], [Font (info) height]);

  // Account for any border type.
  // Wed Oct  9 16:57:22 1996, tiggr@jaguar.ics.ele.tue.nl
  (x, y) = (x + 4.0, y + 4.0);

  = (x, y);
}

Image
  image
{
  = is_image ? info : nil;
}

void
  set_image Image image
pre
  image != nil
{
  (is_image, info) = (YES, Any (image));
  if (image != nil)
    contents = [image name];
}

/******************** Values ********************/
<doc> <h4>Values</h4> </doc>

void
  setDoubleValue double value
{
  if (!is_image)
    {
      MutableByteString s = [MutableByteString new];
      [s print value];
      contents = s;
    }
}

double
  doubleValue
{
  if (!is_image)
    = [contents doubleValue];
}

void
  setStringValue String value
{
  if (is_image)
    {
      is_image = NO;
      info = Any ([system_display systemFont]);
    }
  contents = value;
}

String
  stringValue
{
  if (!is_image)
    = contents;
}

/******************** Event handling ********************/
<doc> <h4>Event handling</h4> </doc>

boolean
  mouseDown MouseEvent e
     inRect (float, float, float, float) (x, y, w, h)
     ofView View v
{
  if (enabled)
    {
      Display d = [v currentDisplay];
      [d set_track_rectangle (x, y, w, h)];

      [self set_highlighted TRUE];
      [v display (x, y, w, h)];
    }
  = enabled;
}

void
  mouseDragged MouseEvent e
	    in View v
{
  Display d = [v currentDisplay];
  float x, y;

  (x, y) = [v convertPoint [e location] fromView nil];

  boolean next = [v mouse (x, y) inRect [d track_rectangle]];
  if (next != highlighted)
    {
      [self set_highlighted next];
      [v display [d track_rectangle]];
    }
}

void
  mouseUp MouseEvent e
       in Control v
{
  if (highlighted)
    {
      highlighted = NO;
      state = !state;
      [v display [[v currentDisplay] track_rectangle]];
    }
}

/******************** drawing ********************/
<doc> <h4>Drawing</h4> </doc>

void
  drawInside (float, float, float, float) (x, y, w, h)
      inView View view
{
  Display d = [view currentDisplay];
  boolean high = highlighted ^ state;

  if (bordered)
    (x, y, w, h) = (x + 1.0, y + 1.0, w - 2.0, h + 2.0);
  else if (bezeled)
    (x, y, w, h) = [d innerWhiteBezel (x, y, w, h)];

  [d setColor high || bezeled ? [Color white] : [Color light]];
  [d fillRect (x, y, w, h)];

  if (is_image)
    {
      Image image = info;
      float iw, ih;

      if (image != nil)
	{
	  (iw, ih) = [image size];

	  [image compositeToPoint (x + (w - iw) / 2.0, y + (h - ih) / 2.0)
		 operation COMPOSITE_COPY];
	}
    }
  else
    {
      Font f = [d systemFont];
      String what = !contents ? "Cell" : contents;

      float sw = [f widthOf what];

      [d setFont f];
      [d setColor enabled ? [Color black] : [Color dark]];

      x = x + (w - sw) / 2.0;
      y = y + (h - [f height] - [f descent]) / 2.0;
      if (state && bordered)
	// CCC y--
	// Fri Oct  4 11:54:57 1996, tiggr@jaguar.ics.ele.tue.nl
	y -= 1.0;

      [d show what at (x, y)];
    }
}

void
  drawSelf (float, float, float, float) (x, y, w, h)
    inView View view
{
  if (bordered || bezeled)
    {
      Display d = [view currentDisplay];

      if (bordered)
	{
	  [d setColor [Color black]];
	  [d drawRect (x, y, w, h)];
	}
      else
	[d drawWhiteBezel (x, y, w, h)];
    }

  [self drawInside (x, y, w, h) inView view];
}

/******************** Archiving ********************/
<doc> <h4>Archiving</h4> </doc>

void
  encodeUsingCoder Encoder coder
{
  if (![coder hasBeenCodedFor [Cell self]])
    {
      [super (tag) encodeUsingCoder coder];
      [coder encode contents];
      [coder encode info];
      [coder encode state];
      [coder encode highlighted];
      [coder encode is_image];
      [coder encode bordered];
      [coder encode bezeled];
      [coder encode enabled];
    }
}

void
  initWithCoder Decoder coder
{
  if (![coder hasBeenCodedFor [Cell self]])
    {
      [super (tag) initWithCoder coder];
      contents = [coder decode];
      info = [coder decode];
      state = [coder decode];
      highlighted = [coder decode];
      is_image = [coder decode];
      bordered = [coder decode];
      bezeled = [coder decode];
      enabled = [coder decode];
    }
}

end;
