<copyright> TAG ClipView class.
    Written by <a href="mailto:tiggr@es.ele.tue.nl">Pieter J. Schoenmakers</a>

    Copyright &copy; 1997 Pieter J. Schoenmakers.
    All rights reserved.

    This file is part of TAG.  TAG is distributed under the terms of the
    TAG License, a copy of which can be found in the TAG distribution; see
    the file LICENSE.

    <id>$Id: ClipView.t,v 1.11 1998/03/02 17:22:49 tiggr Exp $</id>
    </copyright>

implementation class
ClipView: View

end;

implementation instance
ClipView
{
  <doc> The {View} being displayed within this {ClipView}.  </doc>
  public View document_view;

  <doc> The background color displayed by this {ClipView}.  This is the
      color that shows up next to the document view if that is smaller
      than the (visible area of) the {ClipView}.  </doc>
  public mutable Color background_color;
}

<doc> In addition to what {super} does, set the background color.  </doc>
id
  initFrame (float, float, float, float) (x, y, w, h)
{
  background_color = [Color light];
  = [super initFrame (x, y, w, h)];
}

<doc> Draw the background.  </doc>
void
  drawSelf (float, float, float, float) (x, y, w, h)
{
  Display d = [self currentDisplay];

  [d setColor background_color];
  [d fillRect (x, y, w, h)];
}

/******************** Hierarchy ********************/
<doc><h4>Hierarchy</h4></doc>

<doc> Set the {document_view}.  </doc>
void
  set_document_view View view
{
  if (document_view == view)
    return;

  if (document_view != nil)
    [document_view removeFromSuperview];

  document_view = view;

  // Needed?
  // Mon Mar  2 17:59:01 1998, tiggr@gerbil.org
  // (, , fw, fh) = [document_view frame];
  // [document_view setFrame (0.0, 0.0, fw, fh)];

  [self addSubview document_view];
}

/******************** Positioning ********************/
<doc><h4>Positioning</h4></doc>

void
  setHorizontalPosition double hpos
pre
  hpos >= 0.0 && hpos <= 1.0
{
  if (!document_view)
    return;

  float fx, fy, fw;

  (fx, fy, fw, ) = [document_view frame];

  /* Moving only makes sense if the DOCUMENT_VIEW is wider than this
     clipview.  */
  if (fw > b_w)
    {
      float new_fx = float (int (-(fw - b_w) * hpos));
      [document_view moveTo (new_fx, fy)];

      /* The distance over which we move.  DISTANCE > 0 means we're
	 increasing the FX (from a negative value), thus scrolling to the
	 left, and copying the left part to the right.  */
      float distance = new_fx - fx;

      float crx, cry, crw, crh, dx, dy, rdx, rdy, rdw, rdh;
      (rdx, rdy, rdw, rdh) = (crx, cry, crw, crh) = (b_x, b_y, b_w, b_h);
      (dx, dy) = (b_x, b_y);

      if (distance > 0.0)
	{
	  dx += distance;
	  crw -= distance;
	  rdw = distance;
	}
      else
	{
	  crx -= distance;
	  crw += distance;
	  rdx += rdw + distance;
	  rdw = -distance;
	}

      [self lockFocus];
      if (crw > 0.0)
	[[self currentDisplay] copyRect (crx, cry, crw, crh) to (dx, dy)];
      [self unlockFocus];

      (rdx, rdy, rdw, rdh) = [tag clipRect (rdx, rdy, rdw, rdh)
				  to (b_x, b_y, b_w, b_h)];
      [self display (rdx, rdy, rdw, rdh)];
    }
}

void
  setVerticalPosition double vpos
pre
  vpos >= 0.0 && vpos <= 1.0
{
  if (!document_view)
    return;

  float fx, fy, fh;

  (fx, fy, , fh) = [document_view frame];

  /* Moving only makes sense if the DOCUMENT_VIEW is higher than this
     clipview.  */
  if (fh > b_h)
    {
      float new_fy = float (int (-(fh - b_h) * vpos));
      [document_view moveTo (fx, new_fy)];

      /* The distance over which we move.  DISTANCE > 0 means we're
	 increasing the FY (from a negative value), thus scrolling down,
	 and copying the lower part higher.  */
      float distance = new_fy - fy;

      float crx, cry, crw, crh, dx, dy, rdx, rdy, rdw, rdh;
      (rdx, rdy, rdw, rdh) = (crx, cry, crw, crh) = (b_x, b_y, b_w, b_h);
      (dx, dy) = (b_x, b_y);

      if (distance > 0.0)
	{
	  dy += distance;
	  crh -= distance;
	  rdh = distance;
	}
      else
	{
	  cry -= distance;
	  crh += distance;
	  rdy += rdh + distance;
	  rdh = -distance;
	}

      Display d = [self currentDisplay];

      [self lockFocus];
      if (crh > 0.0)
	[d copyRect (crx, cry, crw, crh) to (dx, dy)];
      [self unlockFocus];

      (rdx, rdy, rdw, rdh) = [tag clipRect (rdx, rdy, rdw, rdh)
				  to (b_x, b_y, b_w, b_h)];
      [self display (rdx, rdy, rdw, rdh)];
    }
}

/******************** Archiving ********************/
<doc><h4>Archiving</h4></doc>

void
  encodeUsingCoder Encoder coder
{
  if (![coder hasBeenCodedFor [ClipView self]])
    {
      [super encodeUsingCoder coder];
      [coder encode document_view];
      [coder encode background_color];
    }
}

void
  initWithCoder Decoder coder
{
  if (![coder hasBeenCodedFor [ClipView self]])
    {
      [super initWithCoder coder];
      document_view = [coder decode];
      background_color = [coder decode];
    }
}

end;
