<copyright> TAG Color class.
    Written by <a href="mailto:tiggr@es.ele.tue.nl">Pieter J. Schoenmakers</a>

    Copyright &copy; 1996, 1997 Pieter J. Schoenmakers.
    All rights reserved.

    This file is part of TAG.  TAG is distributed under the terms of the
    TAG License, a copy of which can be found in the TAG distribution; see
    the file LICENSE.

    <id>$Id: Color.t,v 1.7 1997/07/22 23:10:57 tiggr Exp $</id>
    </copyright>

implementation class
Color: State,
	tag
{
  <doc> The {WhiteColor} black.  </doc>
  public static instance (id) black;

  <doc> The {WhiteColor} white.  </doc>
  public static instance (id) white;

  <doc> The {WhiteColor} light gray (2/3 white).  </doc>
  public static instance (id) light;

  <doc> The {WhiteColor} dark gray (1/3 white).  </doc>
  public static instance (id) dark;
}

<doc> Allocate the well known colors.  </doc>
redefine void
  load Array arguments
{
  black = [WhiteColor with 0.0];
  dark = [WhiteColor with 1.0 / 3.0];
  light = [WhiteColor with 2.0 / 3.0];
  white = [WhiteColor with 1.0];
}

end;

implementation instance
Color

deferred int
  colorSpace;

deferred boolean
  equalRGBColor RGBColor other;

deferred boolean
  equalWhiteColor WhiteColor other;

end;

/******************** WhiteColor ********************/

implementation class
WhiteColor: Color

<doc> (For the time being,) the designated alloctator.  </doc>
instance (id)
  with float white
{
  = [[self alloc] init white];
}

end;

implementation instance
WhiteColor
{
  <doc> The gray level of this color.  </doc>
  float gray;
}

<doc> Return {COLOR_SPACE_DEVICE_WHITE}.  </doc>
int
  colorSpace
{
  = COLOR_SPACE_DEVICE_WHITE;
}

<doc> Designated initializer.  </doc>
id
  init float w
{
  gray = w;

  = self;
}

float
  whiteComponents
{
  = gray;
}

/******************** equality ********************/
<doc> <h4>Equality</h4>  </doc>

redefine boolean
  equal Color other
{
  = self == other || [other equalWhiteColor self];
}

boolean
  equalRGBColor RGBColor other
{
  = NO;
}

boolean
  equalWhiteColor WhiteColor other
{
  = gray == [other whiteComponents];
}

redefine int
  hash
{
  = int (65535.0d * gray);
}

/******************** Archiving ********************/
<doc><h4>Archiving</h4></doc>

// Should do color uniquing...
// Tue Jul 22 23:05:24 1997, tiggr@tricky.es.ele.tue.nl

void
  encodeUsingCoder Encoder coder
{
  if (![coder hasBeenCodedFor [WhiteColor self]])
    {
      [super encodeUsingCoder coder];
      [coder encode gray];
    }
}

void
  initWithCoder Decoder coder
{
  if (![coder hasBeenCodedFor [WhiteColor self]])
    {
      [super initWithCoder coder];
      gray = [coder decode];
    }
}

end;

/******************** RGBColor ********************/

implementation class
RGBColor: Color

<doc> (For the time being,) the designated alloctator.  </doc>
instance (id)
  with (float, float, float) (r, g, b)
{
  = [[self alloc] init (r, g, b)];
}

end;

implementation instance
RGBColor
{
  <doc> Values of the red, green and blue components.  </doc>
  public float red, green, blue;
}

<doc> Return {COLOR_SPACE_DEVICE_RGB}.  </doc>
int
  colorSpace
{
  = COLOR_SPACE_DEVICE_RGB;
}

<doc> Designated initializer.  </doc>
private id
  init (float, float, float) (r, g, b)
{
  (red, green, blue) = (r, g, b);

  = self;
}

<doc> Retrieve the RGB components.  </doc>
(float, float, float)
  rgbComponents
{
  = (red, green, blue);
}

/******************** equality ********************/
<doc> <h4>Equality</h4>  </doc>

redefine boolean
  equal Color other
{
  = self == other || [other equalRGBColor self];
}

boolean
  equalRGBColor RGBColor other
{
  float r, g, b;

  (r, g, b) = [other rgbComponents];

  = r == red && g == green && b == blue;
}

boolean
  equalWhiteColor WhiteColor other
{
  = NO;
}

redefine int
  hash
{
  = (int (65535.0d * red) + int (65535.0d * green) << 1
     + int (65535.0d * blue) << 2);
}

/******************** Archiving ********************/
<doc><h4>Archiving</h4></doc>

// Should do color uniquing...
// Tue Jul 22 23:05:24 1997, tiggr@tricky.es.ele.tue.nl

void
  encodeUsingCoder Encoder coder
{
  if (![coder hasBeenCodedFor [RGBColor self]])
    {
      [super encodeUsingCoder coder];
      [coder encode red];
      [coder encode green];
      [coder encode blue];
    }
}

void
  initWithCoder Decoder coder
{
  if (![coder hasBeenCodedFor [RGBColor self]])
    {
      [super initWithCoder coder];
      red = [coder decode];
      green = [coder decode];
      blue = [coder decode];
    }
}

end;
