<copyright> TAG Control class.
    Written by <a href="mailto:tiggr@es.ele.tue.nl">Pieter J. Schoenmakers</a>

    Copyright &copy; 1996, 1997 Pieter J. Schoenmakers.
    All rights reserved.

    This file is part of TAG.  TAG is distributed under the terms of the
    TAG License, a copy of which can be found in the TAG distribution; see
    the file LICENSE.

    <id>$Id: Control.t,v 1.19 1998/02/24 17:22:57 tiggr Exp $</id>
    </copyright>

implementation class
Control: View, Values
{
  <doc> The default {Cell} class to be used for instances of receiving
      class.  </doc>
  public mutable class (Cell) cell_class;
}

end;

implementation instance
Control
{
  <doc> The {Cell} we're employing.  </doc>
  public mutable ActionCell cell;

  <doc> Iff {TRUE}, we're tracking the {cell} and pass it {mouseDrag} and
      {mouseUp} events.  </doc>
  boolean tracking;
}

<doc> Let the {cell} draw itself in the receiving {Control}.  </doc>
void
  drawSelf (float, float, float, float) (x, y, w, h)
{
  [cell drawSelf (b_x, b_y, b_w, b_h) inView self];
}

id
  initFrame (float, float, float, float) (x, y, w, h)
{
  float dx, dy;

  if (cell_class != nil)
    {
      [self set_cell ActionCell ([cell_class new])];
      (dx, dy) = [cell cellSize];

      if (w < dx)
	w = dx;
      if (h < dy)
	h = dy;
    }

  = [super initFrame (x, y, w, h)];
}

/******************** Values ********************/
<doc> <h4>Values</h4> </doc>

void
  setDoubleValue double value
{
  [cell setDoubleValue value];
  // needs_display = TRUE;
  [self display];
}

double
  doubleValue
{
  = [cell doubleValue];
}

void
  setStringValue String value
{
  [cell setStringValue value];
  [self display];
}

String
  stringValue
{
  = [cell stringValue];
}

/******************** target/action ********************/
<doc> <h4>Target/Action</h4> </doc>

selector
  action
{
  = [cell action];
}

Any
  target
{
  = [cell target];
}

void
  setTarget All t
{
  [cell setTarget t];
}

void
  setAction selector a
{
  [cell setAction a];
}

void
  sendAction selector action
	  to All target
{
  [[Application shared] sendAction action to target from self];
}

/******************** Event handling ********************/
<doc> <h4>Event handling</h4> </doc>

<doc> Pass the mouse-down {event} to our {cell}.  </doc>
void
  mouseDown MouseEvent event
{
  if (cell != nil)
    tracking = [cell mouseDown event inRect [self bounds] ofView self];
}

<doc> Pass the mouse-dragged {event} to our {cell}.  </doc>
void
  mouseDragged MouseEvent event
{
  if (cell != nil && tracking)
    [cell mouseDragged event in self];
}

<doc> Pass the mouse-up {event} to our {cell}.  </doc>
void
  mouseUp MouseEvent event
{
  if (cell != nil && tracking)
    [cell mouseUp event in self];
}

/******************** Archiving ********************/
<doc> <h4>Archiving</h4> </doc>

void
  encodeUsingCoder Encoder coder
{
  if (![coder hasBeenCodedFor [Control self]])
    {
      [super encodeUsingCoder coder];
      [coder encode cell];
    }
}

void
  initWithCoder Decoder coder
{
  if (![coder hasBeenCodedFor [Control self]])
    {
      [super initWithCoder coder];
      cell = [coder decode];
    }
}

end;
