<copyright> TAG DeviceDescription class.
    Written by <a href="mailto:tiggr@es.ele.tue.nl">Pieter J. Schoenmakers</a>

    Copyright &copy; 1996, 1997 Pieter J. Schoenmakers.
    All rights reserved.

    This file is part of TAG.  TAG is distributed under the terms of the
    TAG License, a copy of which can be found in the TAG distribution; see
    the file LICENSE.

    <id>$Id: DeviceDescription.t,v 1.4 1998/03/03 00:17:24 tiggr Exp $</id>
    </copyright>

implementation class
DeviceDescription: Device
{
  <doc> The set containing all unique (shared) device descriptions.  </doc>
  static MutableEqSet devices;
}

<doc> Return the {DeviceDescription} with the indicated characteristics,
    creating it if it does not yet exist.  </doc>
instance (id)
	     on int device_type
     resolution (float, float) (xres, yres)
	  frame (float, float, float, float) (fx, fy, width, height)
     colorSpace int color_space
  bitsPerSample int bps
{
  DeviceDescription d;

  if (!devices)
    devices = [MutableEqSet new];
  else
    {
      Enumerator e = [devices enumerator];
      boolean b;

      /* Uniquing is done by manual comparison since {DeviceDescriptions}
         consider themselves equal only if they're identical.  */
      while ({(b, d) = [e next]; b;})
	{
	  float x, y, w, h;

	  (x, y) = [d resolution];
	  if (x != xres || y != yres)
	    continue;

	  (x, y, w, h) = [d frame];
	  if (x != fx || y != fy || w != width || h != height)
	    continue;

	  if (device_type == [d device_type] && bps == [d bitsPerSample]
	      && color_space == [d colorSpace])
	    return d;
	}
    }

  d = [[self alloc] initOn device_type
		    resolution (xres, yres) frame (fx, fy, width, height)
		    colorSpace color_space bitsPerSample bps];
  [devices add d];
  = d;
}

end;

implementation instance
DeviceDescription
{
  <doc> The device type.  </doc>
  public int device_type;

  <doc> The significant number of bits per sample, rounded up.  </doc>
  int bits_per_sample;

  <doc> The device's intrinsic color space.  </doc>
  int color_space;

  <doc> Resolution in pixels per inch.  </doc>
  float res_x, res_y;

  <doc> The frame.  </doc>
  float f_x, f_y, f_w, f_h;
}

<doc> Have the {Device} tell whether it is equal to this
    {DeviceDescription}.  </doc>
redefine boolean
  equal Device d
{
  = [d equalDeviceDescription self];
}

<doc> Return {YES} iff {d} is identical to {self}.  </doc>
redefine boolean
  equalDeviceDescription DeviceDescription d
{
  = self == d;
}

<doc> Designated initializer.  </doc>
id
	 initOn int dt
     resolution (float, float) (x, y)
	  frame (float, float, float, float) (fx, fy, fw, fh)
     colorSpace int cs
  bitsPerSample int bps
{
  (device_type, bits_per_sample, color_space) = (dt, bps, cs);
  (f_x, f_y, f_w, f_h) = (fx, fy, fw, fh);
  (res_x, res_y) = (x, y);

  = self;
}

<doc> Return the device's bits per sample.  </doc>
int
  bitsPerSample
{
  = bits_per_sample;
}

<doc> Return the device's color space.  </doc>
int
  colorSpace
{
  = color_space;
}

(float, float, float, float)
  frame
{
  = (f_x, f_y, f_w, f_h);
}

<doc> Return {YES} if the device is on a screen or intended to be
    displayed on one.  </doc>
redefine boolean
  isScreen
{
  = device_type == DEVICE_TYPE_SCREEN;
}

<doc> Return {YES} if the device is a printer.  </doc>
redefine boolean
  isPrinter
{
  = device_type == DEVICE_TYPE_PRINTER;
}

<doc> Return the resulution of the device, in pixels per inch.  </doc>
(float, float)
  resolution
{
  = (res_x, res_y);
}

end;
