<copyright> TAG Display class.
    Written by <a href="mailto:tiggr@es.ele.tue.nl">Pieter J. Schoenmakers</a>

    Copyright &copy; 1996, 1997 Pieter J. Schoenmakers.
    All rights reserved.

    This file is part of TAG.  TAG is distributed under the terms of the
    TAG License, a copy of which can be found in the TAG distribution; see
    the file LICENSE.

    <id>$Id: Display.t,v 1.31 1998/07/21 15:07:49 tiggr Exp $</id>
    </copyright>

implementation class
Display: State,
	tag
{
  <doc> The registered Concrete Implementations of TAG.  </doc>
  static MutableMapped registered_cits;

  <doc> The currently active displays.  </doc>
  static MutableIndexed active_displays;

  <doc> Displays specified on the command line or environment.  Each
      element is of the form `cit:display'.  </doc>
  static MutableIndexed cmd_displays;

  <doc> The stack of locked focusses in this thread.  </doc>
  local static MutableArray focus_stack;

  // CCC This should be an `instance (id)', but that types wrongly in the
  // context of the Display instances.
  // Wed Oct  2 15:09:15 1996, tiggr@jaguar.ics.ele.tue.nl
  <doc> The currently active display in this thread.  </doc>
  local public static Display current;

  <doc> Iff {TRUE}, each display needs a lock (because we're running
      multi-threaded).  </doc>
  static boolean locks_needed;

  <doc> Mapping from tag class objects to CIT class objects which are to
      be preferred for this display class.  For example, the X cit uses
      this to prefer {x.Window} objects to (the uselessly abstract)
      {tag.Window} objects.  </doc>
  MutableMapped preferred_classes;
}

<doc> Output information on the {:} options understood by the
    {tag.Display} class.  </doc>
OutputStream (s)
  help OutputStream s
  done MutableKeyed done
{
  class (State) cls = [Display self];

  if (!done[cls])
    {
      [done add cls];

      s = [[s print "tag.Display
  :display c:d        	open the display <d> on the cit <c>"] nl];
    }
}

<doc> Get the `:display' arguments from the command line.   </doc>
void
  load MutableArray arguments
{
  int i, n = [arguments length];
  String s;

  for (i = 0; i < n;)
    {
      ByteString a = arguments[i];

      if ([":display" equal a])
	{
	  String s = arguments[i + 1];
	  [arguments removeElements (i, 2)];
	  n -= 2;
	  [self registerDisplayRequest s];
	}
      else
	i++;
    }
}

<doc> Manipulate {locks_needed} according to {yes_or_no}, and inform the
    active displays.  </doc>
void
  set_locks_needed boolean yes_or_no
{
  if (locks_needed != yes_or_no)
    {
      locks_needed = yes_or_no;
      if (active_displays != nil)
	[active_displays makeElementsPerform
		     [tom.Invocation of selector (void set_locks_needed boolean)
			 with locks_needed]];
    }
}

/******************** Display management ********************/
<doc> <h4>Display management</h4> </doc>

<doc> Return the active displays.  </doc>
Indexed
  active_displays
{
  = active_displays;
}

<doc> Return <em>the</em> default display.  </doc>
instance (id)
  default
{
  = active_displays[0];
}

<doc> Have the active displays {flushOutput}.  </doc>
void
  flushOutput
{
  int i, m, n = [active_displays length];

  for (i = 0; i < n;)
    {
      Display d = active_displays[i];

      [d flushOutput];

      /* The display could have discovered itself closed during the flush,
         and thus have been removed from the active displays.  */
      m = [active_displays length];
      if (m == n)
	i++;
      else
	n = m;
    }
}

<doc> Attempt to open the {display} on the receiving CIT.  </doc>
deferred instance (id)
  openDisplay String display;

<doc> Open the displays as indicated by the user on the command-line, or
    whatever other way of indication s/he has.  Return {TRUE} upon
    success, i.e. when at least one display was openend.  </doc>
boolean
  openDisplays
{
  if (cmd_displays != nil)
    {
      int i, n = [cmd_displays length];

      for (i = 0; i < n; i++)
	[self openQualifiedDisplay cmd_displays[i]];
    }

  if (!active_displays)
    [[[self error] print "Unable to open any display"] nl];
  else
    system_display = [Display default];

  = active_displays != nil;
}

instance (id) (display)
  openQualifiedDisplay String name
{
  Array a = [name componentsSeparatedBy ':' limit: 2];
  Display d;

  if ([a length] != 2)
    [[[self error] print ("bad qualified display `", name, '\'')] nl];
  else
    {
      String cit_name = a[0];
      class (Display) dc = registered_cits[cit_name];

      if (!dc)
	[[[self error] print ("no cit `", cit_name, '\'')] nl];
      else
	display = [dc openDisplay a[1]];
    }
}

<doc> Register the (newly openened) {display}.  </doc>
void
  registerDisplay Display display
{
  if (!active_displays)
    active_displays = [MutableObjectArray new];
  [active_displays add display];
}

<doc> Unregister the {display}.  </doc>
void
  unregisterDisplay Display display
{
  int i = [active_displays indexOfIdentical display];

  if (i >= 0)
    [active_displays removeElementAt i];
  else
    [[[self error] print (cmd, ": could not find display ", display)] nl];

  if (![active_displays length])
    {
      [[[self error] print "all displays closed"] nl];
      [Runtime exit 0];
    }
}

<doc> According to a commandline option, the {display} is to be opened;
    register it in the {cmd_displays}.  </doc>
void
  registerDisplayRequest String name
{
  if (!cmd_displays)
    cmd_displays = [MutableObjectArray new];
  [cmd_displays add name];
}

/******************** CIT management ********************/
<doc> <h4>CIT management</h4>

    A `CIT' is a `Concrete Implementation of TAG'.  Every CIT is
    represented by its {Display} class.  </doc>

<doc> Register the {Display} class {cit} under the {name}.  </doc>
void
  registerCIT class (Display) cit
     withName String name
{
  if (!registered_cits)
    registered_cits = [MutableDictionary new];
  registered_cits[name] = cit;
}

<doc> Return the preferred CIT class for the TAG class c.  If there is no
    preferred class, the argument class is returned.  </doc>
class (State)
  preferred_class class (State) c
{
  class (State) d = preferred_classes[c];

  = !d ? c : d;
}

end;

/******************************  ******************************/

implementation instance
Display
{
  <doc> The (recursive) lock we use to protect us from being busy in more
      than one thread.  </doc>
  Lock lock;

  <doc> The currently locked {view}.  </doc>
  public View locked_view;

  <doc> A rectangle for use by handling mouse events.  This is, for
      example, set by a {Cell} while handling a {mouseDown}, to register
      its borders, for use in {mouseDragged} and {mouseUp}.  </doc>
  float track_x, track_y, track_w, track_h;

  <doc> The events used for dispatching.  Every type of {Event} has its
      own cache copy here, which is reinitialized when needed.  When you
      want to keep an event around for a while, you'll have to make a
      copy.

      The {event} is the `current' event.  </doc>
  public Event event;

  <doc> The event used for dispatching mouse events.  </doc>
  MouseEvent e_mouse;
}

<doc> Designated initializer.  </doc>
id
  init
{
  [self set_locks_needed locks_needed];
  = [super (tag) init];
}

<doc> Return the {CachedImageRep} class to be used to display on the
    rceiving kind of {Display}.  </doc>
deferred class (CachedImageRep)
  cachedImageRepClass;

<doc> Close this display.  </doc>
deferred void
  close;

// This is a convenience method, really...
// Thu Mar 12 22:54:01 1998, tiggr@gerbil.org
<doc> If this display is associated with a file, return it.  </doc>
deferred ByteStream
  output;

<doc> Flush any pending output.  The default implementation does nothing.
    Returns {self}.  </doc>
id
  flushOutput
{
  = self;
}

<doc> Return a description of the currently active output device, i.e. the
    device on which the {locked_view} would display.  </doc>
Device
  deviceDescription
{
  = [[locked_view window] deviceDescription];
}

<doc> Do the actual work of locking the {view}, now we have the lock.  </doc>
void
  doLockView View view
{
  if (!focus_stack)
    focus_stack = [MutableObjectArray new];

  [focus_stack add view];
  locked_view = view;
  current = self;
}

<doc> Do the actual work of unlocking the {view}, before releasing the
    lock.  </doc>
void
  doUnlockView View view
{
  int l = [focus_stack length];
  id v = focus_stack[l - 1];

  if (v != view)
    [[Condition for self class program-condition
		message "wrong view unlocked"] signal];

  locked_view = l > 1 ? focus_stack[l - 2] : nil;
  [focus_stack truncate l - 1];
  current = !locked_view ? nil : [[locked_view screen] display];
}

<doc> Lock this display for the {view} to this thread.  </doc>
void
  lockView View view
{
  if (lock != nil)
    [lock lock];

  [self doLockView view];
}

<doc> Set or clear the {lock}.  </doc>
void
  set_locks_needed boolean yes_or_no
{
  if (yes_or_no)
    {
      if (!lock)
	lock = [RecursiveLock new];
    }
  else if (lock != nil)
    {
      // Clear anyone waiting on it.
      // Sun Dec 21 00:57:36 1997, tiggr@tricky.ics.ele.tue.nl
      lock = nil;
    }
}

<doc> Set the track rectangle.  </doc>
void
  set_track_rectangle (float, float, float, float) (x, y, w, h)
{
  (track_x, track_y, track_w, track_h) = (x, y, w, h);
}

<doc> Return the track rectangle.  </doc>
(float, float, float, float)
  track_rectangle
{
  = (track_x, track_y, track_w, track_h);
}

<doc> Unlock the view.  </doc>
void
  unlockView View view
{
  [self doUnlockView view];

  if (lock != nil)
    [lock unlock];
}

/******************** screens ********************/

<doc> Return the screen of the current {locked_view}.  </doc>
Screen
  locked_screen
{
  = [locked_view screen];
}

<doc> Return the screens (subclass of {Screen}) attached to this display.
    </doc>
deferred Indexed
  screens;

<doc> Return this display's default screen.  </doc>
Screen
  defaultScreen
{
  = [self screens][0];
}

/******************** blitting ********************/
<doc><h4>Blitting</h4></doc>

<doc> Copy the indicated rectange in the currently {locked_view} to the
    destination point.  </doc>
deferred void
  copyRect (float, float, float, float) (rx, ry, rw, rh)
	to (float, float) (x, y);

/******************** clipping ********************/
<doc><h4>Clipping</h4></doc>

<doc> Start clipping to the indicated rectangle on the currently
    {locked_view}.  </doc>
deferred void
  clipToRect (float, float, float, float) (x, y, w, h);

// Nested clipping...
// Sat Dec 13 01:23:23 1997, tiggr@tricky.ics.ele.tue.nl
<doc> Cancel a previous {clipToRect}.  </doc>
deferred void
  unclip;

/******************** paths ********************/
<doc><h4>Paths</h4></doc>

<doc> These are all terribly familiar...  </doc>

<doc> Close the current path.  </doc>
deferred void
  closepath;

deferred (float, float)
  currentpoint;

deferred void
  curveto (float, float, float, float, float, float) (x1, y1, x2, y2, x3, y3);

deferred void
  eofill;

deferred void
  fill;

deferred void
  flattenpath;

deferred void
  lineto (float, float) (x, y);

deferred void
  newpath;

void
  rlineto (float, float) (x, y)
{
  float x1, y1;
  (x1, y1) = [self currentpoint];
  [self lineto (x + x1, y + y1)];
}

deferred void
  moveto (float, float) (x, y);

deferred (float, float, float, float) (x1, y1, x2, y2)
  pathbbox;

void
  rmoveto (float, float) (x, y)
{
  float x1, y1;
  (x1, y1) = [self currentpoint];
  [self moveto (x + x1, y + y1)];
}

deferred void
  stroke;

/******************** drawing ********************/
<doc><h4>Drawing</h4></doc>

deferred void
  beep;

deferred void
  drawLine (float, float, float, float) (x1, y1, x2, y2);

deferred void
  fillRect (float, float, float, float) (x, y, w, h);

deferred void
  setColor Color c;

deferred void
  setFont Font f;

deferred void
  show String text
    at (float, float) (x, y);

void
	drawRect (float, float, float, float) (x, y, w, h)
  withBorderType int border_type
{
  if (border_type == BORDER_NONE)
    void;
  else if (border_type == BORDER_LINE)
    {
      [self setColor [Color black]];
      [self drawRect (x, y, w, h)];
    }
  else if (border_type == BORDER_BEZEL)
    [self drawGrayBezel (x, y, w, h)];
  else
    [self drawGroove (x, y, w, h)];
}

(float, float, float, float)
      innerRect (float, float, float, float) (x, y, w, h)
  forBorderType int border_type
{
  if (border_type == BORDER_NONE)
    = (x, y, w, h);
  else if (border_type == BORDER_LINE)
    = (x + 1.0, y + 1.0, w - 2.0, h - 2.0);
  else
    = (x + 2.0, y + 2.0, w - 4.0, h - 4.0);
}

<doc> Draw a rectangle (in the current color, etc) within the rectangle
    denoted by {(x, y, w, h)}.  Like all rectangular routines, this draws
    inside the given rectangle.  </doc>
void
  drawRect (float, float, float, float) (x, y, w, h)
{
  [self drawLine (x, y, x, y + h - 1.0)];
  [self drawLine (x, y + h - 1.0, x + w - 1.0, y + h - 1.0)];
  [self drawLine (x + w - 1.0, y + h - 1.0, x + w - 1.0, y)];
  [self drawLine (x + w - 1.0, y, x, y)];
}

<doc> Draw a button outline, possibly highlighted, and return its inner
    rectangle.  </doc>
(float, float, float, float)
    drawButton (float, float, float, float) (x, y, w, h)
  highlighted: boolean high = NO
	 fill: boolean fill = NO
{
  if (high)
    {
      [self setColor [Color white]];
      [self drawLine (x + w - 1.0, y + h - 1.0, x + w - 1.0, y)];
      [self drawLine (x + w - 1.0, y, x, y)];

      [self setColor [Color dark]];
      [self drawLine (x + 1.0, y + 1.0, x, y + 1.0)];
      [self drawLine (x, y + 1.0, x, y + h - 1.0)];
      [self drawLine (x, y + h - 1.0, x + w - 2.0, y + h - 1.0)];
      [self drawLine (x + w - 2.0, y + h - 1.0, x + w - 2.0, y + h - 2.0)];

      [self setColor [Color black]];
      [self drawLine (x + 1.0, y + 2.0, x + 1.0, y + h - 2.0)];
      [self drawLine (x + 1.0, y + h - 2.0, x + w - 3.0, y + h - 2.0)];

      (x, y, w, h) = (x + 1.0, y + 1.0, w - 3.0, h - 3.0);
    }
  else
    {
      [self setColor [Color white]];
      [self drawLine (x, y, x, y + h - 1.0)];
      [self drawLine (x, y + h - 1.0, x + w - 1.0, y + h - 1.0)];

      [self setColor [Color black]];
      [self drawLine (x + w - 1.0, y + h - 1.0, x + w - 1.0, y)];
      [self drawLine (x + w - 1.0, y, x, y)];

      [self setColor [Color dark]];
      [self drawLine (x + 1.0, y + 1.0, x + w - 2.0, y + 1.0)];
      [self drawLine (x + w - 2.0, y + 1.0, x + w - 2.0, y + h - 2.0)];

      (x, y, w, h) = (x + 1.0, y + 2.0, w - 3.0, h - 3.0);
    }

  if (fill)
    {
      [self setColor [Color light]];
      [self fillRect (x, y, w, h)];
    }

  = (x, y, w, h);
}

<doc> Return the inner rectangle if a, possibly highlighted, button would
    have been drawn in the given rectangle.  </doc>
(float, float, float, float)
   innerButton (float, float, float, float) (x, y, w, h)
  highlighted: boolean high = NO
{
  = (high ? (x + 2.0, y + 1.0, w - 3.0, h - 3.0)
     : (x + 1.0, y + 2.0, w - 3.0, h - 3.0));
}

<doc> Draw a gray bezel.  That is a highlighted button.  </doc>
(float, float, float, float)
  drawGrayBezel (float, float, float, float) (x, y, w, h)
	  fill: boolean fill = NO
{
  = [self drawButton (x, y, w, h) highlighted: YES fill: fill];
}

<doc> Return the inner rectangle if a gray bezel would have been drawn in
    the given rectangle.  </doc>
(float, float, float, float)
   innerGrayBezel (float, float, float, float) (x, y, w, h)
{
  = [self innerButton (x, y, w, h) highlighted: YES];
}

<doc> Draw a white bezel.  </doc>
(float, float, float, float)
  drawWhiteBezel (float, float, float, float) (x, y, w, h)
	   fill: boolean fill = NO
{
  [self setColor [Color light]];
  [self drawLine (x + 1.0, y + 1.0, x + w - 2.0, y + 1.0)];
  [self drawLine (x + w - 2.0, y + 1.0, x + w - 2.0, y + h - 2.0)];

  [self setColor [Color dark]];
  [self drawLine (x, y + 1.0, x, y + h - 1.0)];
  [self drawLine (x, y + h - 1.0, x + w - 1.0, y + h - 1.0)];
  [self drawLine (x + 1.0, y + 2.0, x + 1.0, y + h - 2.0)];
  [self drawLine (x + 1.0, y + h - 2.0, x + w - 2.0, y + h - 2.0)];

  [self setColor [Color white]];
  [self drawLine (x, y, x + w - 1.0, y)];
  [self drawLine (x + w - 1.0, y, x + w - 1.0, y + h - 2.0)];

  (x, y, w, h) = (x + 2.0, y + 2.0, w - 4.0, h - 4.0);
  if (fill)
    [self fillRect (x, y, w, h)];

  = (x, y, w, h);
}

<doc> Return the inner rectangle if a white bezel would have been drawn in
    the given rectangle.  </doc>
(float, float, float, float)
   innerWhiteBezel (float, float, float, float) (x, y, w, h)
{
  = (x + 2.0, y + 2.0, w - 4.0, h - 4.0);
}

<doc> Draw a rectangle, top-left being black, bottom-right being white.
    </doc>
(float, float, float, float)
  drawBWRect (float, float, float, float) (x, y, w, h)
{
  [self setColor [Color white]];
  [self drawLine (x + 1.0, y, x + w - 1.0, y)];
  [self drawLine (x + w - 1.0, y, x + w - 1.0, y + h - 2.0)];

  [self setColor [Color black]];
  [self drawLine (x + w - 1.0, y + h - 1.0, x, y + h - 1.0)];
  [self drawLine (x, y + h - 1.0, x, y)];

  = (x + 1.0, y + 1.0, w - 2.0, h - 2.0);
}

(float, float, float, float)
  innerBWRect (float, float, float, float) (x, y, w, h)
{
  = (x + 1.0, y + 1.0, w - 2.0, h - 2.0);
}

<doc> Draw a groove.  </doc>
(float, float, float, float)
  drawGroove (float, float, float, float) (x, y, w, h)
{
  [self setColor [Color dark]];
  [self drawLine (x, y, x, y + h - 1.0)];
  [self drawLine (x, y + h - 1.0, x + w - 1.0, y + h - 1.0)];
  [self drawLine (x + 2.0, y + 1.0, x + w - 2.0, y + 1.0)];
  [self drawLine (x + w - 2.0, y + 1.0, x + w - 2.0, y + h - 3.0)];

  [self setColor [Color white]];
  [self drawLine (x + 1.0, y, x + 1.0, y + h - 2.0)];
  [self drawLine (x + 1.0, y + h - 2.0, x + w - 1.0, y + h - 2.0)];
  [self drawLine (x + w - 1.0, y + h - 2.0, x + w - 1.0, y)];
  [self drawLine (x + w - 1.0, y, x + 1.0, y)];

  = (x + 2.0, y + 2.0, w - 4.0, h - 4.0);
}

<doc> Return the inner rectangle of a groove.  </doc>
(float, float, float, float)
  innerGroove (float, float, float, float) (x, y, w, h)
{
  = (x + 2.0, y + 2.0, w - 4.0, h - 4.0);
}

/******************** Fonts ********************/
<doc> <h4>Fonts</h4> </doc>

<doc> Return the font with with {name} and {size}.  It is possible that
    the {size} is ignored.  </doc>
deferred Font
  fontNamed String name
       size int size;

<doc> Return the system font at the normal size.  </doc>
deferred Font (result)
  systemFont
post
  result != nil;

end;
