<copyright> TAG Image class.
    Written by <a href="mailto:tiggr@es.ele.tue.nl">Pieter J. Schoenmakers</a>

    Copyright &copy; 1996, 1997 Pieter J. Schoenmakers.
    All rights reserved.

    This file is part of TAG.  TAG is distributed under the terms of the
    TAG License, a copy of which can be found in the TAG distribution; see
    the file LICENSE.

    <id>$Id: Image.t,v 1.11 1998/02/24 22:52:26 tiggr Exp $</id>
    </copyright>

<doc> The image handling class.  </doc>
implementation class
Image: State,
	tag
{
  <doc> Images on their name.  </doc>
  static MutableDictionary image_by_name;

  <doc> The directories in which we find images.  </doc>
  static MutableIndexed image_path;

  // Only one, for now...
  // Sun Feb  1 20:03:55 1998, tiggr@tnt.ics.ele.tue.nl
  <doc> The image loader class.  </doc>
  static public mutable ImageLoader image_loader;
}

<doc> Output information on the {Image} arguments.  </doc>
OutputStream (s)
  help OutputStream s
  done MutableKeyed done
{
  class (State) cls = [Image self];

  if (!done[cls])
    {
      [done add cls];

      s = [[s print ("tag.Image
  :image-dir <name>     add the image directory <name> to the image path.
                        default: ", IMAGE_DIR)] nl];
    }
}

<doc> Set the {image_directory}, which defaults to {IMAGE_DIR}.  </doc>
void
  load MutableArray arguments
{
  int i, n = [arguments length];

  image_path = [MutableObjectArray with IMAGE_DIR];
  image_by_name = [MutableDictionary new];

  while (i < n)
    if ([arguments[i] equal ":image-dir"])
      {
	[image_path add [File filename-as-directory arguments[i + 1]]];
	[arguments removeElements (i, 2)];
	n -= 2;
      }
    else
      i++;
}

<doc> Create and return a new image with the name {n} from the
    representative {rep}.  </doc>
Image (image)
  createImage ImageRep rep
     withName String n
{
  image = [[self alloc] initWithSize [rep size]];
  [image addRepresentation rep];
  [image setName n];
}

<doc> Return the {image_path}.  </doc>
Indexed
  image_path
{
  = image_path;
}

<doc> Return the {Image} know by the {name}.  </doc>
instance (id) (image)
  named String name
{
  image = image_by_name[name];
  if (!image && image_loader != nil)
    image = [image_loader loadImageNamed name];
}

<doc> <h4>Managing {ImageRep} Subclasses</h4> </doc>

<doc> Returns an array of file types recognized by the {Image} without
    filtering. This list comes from all registered {ImageRep}s.  </doc>
Indexed
  imageUnfilteredFileTypes
{
  [self unimplemented cmd];
}

<doc> Returns an array of pasteboard types recognized by the {Image}.
    </doc>
Indexed
  imageUnfilteredPasteboardTypes
{
  [self unimplemented cmd];
}

<doc> <h4>Testing Image Data Sources</h4> </doc>

<doc> Returns YES if the receiver can create a representation from
    pasteboard; otherwise, returns NO.  </doc>
boolean
  canInitWithPasteboard Pasteboard pasteboard
{
  [self unimplemented cmd];
}

<doc> Returns an array of supported image data file types.  </doc>
Indexed
  imageFileTypes
{
  [self unimplemented cmd];
}

<doc> Returns an array of supported pasteboard types.  </doc>
Indexed
  imagePasteboardTypes
{
  [self unimplemented cmd];
}

end;

/****************************** ******************************/

implementation instance
Image
{
  <doc> The name.  </doc>
  public String name;

  <doc> The size of this image.  </doc>
  public float width, height;

  <doc> The representations of this image.  </doc>
  MutableIndexed representations;

  <doc> The cached representations for this image.  </doc>
  MutableIndexed cached;
}

<doc> <h4>Initializing a New Image Instance</h4> </doc>

<doc> Initializes the new {Image} from the data in filename. The file is
    assumed to persist and may be reread later if the {Image} is resized
    or otherwise modified.  </doc>
id
  initByReferencingFile String filename
{
  [self unimplemented cmd];
}

<doc> Initializes the new {Image} from the data in filename.  </doc>
id
  initWithContentsOfFile String filename
{
  [self unimplemented cmd];
}

<doc> Initializes the new {Image} from data.  </doc>
id
  initWithData Data data
{
  [self unimplemented cmd];
}

<doc> Initializes the new {Image} with the data in pasteboard.  </doc>
id
  initWithPasteboard Pasteboard pasteboard
{
  [self unimplemented cmd];
}

<doc> Initialize the new {Image} to the specified size.  </doc>
id
  initWithSize (float, float) (x, y)
{
  representations = [MutableObjectArray new];
  (width, height) = (x, y);

  = self;
}

<doc> <h4>Setting the Size of the Image</h4> </doc>

<doc> Sets the size of the image to aSize in base coordinates.  </doc>
void
  setSize (float, float) (w, h)
{
  (width, height) = (w, h);
}

<doc> Returns the size of the image.  </doc>
(float, float)
  size
{
  = (width, height);
}

<doc> <h4>Referring to Images by Name</h4> </doc>

<doc> Set the {name} of the receiving image.  Return {NO} if name is
    already in use; otherwise, return {YES}.  </doc>
boolean
  setName String n
{
  name = n;

  Image im = image_by_name[name];
  if (!im)
    {
      image_by_name[name] = self;
      = YES;
    }
}

<doc> <h4>Specifying the Image</h4> </doc>

<doc> Add the {rep} to the receiver's list of representations.  </doc>
void
  addRepresentation ImageRep rep
{
  [representations add rep];
}

<doc> Add the imageReps from {reps} to the receiver's list of
    representations.  </doc>
void
  addRepresentations Indexed reps
{
  [representations addElementsFrom reps];
}

<doc> Prepares for drawing in the best representation.  </doc>
void
  lockFocus
{
  [self unimplemented cmd];
}

<doc> Prepares for drawing in imageRep.  </doc>
void
  lockFocusOnRepresentation ImageRep imageRep
{
  [self unimplemented cmd];
}

<doc> Balances a previous lockFocus or lockFocusOnRepresentation:.  </doc>
void
  unlockFocus
{
  [self unimplemented cmd];
}

<doc> <h4>Using the Image</h4> </doc>

<doc> Composites the image to aPoint using the operation op.  </doc>
void
  compositeToPoint (float, float) (x, y)
	 operation int op
{
  Display d = [Display current];
  Screen s = [d locked_screen];
  CachedImageRep ir;

  if (!cached)
    cached = [MutableObjectArray new];
  else
    {
      int i, n = [cached length];

      for (i = 0; i < n; i++)
	{
	  CachedImageRep cir = cached[i];
	  if ([cir screen] == s)
	    {
	      ir = cir;
	      break;
	    }
	}
    }

  if (!ir)
    {
      // Pick the best representation...
      // Tue Oct  8 17:39:26 1996, tiggr@jaguar.ics.ele.tue.nl
      BitmapImageRep bm = representations[0];

      class (CachedImageRep) rc = [d cachedImageRepClass];

      ir = [rc withBitmapImageRep bm onScreen s];
      [cached add ir];
    }

  [ir drawAtPoint (x, y)];
}

<doc> Composites the aRect portion of the image to aPoint using the
    operation op.  </doc>
void
  compositeToPoint (float, float) (p_x, p_y)
	  fromRect (float, float, float, float) (x, y, w, h)
	 operation int op
{
  [self unimplemented cmd];
}

<doc> Composites the image to aPoint using the dissolve operator. aFloat
    is a value from 0.0 to 1.0 that determines how much of the resulting
    composite comes from the receiver.  </doc>
void
  dissolveToPoint (float, float) (x, y)
	 fraction float f
{
  [self unimplemented cmd];
}

<doc> Composites the aRect portion of the image to aPoint using the
    dissolve operator. aFloat is a value from 0.0 to 1.0 that determines
    how much of the resulting composite comes from the receiver.  </doc>
void
  dissolveToPoint (float, float) (p_x, p_y)
	 fromRect (float, float, float, float) (x, y, w, h)
	 fraction float f
{
  [self unimplemented cmd];
}

<doc> <h4>Choosing Which Image Representation to Use</h4> </doc>

<doc> Determines whether color matches are preferred.  </doc>
void
  setPrefersColorMatch boolean flag
{
  [self unimplemented cmd];
}

<doc> Returns whether color matches are preferred.  </doc>
boolean
  prefersColorMatch
{
  [self unimplemented cmd];
}

<doc> Sets whether to use EPS representations on mismatch.  </doc>
void
  setUsesEPSOnResolutionMismatch boolean flag
{
  [self unimplemented cmd];
}

<doc> Returns whether to use EPS representations on mismatch.  </doc>
boolean
  usesEPSOnResolutionMismatch
{
  [self unimplemented cmd];
}

<doc> Sets whether resolution multiples match.  </doc>
void
  setMatchesOnMultipleResolution boolean flag
{
  [self unimplemented cmd];
}

<doc> Returns whether resolution multiples match.  </doc>
boolean
  matchesOnMultipleResolution
{
  [self unimplemented cmd];
}

<doc> <h4>Getting the Representations</h4> </doc>

<doc> Returns the best representation for the device described by
    {device_description}. If {device_description} is {nil}, the current
    device is assumed. See NSGraphics.h for appropriate dictionary keys
    and values.  </doc>
ImageRep
  bestRepresentationForDevice Device device_description
{
  [self unimplemented cmd];
}

<doc> Returns an array of all the representations.  </doc>
Indexed
  representations
{
  = representations;
}

<doc> Remove {ImageRep} {rep} from the receiver's list of representations.
    </doc>
void
  removeRepresentation ImageRep rep
{
  [representations removeElement rep];
}

<doc> <h4>Determining How the Image is Stored</h4> </doc>

<doc> Sets whether representations are cached separately.  </doc>
void
  setCachedSeparately boolean flag
{
  [self unimplemented cmd];
}

<doc> Returns whether representations are cached separately.  </doc>
boolean
  isCachedSeparately
{
  [self unimplemented cmd];
}

<doc> Sets whether image data is retained by the object after the image is
    cached.  </doc>
void
  setDataRetained boolean flag
{
  [self unimplemented cmd];
}

<doc> Returns whether image data is retained.  </doc>
boolean
  isDataRetained
{
  [self unimplemented cmd];
}

<doc> Sets whether the default depth limit applies to caches.  </doc>
void
  setCacheDepthMatchesImageDepth boolean flag
{
  [self unimplemented cmd];
}

<doc> Returns whether the default depth limit applies to caches.  </doc>
boolean
  cacheDepthMatchesImageDepth
{
  [self unimplemented cmd];
}

<doc> <h4>Determining How the Image is Drawn</h4> </doc>


<doc> Returns YES to indicate that the receiver's image is valid.  </doc>
boolean
  isValid
{
  [self unimplemented cmd];
}

<doc> If flag is YES, representations are scaled to fit.  </doc>
void
  setScalesWhenResized boolean flag
{
  [self unimplemented cmd];
}

<doc> Returns whether representations are scaled to fit.  </doc>
boolean
  scalesWhenResized
{
  [self unimplemented cmd];
}

<doc> Sets the background color of the image to aColor.  </doc>
void
  setBackgroundColor Color aColor
{
  [self unimplemented cmd];
}

<doc> Returns the background color of the image.  </doc>
Color
  backgroundColor
{
  [self unimplemented cmd];
}

<doc> Overridden to have imageRep draw the representation in aRect.
    </doc>
boolean
  drawRepresentation ImageRep imageRep 
	      inRect (float, float, float, float) (x, y, w, h)
{
  [self unimplemented cmd];
}

<doc> Invalidates caches of all representations, so they will be redrawn.
    </doc>
void
  recache
{
  [self unimplemented cmd];
}

<doc> <h4>Assigning a Delegate</h4> </doc>

<doc> Makes anObject the delegate of the {Image}.  </doc>
void
  setDelegate ImageDelegate anObject
{
  [self unimplemented cmd];
}

<doc> Returns the delegate of the {Image}.  </doc>
ImageDelegate
  delegate
{
  [self unimplemented cmd];
}

<doc> <h4>Producing TIFF Data for the Image</h4> </doc>

<doc> Returns a data object containing TIFF for all representations, using
    their default compressions.  </doc>
Data
  TIFFRepresentation
{
  [self unimplemented cmd];
}

<doc> Returns a data object containing TIFF for all the representations.
    </doc>
Data
  TIFFRepresentationUsingCompression int comp
			      factor float aFloat
{
  [self unimplemented cmd];
}

end;

/******************** ImageDelegate ********************/

implementation class
ImageDelegate

end;

implementation instance
ImageDelegate

<doc> Responds to message that image couldn't be composited into aRect.
    </doc>
deferred Image
  imageDidNotDraw Image sender
	   inRect (float, float, float, float) (x, y, w, h);

end;

<doc> Image loading behaviour, with a temporary interface until {Bundle}s
    are fully functional.  Also, should {BitmapImageRep} know everything
    about any kind of image, should there be a {TIFFBitmapImageRep}, or
    should there be a {TIFFImageLoader} which creates {BitmapImageRep}s?
    </doc>
implementation class
ImageLoader: instance (All)

end;

implementation instance
ImageLoader: instance (All)

deferred Image
  loadImageNamed String name;

end;
