<copyright> TAG ImageRep class.
    Written by <a href="mailto:tiggr@es.ele.tue.nl">Pieter J. Schoenmakers</a>

    Copyright &copy; 1996, 1997 Pieter J. Schoenmakers.
    All rights reserved.

    This file is part of TAG.  TAG is distributed under the terms of the
    TAG License, a copy of which can be found in the TAG distribution; see
    the file LICENSE.

    <id>$Id: ImageRep.t,v 1.4 1997/05/01 16:30:15 tiggr Exp $</id>
    </copyright>

implementation class
ImageRep: State,
	tag
{
  <doc> The registered {ImageRep} classes.  </doc>
  static MutableArray registered;
}

<doc> <h4>Creating an NSImageRep</h4> </doc>

<doc> In subclasses that respond to imageFileTypes and imageRepWithData:,
    returns an object that has been initialized with the data in filename.
    NSImageRep's implementation returns an instance of the appropriate
    registered subclass.  </doc>
instance (id)
  imageRepWithContentsOfFile String filename
{
  [self unimplemented cmd];
}

<doc> In subclasses that respond to imageFileTypes and imageRepWithData:
    (or imageRepWithData:), returns an array of objects that have been
    initialized with the data in filename.  NSImageRep's implementation
    returns an array of objects (each an instance of the appropriate
    registered subclass) that have been initialized with the data in
    filename.  </doc>
Indexed
  imageRepsWithContentsOfFile String filename
{
  [self unimplemented cmd];
}

<doc> In subclasses that respond to imagePasteboardTypes and
    imageRepWithData:, returns an object that has been initialized with
    the data in pasteboard.  NSImageRep's implementation returns an
    instance of the appropriate registered subclass.  </doc>
instance (id)
  imageRepWithPasteboard Pasteboard pasteboard
{
  [self unimplemented cmd];
}

<doc> In subclasses that respond to imagePasteboardTypes and
    imageRepsWithData: (or imageRepWithData:), returns an array of objects
    that have been initialized with the data in pasteboard.  NSImageRep's
    implementation returns an array of objects (each an instance of the
    appropriate registered subclass) that have been initialized with the
    data in pasteboard.  </doc>
Indexed
  imageRepsWithPasteboard Pasteboard pasteboard
{
  [self unimplemented cmd];
}

<doc> <h4>Checking Data Types</h4> </doc>

<doc> Overridden in subclasses to return YES if the receiver can
    initialize itself from data.  </doc>
deferred boolean
  canInitWithData Data data;

<doc> Overridden in subclasses to return YES if the receiver can initialize
    itself from pasteboard.  </doc>
deferred boolean
  canInitWithPasteboard Pasteboard pasteboard;

<doc> Returns an array of strings representing all file types.  </doc>
deferred Indexed
  imageFileTypes;

<doc> Returns an array of strings representing all pasteboard types.
    </doc>
deferred Indexed
  imagePasteboardTypes;

<doc> Returns an array of strings representing directly supported file
    types.  </doc>
deferred Indexed
  imageUnfilteredFileTypes;

<doc> Returns an array of strings representing directly supported
    pasteboards.  </doc>
deferred Indexed
  imageUnfilteredPasteboardTypes;

<doc> <h4>Managing NSImageRep Subclasses</h4> </doc>

<doc> Return the {ImageRep} subclass that handles data of type data.
    </doc>
class (ImageRep)
  imageRepClassForData Data data
{
  [self unimplemented cmd];
}

<doc> Return the {ImageRep} subclass that handles data of file type type.
    </doc>
class (ImageRep)
  imageRepClassForFileType String type
{
  [self unimplemented cmd];
}

<doc> Return the {ImageRep} subclass that handles data of pasteboard type
    type.  </doc>
class (ImageRep)
  imageRepClassForPasteboardType String type
{
  [self unimplemented cmd];
}

<doc> Adds {rep_class} to the registry of available {ImageRep} classes.
    </doc>
void
  registerImageRepClass class (ImageRep) rep_class
{
  /* Can't do this in {load} since we can not indicate on dependencies on
     execution.  */
  if (!registered)
    registered = [MutableObjectArray new];

  [registered add rep_class];
}

<doc> Return the registered {ImageRep} classes.  </doc>
Indexed
  registeredImageRepClasses
{
  = registered;
}

<doc> Remove {rep_class} from the registry of available {ImageRep}
    classes.  </doc>
void
  unregisterImageRepClass class (ImageRep) rep_class
{
  if (registered != nil)
    {
      int i = [registered indexOfIdentical rep_class];

      if (i >= 0)
	[registered removeElementAt i];
    }
}

end;

/****************************** ******************************/

implementation instance
ImageRep
{
  <doc> The size.  </doc>
  float width, height;
}

<doc> <h4>Setting the Size of the Image</h4> </doc>

<doc> Set the size of the image.  </doc>
void
  setSize (float, float) (w, h)
{
  (width, height) = (w, h);
}

<doc> Return the size of the image.  </doc>
(float, float)
  size
{
  = (float (width), float (height));
}

<doc> <h4>Specifying Information about the Representation</h4> </doc>

<doc> Returns the number of bits per pixel in each component.  </doc>
deferred int
  bitsPerSample;

<doc> Returns the name of the image's color space.  </doc>
deferred int
  colorSpaceName;

<doc> Returns whether there is a coverage component.  </doc>
deferred boolean
  hasAlpha;

<doc> Returns whether the representation is opaque.  </doc>
deferred boolean
  isOpaque;

<doc> Return the height specified in the image data.  </doc>
int
  pixelsHigh
{
  = int (height);
}

<doc> Return the width specified in the image data.  </doc>
int
  pixelsWide
{
  = int (width);
}

<doc> Informs the receiver whether there is a coverage component.  </doc>
deferred void
  setAlpha boolean flag;

<doc> Informs the receiver there are anInt bits/pixel in a component.
    </doc>
deferred void
  setBitsPerSample int bps;

<doc> Informs the receiver of the image's color space.  </doc>
deferred void
  setColorSpaceName String name;

<doc> Informs the receiver of the image's opacity.  </doc>
deferred void
  setOpaque boolean flag;

<doc> Informs the receiver that its data is for an image anInt pixels
    high.  </doc>
deferred void
  setPixelsHigh int h;

<doc> Informs the receiver that its data is for an image anInt pixels
    wide.  </doc>
deferred void
  setPixelsWide int w;

<doc> <h4>Drawing the Image</h4> </doc>

<doc> Implemented by subclasses to draw the image.  </doc>
deferred boolean
  draw;

<doc> Modifies current coordinates so the image is drawn at aPoint.
    </doc>
deferred boolean
  drawAtPoint (float, float) (x, y);

<doc> Modifies current coordinates so the image is drawn in aRect.  </doc>
deferred boolean
  drawInRect (float, float, float, float) (x, y, w, h);

end;
