<copyright> TAG Matrix class.
    Written by <a href="mailto:tiggr@es.ele.tue.nl">Pieter J. Schoenmakers</a>

    Copyright &copy; 1997-1998 Pieter J. Schoenmakers.
    All rights reserved.

    This file is part of TAG.  TAG is distributed under the terms of the
    TAG License, a copy of which can be found in the TAG distribution; see
    the file LICENSE.

    <id>$Id: Matrix.t,v 1.14 1998/07/28 10:53:49 tiggr Exp $</id>
    </copyright>

implementation class
MatrixDelegate: instance (All)

end;

implementation instance
MatrixDelegate

<doc> Be informed that the {matrix} is about to display the {cell} at
    position {(row, column)}.  </doc>
void
	   matrix Matrix matrix
  willDisplayCell Cell cell
	       at (int, int) (row, column)
{
}

<doc> Be informed of the start of another mouse event cycle by the {cell}
    of the {matrix}.  </doc>
void
	 matrix Matrix matrix
  didStartCycle Cell cell
{
}

end;

implementation class
Matrix: Control, Action

<doc> Set the {cell_class} to {ActionCell}.  </doc>
void
  load Array arguments
{
  cell_class = [ActionCell self];
}

end;

implementation instance
Matrix
{
  <doc> The cells.  </doc>
  MutableArray cells;

  <doc> The number of rows in the matrix.  </doc>
  public int num_rows;

  <doc> The number of columns in the matrix.  </doc>
  public int num_columns;

  <doc> The size of each cell.  </doc>
  float cell_width, cell_height;

  <doc> The size of the space between the cells.  </doc>
  float intercell_width, intercell_height;

  <doc> In case only one cell is selected, this is the selected cell.
      </doc>
  public Cell selected_cell;

  <doc> The coordinates of the selected cell.  </doc>
  int selected_row, selected_column;

  <doc> The prototype cell.  </doc>
  public mutable Cell prototype;

  <doc> The factory cell class.  </doc>
  class (Cell) this_cell_class;

  <doc> The delegate of this matrix.  </doc>
  public mutable MatrixDelegate delegate;

  <doc> The color for the space between the cells.  </doc>
  public mutable Color background_color;

  <doc> The background color of  cells.  </doc>
  public mutable Color cell_background_color;

  <doc> The action upon a double-click.  </doc>
  public mutable selector double_action;

  <doc> The selection mode of this matrix.  </doc>
  int selection_mode;

  public mutable boolean allows_empty_selection;
  public mutable boolean autosizes_cells;
}

<doc> Return {YES} iff the {selection_mode} is not {MATRIX_MODE_LIST}.
    </doc>
boolean
  acceptsFirstMouse
{
  = selection_mode != MATRIX_MODE_LIST;
}

void
  addColumn
{
  [self unimplemented cmd];
}

void
  addColumnWithCells Indexed new_cells
{
  [self unimplemented cmd];
}

void
  addRow
{
  int i, old_num_rows = num_rows++;

  [cells resize num_rows * num_columns];
  while (i < num_columns)
    [self makeCellAt (old_num_rows, i++)];
}

void
  addRowWithCells Indexed new_cells
pre
  [new_cells length] >= num_columns
{
  int i, old_num_rows = num_rows++;

  [cells resize num_rows * num_columns];
  while (i < num_columns)
    cells[i++] = new_cells[i];
}

<doc> Return the {this_cell_class} of this matrix.  </doc>
class (Cell)
  cell_class
{
  = this_cell_class;
}

Cell
  cellAt (int, int) (row, column)
pre
  row >= 0 && column >= 0 && row < num_rows && column < num_columns
{
  = cells[row * num_columns + column];
}

(float, float, float, float)
  cellFrameAt (int, int) (row, column)
pre
  row >= 0 && column >= 0
{
  = (float (column) * (cell_width + intercell_width),
     b_h - float (row + 1) * cell_height - float (row) * intercell_height,
     cell_width, cell_height);
}

(float, float)
  cellSize
{
  = (cell_width, cell_height);
}

Cell
  cellWithTag int some_tag
{
  [self unimplemented cmd];
}

Indexed
  cells
{
  = cells;
}

void
  deselectAllCells
{
  if ([self deselectSelectedCell])
    [self display];
}

void
  deselectSelectedCell
{
  boolean ignore = [self deselectSelectedCell];
}

<doc> Deselect the selected cell or cells, if allowed, and return whether
    the matrix has changed and needs to be redisplayed.  </doc>
protected boolean
  deselectSelectedCell
{
  if (selected_cell != nil)
    {
      if (allows_empty_selection)
	{
	  [selected_cell set_state FALSE];
	  selected_cell = nil;
	  = YES;
	}
      return;
    }

  // Handle multiple-selected-cells case.
  // Fri Dec 19 22:15:17 1997, tiggr@tricky.ics.ele.tue.nl
}

void
  drawCellAt (int, int) (row, column)
{
  [self unimplemented cmd];
}

<doc> Return {TRUE} if this matrix has a {background_color}.  </doc>
boolean
  drawsBackground
{
  = background_color != nil;
}

<doc> Return {TRUE} if this matrix has a {cell_background_color}.  </doc>
boolean
  drawsCellBackground
{
  = cell_background_color != nil;
}

<doc> Draw that part of the matrix that intersects with the indicated
    rectangle.  </doc>
void
  drawSelf (float, float, float, float) (x, y, w, h)
{
  int r, c;

  if (background_color != nil)
    {
      Display d = [self currentDisplay];
      [d setColor background_color];
      [d fillRect (x, y, w, h)];
    }

  for (r = 0; r < num_rows; r++)
    for (c = 0; c < num_columns; c++)
      {
	int index = r * num_columns + c;
	Cell cell = cells[index];
	float cx, cy, cw, ch;

	(cx, cy, cw, ch) = [self cellFrameAt (r, c)];
	if ([tag rect (cx, cy, cw, ch) intersects (x, y, w, h)])
	  {
	    if (delegate != nil)
	      [delegate matrix self willDisplayCell cell at (r, c)];
	    [cell drawSelf (cx, cy, cw, ch) inView self];
	  }
      }
}

(int, int) (rows, columns)
  getDimensions
{
  = (num_rows, num_columns);
}

<doc> Check the cell at the point {(x, y)} in the receiving Matrix.  If
    {within_cell} is returned {FALSE}, no cell is hit.  Otherwise, {row}
    and {column} identify the cell that is hit.  </doc>
(boolean, int, int) (within_cell, row, column)
  getCellCoordinates (float, float) (x, y)
{
  if (y < 0.0 || y >= b_h || x < 0.0 || x >= b_w)
    return;

  row = int ((b_h - y) / (cell_height + intercell_height));
  if (b_h - y
      > (float (row + 1) * (cell_height + intercell_height) - intercell_height))
    return;

  column = int (x / (cell_width + intercell_width));
  if (x > float (column + 1) * (cell_width + intercell_width) - intercell_width)
    return;

  within_cell = TRUE;
}

(int, int) (row, column)
  getCellCoordinates Cell c
pre
  [cells indexOfIdentical c] >= 0
{
  int i = [cells indexOfIdentical c];
  row = i / num_columns;
  column = i - row * num_columns;
}

id
  init
{
  = [self initFrame (0.0, 0.0, 0.0, 0.0)];
}

id
  initFrame (float, float, float, float) (x, y, w, h)
{
  this_cell_class = [ActionCell self];
  = [self initFrame (x, y, w, h) mode MATRIX_MODE_HIGHLIGHT dimensions (0, 0)];
}

id
   initFrame (float, float, float, float) (x, y, w, h)
	mode int sel_mode
   cellClass class (Cell) cell_factory
  dimensions (int, int) (n_rows, n_cols)
{
  this_cell_class = cell_factory;
  = [self initFrame (x, y, w, h) mode sel_mode dimensions (n_rows, n_cols)];
}

id
   initFrame (float, float, float, float) (x, y, w, h)
	mode int sel_mode
   prototype Cell cell_proto
  dimensions (int, int) (n_rows, n_cols)
{
  prototype = cell_proto;
  = [self initFrame (x, y, w, h) mode sel_mode dimensions (n_rows, n_cols)];
}

protected id
   initFrame (float, float, float, float) (x, y, w, h)
	mode int sel_mode
  dimensions (int, int) (n_rows, n_cols)
{
  selection_mode = sel_mode;

  (num_rows, num_columns) = (n_rows, n_cols);
  cells = [MutableObjectArray withCapacity num_rows * num_columns];
  if (num_rows > 0 && num_columns > 0)
    {
      [cells resize num_rows * num_columns];

      int r, c;
      for (r = 0; r < num_rows; r++)
	for (c = 0; c < num_columns; c++)
	  [self makeCellAt (r, c)];

      if (selection_mode == MATRIX_MODE_RADIO)
	{
	  selected_cell = cells[0];
	  [selected_cell set_state YES];
	}
    }

  if (num_columns != 0)
    cell_width = float (int (w) / num_columns);
  if (num_rows != 0)
    cell_height = float (int (h) / num_rows);

  = [super initFrame (x, y, w, h)];
}

(float, float) (width, height)
  intercellSpacing
{
  = (intercell_width, intercell_height);
}

<doc> Create a new cell at the specified position.  </doc>
Cell (new_cell)
  makeCellAt (int, int) (row, column)
pre
  row >= 0 && column >= 0 && row < num_rows && column < num_columns
{
  int index = row * num_columns + column;

  if (prototype != nil)
    new_cell = [prototype copy];
  else if (this_cell_class != nil)
    new_cell = [this_cell_class new];
  else
    new_cell = [cell_class new];

  cells[index] = new_cell;
}

<doc> Set the {this_cell_class} of this matrix.  </doc>
void
  set_cell_class class (Cell) new_cell_class
{
  this_cell_class = new_cell_class;
}

<doc> Adjust {intercell_width} and {intercell_height}.  The frame is not
    modified, no redisplaying is done.  </doc>
void
  setIntercellSpacing (float, float) (width, height)
{
  (intercell_width, intercell_height) = (width, height);
}

<doc> Adjust the {cell_width} and {cell_height}.  </doc>
void
  sizeTo (float, float) (w, h)
{
  if (w != f_w)
    cell_width = float (int (w / float (num_columns)));
  if (h != f_h)
    cell_height = float (int (h / float (num_rows)));

  [super sizeTo (w, h)];
}

<doc> Adjust the frame of the Matrix to accomodate all the cells.  </doc>
void
  sizeToCells
{
  float w, h;

  /* Update CELL_WIDTH and CELL_HEIGHT.  */
  int i;

  cell_width = cell_height = 0.0;
  for (i = num_rows * num_columns - 1; i >= 0; i--)
    {
      (w, h) = [cells[i] cellSize];
      if (w > cell_width)
	cell_width = w;
      if (h > cell_height)
	cell_height = h;
    }

  [self sizeToFit];
}

<doc> Resize the matrix to fit all cells, without affecting the size of
    the cells or the intercell spacing.  </doc>
void
  sizeToFit
{
  float w = float (num_columns) * cell_width;
  float h = float (num_rows) * cell_height;

  if (num_columns > 1)
    w += float (num_columns - 1) * intercell_width;
  if (num_rows > 1)
    h += float (num_rows - 1) * intercell_height;

  if (w != b_w || h != b_h)
    {
      /* Message SUPER to avoid cell/intercell resizing behaviour.  */
      [super sizeTo (w, h)];
    }
}

/******************** target/action ********************/
<doc> <h4>Target/Action</h4> </doc>

// Control should not blindly forward everything to its cell.
// Tue Feb  3 22:46:19 1998, tiggr@tnt.ics.ele.tue.nl

<doc> Forward to {Action}.  </doc>
selector
  action
{
  = [super (Action) action];
}

Any
  target
{
  = [super (Action) target];
}

void
  setTarget All t
{
  [super (Action) setTarget t];
}

void
  setAction selector a
{
  [super (Action) setAction a];
}

/******************** Event Handling ********************/
<doc><h4>Event Handling</h4></doc>

<doc> Set the {cell} to be the cell getting the hit, and let it handle
    this event.  </doc>
void
  mouseDown MouseEvent event
{
  if (selection_mode == MATRIX_MODE_HIGHLIGHT
      || selection_mode == MATRIX_MODE_RADIO)
    {
      float mx, my;
      boolean hit;
      int r, c;

      (mx, my) = [self convertPoint [event location] fromView nil];
      (hit, r, c) = [self getCellCoordinates (mx, my)];

      if (hit)
	{
	  if (selection_mode == MATRIX_MODE_RADIO)
	    {
	      [selected_cell set_state NO];
	      // Only this cell!
	      // Tue Feb 24 23:24:24 1998, tiggr@gerbil.org
	      [self display];
	    }

	  cell = cells[r * num_columns + c];
	  tracking = [cell mouseDown event inRect [self cellFrameAt (r, c)]
			   ofView self];
	  if (tracking && delegate != nil)
	    [delegate matrix self didStartCycle cell];
	}
    }
}

void
  mouseUp MouseEvent event
{
  if (selection_mode == MATRIX_MODE_RADIO
      && selected_cell == cell && [cell state])
    return;

  /* The selected cell must be properly set, for the target to query.  */
  Cell save_selected_cell = selected_cell;
  selected_cell = cell;
  [super mouseUp event];
  selected_cell = save_selected_cell;

  if ([cell state])
    selected_cell = cell;
  else if (selection_mode == MATRIX_MODE_RADIO
	   || !allows_empty_selection)
    {
      if (!selected_cell)
	selected_cell = cell;
      [selected_cell set_state YES];
      [self display];
    }
}

<doc> Substitute the {action} and/or {target} of the receiving Matrix, if
    the provided arguments, {a} and {t} are default selector, or {nil},
    respectively.  </doc>
void
  sendAction selector a
	  to All t
{
  if (!a)
    a = action;
  if (!t)
    t = target;

  [[Application shared] sendAction a to t from self];
}

/******************** Archiving ********************/
<doc><h4>Archiving</h4></doc>

void
  encodeUsingCoder Encoder coder
{
  if (![coder hasBeenCodedFor [Matrix self]])
    {
      [super (Control) encodeUsingCoder coder];
      [super (Action) encodeUsingCoder coder];
      [coder encode cells];
      [coder encode num_rows];
      [coder encode num_columns];
      [coder encode cell_width];
      [coder encode cell_height];
      [coder encode intercell_width];
      [coder encode intercell_height];
      [coder encode selected_cell];
      [coder encode selected_row];
      [coder encode selected_column];
      [coder encode prototype];
      // This can not currently be encoded!
      // Mon Jan 26 23:38:34 1998, tiggr@tnt.ics.ele.tue.nl
      // [coder encode this_cell_class];
      [coder encode background_color];
      [coder encode cell_background_color];
      [coder encode double_action];
      [coder encode selection_mode];
      [coder encode allows_empty_selection];
      [coder encode autosizes_cells];
    }
}

void
  initWithCoder Decoder coder
{
  if (![coder hasBeenCodedFor [Matrix self]])
    {
      [super (Control) initWithCoder coder];
      [super (Action) initWithCoder coder];
      cells = [coder decode];
      num_rows = [coder decode];
      num_columns = [coder decode];
      cell_width = [coder decode];
      cell_height = [coder decode];
      intercell_width = [coder decode];
      intercell_height = [coder decode];
      selected_cell = [coder decode];
      selected_row = [coder decode];
      selected_column = [coder decode];
      prototype = [coder decode];

      // See above.
      // Mon Jan 26 23:38:46 1998, tiggr@tnt.ics.ele.tue.nl
      // this_cell_class = [coder decode];
      this_cell_class = [ActionCell self];

      background_color = [coder decode];
      cell_background_color = [coder decode];
      double_action = [coder decode];
      selection_mode = [coder decode];
      allows_empty_selection = [coder decode];
      autosizes_cells = [coder decode];
    }
}

end;
