<copyright> TAG Menu class.
    Written by <a href="mailto:tiggr@gerbil.org">Pieter J. Schoenmakers</a>

    Copyright &copy; 1998 Pieter J. Schoenmakers.
    All rights reserved.

    This file is part of TAG.  TAG is distributed under the terms of the
    TAG License, a copy of which can be found in the TAG distribution; see
    the file LICENSE.

    <id>$Id: Menu.t,v 1.5 1998/04/15 22:38:34 tiggr Exp $</id>
    </copyright>

implementation class
Menu: tag, MatrixDelegate

end;

implementation instance
Menu
{
  <doc> The window displaying this menu.  </doc>
  public Panel window;

  <doc> The matrix holding the menu items.  </doc>
  public Matrix matrix;

  <doc> The menu causing us to be opened.  </doc>
  public id supermenu;

  <doc> The currently opened submenu.  </doc>
  public id submenu;
}

<doc> Designated initializer.  </doc>
id
  initWithTitle String title
{
  Screen s = [system_display defaultScreen];
  float fx, fy, fw, fh;

  (fx, fy, fw, fh) = [[s deviceDescription] frame];

  window = [s panelWithContentRect (fx, fy + fh - 100.0, 100.0, 100.0)
	      styleMask WINDOW_STYLE_TITLED backing BACKING_STORE_RETAINED];
  matrix = [[Matrix alloc] initFrame [[window content_view] frame]
			   mode MATRIX_MODE_HIGHLIGHT
			   cellClass [MenuCell self] dimensions (0, 1)];
  [matrix set_allows_empty_selection YES];
  [matrix set_delegate self];

  [[window content_view] addSubview matrix];
  [window setTitle title];
  = [super init];
}

MenuCell
  addItem String name
   action selector sel
  target: All target = nil
{
  [matrix addRow];
  Indexed x = [matrix cells];

  MenuCell bc = x[[x length] - 1];
  [bc setStringValue name];
  [bc setHighlightsBy CELL_MASK_CHANGE_BACKGROUND | CELL_MASK_CONTENTS];
  [bc set_bordered TRUE];
  [bc setTarget target];
  [bc setAction sel];
  = bc;

  [self sizeToCells];
}

void
  closeMenuTree
{
  if (submenu != nil)
    {
      [submenu closeMenuTree];
      submenu = nil;
    }
  [window orderOut self];
}

void
  hookSubmenu Menu submenu
	   to MenuCell cell
pre
  [[matrix cells] indexOfIdentical cell] >= 0
{
  [submenu set_supermenu self];
  [cell set_submenu submenu];
  [self sizeToCells];
}

protected void
  set_submenu Menu menu
pre
  submenu == nil
{
  submenu = menu;
}

protected void
  set_supermenu Menu menu
{
  supermenu = menu;
}

void
  showMenu Matrix sender
{
  [supermenu set_submenu self];

  Window sw = [sender window];
  float x, y, w, h;

  (x, y, w, h) = [sw frame];
  (x, y) = (x + w, y + h);
  (, , w, h) = [window frame];

  [window setFrame (x, y - h, w, h)];
  [window orderFront self];
}

void
  sizeToCells
{
  float x, y, w, h, oh;

  [matrix sizeToCells];
  (, , w, h) = [matrix frame];
  (x, y, , oh) = [window frame];
  [window setFrame (x, y + oh - h, w, h)];
}

/******************** MatrixDelegate ********************/

void
	 matrix Matrix matrix
  didStartCycle Cell cell
{
  if (submenu != nil)
    {
      [submenu closeMenuTree];
      submenu = nil;
    }
}

/******************** Archiving ********************/
<doc> <h4>Archiving</h4> </doc>

void
  encodeUsingCoder Encoder coder
{
  if (![coder hasBeenCodedFor [Menu self]])
    {
      [super encodeUsingCoder coder];
      [coder encode window];
      [coder encode matrix];
      [coder encode supermenu];
    }
}

void
  initWithCoder Decoder coder
{
  if (![coder hasBeenCodedFor [Menu self]])
    {
      [super initWithCoder coder];
      window = [coder decode];
      matrix = [coder decode];
      supermenu = [coder decode];
    }
}

end;
