<copyright> TAG ScrollView class.
    Written by <a href="mailto:tiggr@es.ele.tue.nl">Pieter J. Schoenmakers</a>

    Copyright &copy; 1997 Pieter J. Schoenmakers.
    All rights reserved.

    This file is part of TAG.  TAG is distributed under the terms of the
    TAG License, a copy of which can be found in the TAG distribution; see
    the file LICENSE.

    <id>$Id: ScrollView.t,v 1.14 1998/07/21 15:07:52 tiggr Exp $</id>
    </copyright>

implementation class
ScrollView: View

end;

implementation instance
ScrollView
{
  <doc> Our content view, usually a {ClipView}.  </doc>
  public ClipView content_view;

  <doc> Our horizontal scroller, if we have one.  </doc>
  public mutable Scroller scroller_h;

  <doc> Our vertical scroller, if we have one.  </doc>
  public mutable Scroller scroller_v;

  <doc> The kind of border drawn around this view.  </doc>
  public mutable int border_type;

  boolean needs_tile;
}

<doc> Designated initializer.  </doc>
id
  initFrame (float, float, float, float) (x, y, w, h)
{
  border_type = BORDER_BEZEL;
  = [super initFrame (x, y, w, h)];
}

<doc> Do not pass to our {superview}; do invoke {tile}.  </doc>
void
  descendantFrameChanged View descendant
{
  [self tile];
}

void
  drawSelf (float, float, float, float) (x, y, w, h)
{
  if (needs_tile)
    {
      [self tile];
      needs_tile = FALSE;
    }

  Display d = [self currentDisplay];
  float ix, iy, iw, ih;

  (ix, iy, iw, ih) = [d innerRect (0.0, 0.0, b_w, b_h)
			forBorderType border_type];

  if (!(x >= ix && y >= iy && x + w < ix + iw && y + h < iy + ih))
    [d drawRect (0.0, 0.0, b_w, b_h) withBorderType border_type];
}

<doc> Invoke {tile} after the resize.  </doc>
void
  sizeTo (float, float) (x, y)
{
  [super sizeTo (x, y)];
  [self tile];
}

<doc> Invoke {tile}.  </doc>
void
  resizeSubviews (float, float) (old_w, old_h)
{
  [self tile];
}

// The type should only be the clipping interface (of a View) to the
// ClipView so other views can implement it if they so desire.
// Mon Dec  8 23:57:21 1997, tiggr@tricky.es.ele.tue.nl
<doc> Set the {content_view}, which usually is a {ClipView}.  It will be
    added as a subview, and resized to fit the document region of the
    receiving ScrollView.  </doc>
void
  set_content_view ClipView view
{
  if (content_view != nil)
    [content_view removeFromSuperview];

  content_view = view;
  [self addSubview content_view];
  [self tile];
}

<doc> Set the document view, which will actually become the document view
    of the {content_view} {ClipView}.  </doc>
void
  set_document_view View view
{
  if (content_view != nil)
    {
      [content_view set_document_view view];
      [self tile];
    }
}

<doc> If {yes_p} this ScrollView will have a horizontal scroller.
    Otherwise, it will not.  </doc>
void
  set_horizontal_scroller_required boolean yes_p
{
  if (yes_p)
    {
      if (!scroller_h)
	{
	  scroller_h = [[Scroller alloc]
			 initFrame (0.0, 0.0, SCROLLER_WIDTH + 1.0,
				    SCROLLER_WIDTH)];
	  [scroller_h setTarget self];
	  [scroller_h setAction selector (void getHorizontalFrom All)];
	  [scroller_h set_continuous YES];
	  [self addSubview scroller_h];
	  [self tile];
	}
    }
  else if (scroller_h != nil)
    {
      [scroller_h removeFromSuperview];
      scroller_h = nil;
      [self tile];
    }
}

<doc> If {yes_p} this ScrollView will have a vertical scroller.
    Otherwise, it will not.  </doc>
void
  set_vertical_scroller_required boolean yes_p
{
  if (yes_p)
    {
      if (!scroller_v)
	{
	  scroller_v = [[Scroller alloc]
			 initFrame (0.0, 0.0, SCROLLER_WIDTH,
				    SCROLLER_WIDTH + 1.0)];
	  [scroller_v setTarget self];
	  [scroller_v setAction selector (void getVerticalFrom All)];
	  [scroller_v set_continuous YES];
	  [self addSubview scroller_v];
	  [self tile];
	}
    }
  else if (scroller_v != nil)
    {
      [scroller_v removeFromSuperview];
      scroller_v = nil;
      [self tile];
    }
}

/******************** Positioning ********************/
<doc><h4>Positioning</h4></doc>

<doc> Get a new position from the {scroller}.  </doc>
private (boolean, double) (changed, new_pos)
  getNewPos Scroller scroller
{
  double pos = [scroller doubleValue];
  double perc = [scroller percentage];
  int how = [scroller hitPart];

  new_pos = pos;

  if (how == SCROLLER_PART_LESS)
    new_pos -= perc * 0.1;
  else if (how == SCROLLER_PART_MORE)
    new_pos += perc * 0.1;
  else if (how == SCROLLER_PART_PAGE_LESS)
    new_pos -= perc * 0.9;
  else if (how == SCROLLER_PART_PAGE_MORE)
    new_pos += perc * 0.9;
  else
    {
      changed = TRUE;
      return;
    }

  if (new_pos < 0.0)
    new_pos = 0.0;
  else if (new_pos > 1.0)
    new_pos = 1.0;

  if (pos != new_pos)
    {
      [scroller setDoubleValue new_pos];
      changed = TRUE;
    }
}

<doc> Be informed by the horizontal scroller of a change in horizontal
    position.  </doc>
void
  getHorizontalFrom Scroller sender
{
  if (content_view != nil)
    {
      boolean changed;
      double pos;

      (changed, pos) = [self getNewPos sender];
      if (changed)
	[content_view setHorizontalPosition pos];	
    }
}

<doc> Be informed by the vertical scroller of the new vertical position.
    </doc>
void
  getVerticalFrom Scroller sender
{
  if (content_view != nil)
    {
      boolean changed;
      double pos;

      (changed, pos) = [self getNewPos sender];
      if (changed)
	[content_view setVerticalPosition pos];	
    }
}

<doc> Update the frames of the {content_view}, its {document_view}, and
    the scrollers.  </doc>
void
  tile
{
  float ibx, iby, ibw, ibh;

  if (!window)
    {
      needs_tile = YES;
      return;
    }

  (ibx, iby, ibw, ibh) = [[self currentDisplay] innerRect (0.0, 0.0, b_w, b_h)
						forBorderType border_type];

  float bw = ibw, bx = ibx;
  float bh = ibh, by = iby;

  if (scroller_v != nil)
    {
      [scroller_v setFrame (bx, by, SCROLLER_WIDTH, bh)];
      bx += SCROLLER_WIDTH;
      bw -= SCROLLER_WIDTH;
    }

  if (scroller_h != nil)
    {
      [scroller_h setFrame (bx, by, bw, SCROLLER_WIDTH)];
      by += SCROLLER_WIDTH;
      bh -= SCROLLER_WIDTH;
    }

  if (content_view != nil)
    {
      [content_view setFrame (bx, by, bw, bh)];

      View document_view = [content_view document_view];
      if (document_view != nil)
	{
	  float dfx, dfy, dfw, dfh;
	  boolean changed;

	  (dfx, dfy, dfw, dfh) = [document_view frame];

	  if (dfy + dfh < bh)
	    {
	      changed = YES;
	      dfy = bh - dfh;
	      // if (!flipped &&
	      if (dfy > 0.0)
		dfy = 0.0;
	    }

	  if (dfx + dfw < bw)
	    {
	      changed = YES;
	      dfx = bw - dfw;
	      if (dfx > 0.0)
		dfx = 0.0;
	    }

	  if (changed)
	    [document_view setFrame (dfx, dfy, dfw, dfh)];
	}
    }

  [self syncScrollers];
  needs_display = TRUE;
}

<doc> Update the percentage and position of the scrollers, according to
    the dimensions and position of the {document_view} and {content_view}.
    </doc>
protected void
  syncScrollers
{
  float cfx, cfy, cfw, cfh, dfx, dfy, dfw, dfh;

  if (content_view != nil)
    {
      (cfx, cfy, cfw, cfh) = [content_view frame];

      View document_view = [content_view document_view];
      if (document_view != nil)
	(dfx, dfy, dfw, dfh) = [document_view frame];

      //      if (scroller_h != nil)
      //	(cfy, cfh) = (cfy + SCROLLER_WIDTH, cfh - SCROLLER_WIDTH);
      //      if (scroller_v != nil)
      //	(cfx, cfw) = (cfx + SCROLLER_WIDTH, cfw - SCROLLER_WIDTH);
    }

  if (scroller_h != nil)
    {
      if (dfw > cfw)
	[scroller_h setValue -dfx / (dfw - cfw) andPercentage cfw / dfw];
      else
	[scroller_h setValue 0.0 andPercentage 1.0];
    }

  if (scroller_v != nil)
    {
      if (dfh > cfh)
	[scroller_v setValue -dfy / (dfh - cfh) andPercentage cfh / dfh];
      else
	[scroller_v setValue 0.0 andPercentage 1.0];
    }
}

/******************** Archiving ********************/
<doc><h4>Archiving</h4></doc>

void
  encodeUsingCoder Encoder coder
{
  if (![coder hasBeenCodedFor [ScrollView self]])
    {
      [super encodeUsingCoder coder];
      [coder encode content_view];
      [coder encode scroller_h];
      [coder encode scroller_v];
      [coder encode border_type];
    }
}

void
  initWithCoder Decoder coder
{
  if (![coder hasBeenCodedFor [ScrollView self]])
    {
      [super initWithCoder coder];
      content_view = [coder decode];
      scroller_h = [coder decode];
      scroller_v = [coder decode];
      border_type = [coder decode];
    }
}

end;
