<copyright> TAG Scroller class.
    Written by <a href="mailto:tiggr@es.ele.tue.nl">Pieter J. Schoenmakers</a>

    Copyright &copy; 1997 Pieter J. Schoenmakers.
    All rights reserved.

    This file is part of TAG.  TAG is distributed under the terms of the
    TAG License, a copy of which can be found in the TAG distribution; see
    the file LICENSE.

    <id>$Id: Scroller.t,v 1.14 1998/07/21 15:07:53 tiggr Exp $</id>
    </copyright>

implementation class
Scroller: Action, Control, Values
{
  <doc> Horizontal scrolling arrow images.  Order is: left, right, page
      left, page right, plux highlighted versions.  </doc>
  static Array arrows_hor;

  <doc> Similarly, the vertial arrows, with left being bottom, and right
      being top.  </doc>
  static Array arrows_ver;

  <doc> The knob image.  </doc>
  static Image knob;

  <doc> The width of the horizontal single arrow image.  </doc>
  static float normal_arrow_width;

  <doc> The width of the horizontal page arrow image.  </doc>
  static float page_arrow_width;

  <doc> The configuration of scrollers in this app.  </doc>
  static int scroller_configuration;

  // This should not be here, but part of the display, since multiple
  // displays can scroll...
  // Tue Feb 24 16:38:28 1998, tiggr@gerbil.org
  <doc> The timer used to time scroll events.  </doc>
  Timer scroll_timer;

  // Same remark applies...
  // Tue Feb 24 16:39:37 1998, tiggr@gerbil.org
  <doc> The so-called position of the highlighted arrow.  </doc>
  float highlighted_pos;

  // Same here...
  // Tue Feb 24 17:09:59 1998, tiggr@gerbil.org
  <doc> During mouse dragging, the distance from the mouse to the lower
      (left, bottom) side of the knob, in the positive (right, upper)
      direction.  </doc>
  float offset;

  <doc> The extra space needed for the scroller knob to move before we
      introduce the page scrolling buttons.  </doc>
  const EXCESS_PAGE_ROOM = 10.0;

  <doc> The time in seconds between scroll events.  </doc>
  const SCROLL_REPEAT = 0.05;
  const SCROLL_PAGE_REPEAT = 0.1;
}

<doc> Load the images and set {arrows_width} and {arrows_height}.  </doc>
private void
  load_images
{
  Array h_names
    = [ObjectArray with ("scroll-left", "scroll-right",
			 "scroll-left-page", "scroll-right-page",
			 "scroll-left-h", "scroll-right-h",
			 "scroll-left-page-h", "scroll-right-page-h")];
  Array v_names
    = [ObjectArray with ("scroll-down", "scroll-up",
			 "scroll-down-page", "scroll-up-page",
			 "scroll-down-h", "scroll-up-h",
			 "scroll-down-page-h", "scroll-up-page-h")];
  int i, n = [h_names length];
  MutableObjectArray o = [MutableObjectArray withCapacity n];

  for (i = 0; i < n; i++)
    o[i] = [Image named h_names[i]];
  arrows_hor = o;

  o = [MutableObjectArray withCapacity n];
  for (i = 0; i < n; i++)
    o[i] = [Image named v_names[i]];
  arrows_ver = o;

  (normal_arrow_width, ) = [arrows_hor[0] size];
  (page_arrow_width, ) = [arrows_hor[2] size];
  knob = [Image named "scroll-knob"];
}

<doc> Return the positional information for each of the elements of the
    scroller, with the indicated features.  A position of -1.0 means the
    indicated part of the scroller is not to be drawn.  If {knob_length ==
    -1}, the knob is not to be drawn; if in addition {knob_pos == -1} the
    size of the scroller is too small to draw the knob.  </doc>
(float, float, float, float, float, float, float, float)
(pos_less, pos_more, pos_less_page, pos_more_page,
 bg_pos, bg_length, knob_pos, knob_length)
  sizesForLength float length
	   value float value
      percentage float percentage
      horizontal boolean horizontal_p
       withPages boolean page_p
pre
  value >= 0.0 && value <= 1.0
  && percentage >= 0.0 && percentage <= 1.0
{
  if (percentage == 1.0)
    {
      /* Nothing to be drawn.  */
      return (-1.0, -1.0, -1.0, -1.0, 1.0, length - 2.0, 1.0, -1.0);
    }

  if (length < 2.0 + 2.0 * normal_arrow_width)
    {
      /* Not enough room to draw anything.  */
      return (-1.0, -1.0, -1.0, -1.0, 1.0, length - 2.0, -1.0, -1.0);
    }

  pos_less = 1.0;
  pos_more = 1.0 + normal_arrow_width;
  bg_pos = 2.0 + 2.0 * normal_arrow_width;

  if (length - 2.0 - bg_pos <= SCROLLER_WIDTH)
    {
      /* Not enough room for anything but the single arrows.  */
      return (pos_less, pos_more, -1.0, -1.0,
	      bg_pos, length - 1.0 - bg_pos, -1.0, -1.0);
    }

  /* We have space for the single arrows and the knob.  Maybe also (more
     than enough) for the page arrows, if so desired.  */
  if (page_p && length >= (SCROLLER_WIDTH + EXCESS_PAGE_ROOM
			   + 2.0 + 2.0 * normal_arrow_width
			   + 2.0 * page_arrow_width))
    {
      pos_less += 2.0 * page_arrow_width;
      pos_more += 2.0 * page_arrow_width;
      (pos_less_page, pos_more_page) = (1.0, 1.0 + page_arrow_width);
      bg_pos = bg_pos + 2.0 * page_arrow_width;
    }
  else
    pos_less_page = pos_more_page = -1.0;

  knob_length = percentage * (length - 1.0 - bg_pos);
  if (knob_length < SCROLLER_WIDTH)
    knob_length = SCROLLER_WIDTH;
  else
    knob_length = float (int (knob_length));

  knob_pos = bg_pos + value * (length - 1.0 - bg_pos - knob_length);
  knob_pos = float (int (knob_pos + 0.5));

  bg_length = length - 1.0 - bg_pos;
}

end;

implementation instance
Scroller
{
  <doc> The size of the knob, between 0.0 and 1.0 (inclusive).  Too small
      a size is clipped to {SCROLLER_WIDTH} when drawing.  </doc>
  public float percentage;

  <doc> The position of the knob, between 0.0 and 1.0, as a positive
      offset from the bottom/left.  </doc>
  float value;

  <doc> The arrow that is highlighted, plus 1: {SCROLLER_PART_KNOB} is no
      arrow; {SCROLLER_PART_LESS} is lower, {SCROLLER_PART_MORE} is
      higher, {SCROLLER_PART_PAGE_LESS} is page-lower, and
      {SCROLLER_PART_PAGE_MORE} is page-higher.  In addition, a value of
      {SCROLLER_PART_NONE} means nothing useful was being touched.  </doc>
  public int highlighted_arrow;

  <doc> A continuous scroller will update its target during dragging.  A
      non-continuous one only upon {mouseUp}.  </doc>
  public mutable boolean continuous;

  <doc> Configuration of this scroller.  </doc>
  byte configuration;
}

<doc> Designated initializer.  This invokes {load_images} if needed.
    Default, a vertical scroller will (attempt to) show page scrolling; a
    horizontal scroller will not.  </doc>
id
  initFrame (float, float, float, float) (x, y, w, h)
{
  if (!arrows_hor)
    [isa load_images];

  percentage = 1.0;
  configuration = byte (scroller_configuration
			| (w > h ? 0 : SCROLLER_CONFIG_PAGE));

  = [super initFrame (x, y, w, h)];
}

<doc> Return the {value}.  </doc>
double
  doubleValue
{
  = value;
}

<doc> Set the {value} and invoke {display}.  </doc>
void
  setDoubleValue double v
pre
  v >= 0.0 && v <= 1.0
{
  value = float (v);
  [self display];
}

private void
  drawArrow int arrow
       from Indexed array
	 at float pos
    clipped (float, float, float, float) (x, y, w, h)
{
  Image im = array[arrow - 1 + (arrow == highlighted_arrow ? 4 : 0)];
  boolean vertical = b_h > b_w;

  if ((vertical && (pos + normal_arrow_width > y && pos < y + h))
      || (!vertical && (pos + normal_arrow_width > x && pos < x + w)))
    [im compositeToPoint (vertical ? 1.0 : pos, vertical ? pos : 1.0)
	operation COMPOSITE_COPY];
}

void
  drawSelf (float, float, float, float) (x, y, w, h)
{
  boolean vertical = b_h > b_w;
  float length = vertical ? b_h : b_w;
  Indexed arrows = vertical ? arrows_ver : arrows_hor;
  Display d = [self currentDisplay];

  [d setColor [Color light]];
  if (vertical)
    [d drawRect (0.0, 0.0, b_w - 1.0, b_h)];
  else
    [d drawRect (0.0, 0.0, b_w, b_h - 1.0)];

  [d setColor [Color black]];
  if (vertical)
    [d drawLine (b_w - 1.0, 0.0, b_w - 1.0, b_h)];
  else
    [d drawLine (0.0, b_h - 1.0, b_w, b_h - 1.0)];

  float pos_less, pos_more, pos_less_page, pos_more_page;
  float bg_pos, bg_length, knob_pos, knob_length;
  (pos_less, pos_more, pos_less_page, pos_more_page,
   bg_pos, bg_length, knob_pos, knob_length)
    = [isa sizesForLength length value value
	   percentage percentage horizontal !vertical
	   withPages (configuration & SCROLLER_CONFIG_PAGE != 0)];

  if (pos_less >= 0.0)
    [self drawArrow 1 from arrows at pos_less clipped (x, y, w, h)];
  if (pos_more >= 0.0)
    [self drawArrow 2 from arrows at pos_more clipped (x, y, w, h)];
  if (pos_less_page >= 0.0)
    [self drawArrow 3 from arrows at pos_less_page clipped (x, y, w, h)];
  if (pos_more_page >= 0.0)
    [self drawArrow 4 from arrows at pos_more_page clipped (x, y, w, h)];

  if (knob_length == -1.0)
    {
      [d setColor knob_pos == -1.0 ? [Color light] : [Color dark]];
      if (vertical)
	[d fillRect (1.0, bg_pos, b_w - 3.0, b_h - bg_pos - 1.0)];
      else
	[d fillRect (bg_pos, 1.0, b_w - bg_pos - 1.0, b_h - 3.0)];
    }
  else
    {
      float bx, by, bw, bh;
      float xo, yo;

      if (vertical)
	{
	  [d setColor [Color dark]];
	  if (bg_pos != knob_pos)
	    [d fillRect (1.0, bg_pos, b_w - 3.0, knob_pos - bg_pos)];
	  if (value < 1.0)
	    [d fillRect (1.0, knob_pos + knob_length,
			 b_w - 3.0, b_h - 1.0 - knob_pos - knob_length)];
	  (bx, by, bw, bh) = (1.0, knob_pos, b_w - 3.0, knob_length);
	  xo = -1.0;
	}
      else
	{
	  [d setColor [Color dark]];
	  if (bg_pos != knob_pos)
	    [d fillRect (bg_pos, 1.0, knob_pos - bg_pos, b_h - 3.0)];
	  if (value < 1.0)
	    [d fillRect (knob_pos + knob_length, 1.0,
			 b_w - 2.0 - knob_pos - knob_length, b_h - 3.0)];
	  (bx, by, bw, bh) = (knob_pos, 1.0, knob_length, b_h - 3.0);
	  yo = 1.0;
	}

      float iw, ih;
      (iw, ih) = [knob size];

      [d drawButton (bx, by, bw, bh) fill: YES];
      [knob compositeToPoint (bx + xo + (bw - iw) / 2.0,
			      by + yo + (bh - ih) / 2.0)
	    operation COMPOSITE_COPY];
    }
}

<doc> Return the {highlighted_arrow}.  </doc>
int
  hitPart
{
  return highlighted_arrow;
}

void
  mouseDown MouseEvent event
{
  if (percentage == 1.0)
    return;

  boolean vertical = b_h > b_w;
  float length = vertical ? b_h : b_w;

  float pos_less, pos_more, pos_less_page, pos_more_page;
  float bg_pos, bg_length, knob_pos, knob_length;
  (pos_less, pos_more, pos_less_page, pos_more_page,
   bg_pos, bg_length, knob_pos, knob_length)
    = [isa sizesForLength length value value
	   percentage percentage horizontal !vertical
	   withPages (configuration & SCROLLER_CONFIG_PAGE != 0)];

  float mx, my;
  (mx, my) = [self convertPoint [event location] fromView nil];

  float k, mouse = vertical ? my : mx;

  Indexed arrows = vertical ? arrows_ver : arrows_hor;
  if (mouse >= bg_pos && mouse < bg_pos + bg_length)
    if (knob_pos == -1.0)
      highlighted_arrow = SCROLLER_PART_NONE;
    else if (mouse >= knob_pos && mouse < knob_pos + knob_length)
      {
	highlighted_arrow = SCROLLER_PART_KNOB;
	offset = mouse - knob_pos;
      }
    else
      {
	highlighted_arrow = SCROLLER_PART_BACKGROUND;

	float k = mouse - knob_length / 2.0;
	if (k < bg_pos)
	  k = bg_pos;
	else if (k > bg_pos + bg_length - knob_length)
	  k = bg_pos + bg_length - knob_length;
	value = (k - bg_pos) / (bg_length - knob_length);
	offset = mouse - k;
	if (vertical)
	  [self display (1.0, bg_pos, b_w - 3.0, bg_length)];
	else
	  [self display (bg_pos, 1.0, bg_length, b_h - 3.0)];
	if (continuous)
	  [self sendAction action to target];
      }
  else if (mouse >= pos_less && mouse < pos_less + normal_arrow_width
	   && pos_less != -1.0)
    (highlighted_arrow, highlighted_pos) = (1, pos_less);
  else if (mouse >= pos_more && mouse < pos_more + normal_arrow_width
	   && pos_more != -1.0)
    (highlighted_arrow, highlighted_pos) = (2, pos_more);
  else if (mouse >= pos_less_page && mouse < pos_less_page + page_arrow_width
	   && pos_less_page != -1.0)
    (highlighted_arrow, highlighted_pos) = (3, pos_less_page);
  else if (mouse >= pos_more_page && mouse < pos_more_page + page_arrow_width
	   && pos_more_page != -1.0)
    (highlighted_arrow, highlighted_pos) = (4, pos_more_page);
  else
    highlighted_arrow = SCROLLER_PART_NONE;

  if (highlighted_arrow > 0)
    {
      Invocation i = [Invocation of selector (void sendAction selector to All)
				 to: self with (action, target)];

      [i fire];

      [self lockFocus];
      [self drawArrow highlighted_arrow from arrows
	    at highlighted_pos clipped (b_x, b_y, b_w, b_h)];
      [self unlockFocus];

      scroll_timer
	= [Timer withInterval (highlighted_arrow < SCROLLER_PART_PAGE_LESS
			       ? SCROLL_REPEAT : SCROLL_PAGE_REPEAT)
		 invocation i repeats: YES];
      [scroll_timer schedule];
    }
}

void
  mouseDragged MouseEvent event
{
  if (highlighted_arrow == SCROLLER_PART_BACKGROUND)
    highlighted_arrow = SCROLLER_PART_KNOB;

  if (percentage == 1.0 || highlighted_arrow == -1)
    return;

  if (highlighted_arrow == SCROLLER_PART_KNOB)
    {
      boolean vertical = b_h > b_w;
      float length = vertical ? b_h : b_w;

      float pos_less, pos_more, pos_less_page, pos_more_page;
      float bg_pos, bg_length, knob_pos, knob_length;
      (pos_less, pos_more, pos_less_page, pos_more_page,
       bg_pos, bg_length, knob_pos, knob_length)
	= [isa sizesForLength length value value
	       percentage percentage horizontal !vertical
	       withPages (configuration & SCROLLER_CONFIG_PAGE != 0)];
  
      float mx, my;
      (mx, my) = [self convertPoint [event location] fromView nil];

      float mouse = vertical ? my : mx;
      float k = mouse - offset;

      if (k < bg_pos)
	k = bg_pos;
      else if (k > bg_pos + bg_length - knob_length)
	k = bg_pos + bg_length - knob_length;

      float new_value = (k - bg_pos) / (bg_length - knob_length);
      if (value != new_value)
	{
	  value = new_value;
	  if (vertical)
	    [self display (1.0, bg_pos, b_w - 3.0, bg_length)];
	  else
	    [self display (bg_pos, 1.0, bg_length, b_h - 3.0)];
	  if (continuous)
	    [self sendAction action to target];
	}
    }
}

void
  mouseUp MouseEvent event
{
  if (percentage == 1.0)
    return;

  if (highlighted_arrow > 0)
    {
      int arrow = highlighted_arrow;
      highlighted_arrow = SCROLLER_PART_NONE;

      [scroll_timer cancel];
      /* Look mom!  Manual storage management!  */
      scroll_timer = nil;

      [self lockFocus];
      [self drawArrow arrow from b_h > b_w ? arrows_ver : arrows_hor
	    at highlighted_pos clipped (b_x, b_y, b_w, b_h)];
      [self unlockFocus];
    }
  else if (!continuous)
    {
      [self sendAction action to target];
      highlighted_arrow = SCROLLER_PART_NONE;
    }
}

void
       setValue float v
  andPercentage float p
pre
  v >= 0.0 && v <= 1.0 && p >= 0.0 && p <= 1.0
{
  if (v != value || p != percentage)
    {
      (value, percentage) = (v, p);
      needs_display = TRUE;
    }
}

/******************** target/action ********************/
<doc> <h4>Target/Action</h4> </doc>

<doc> Forward to {Action}.  </doc>
selector
  action
{
  = [super (Action) action];
}

Any
  target
{
  = [super (Action) target];
}

void
  setTarget All t
{
  [super (Action) setTarget t];
}

void
  setAction selector a
{
  [super (Action) setAction a];
}

/******************** Archiving ********************/
<doc><h4>Archiving</h4></doc>

void
  encodeUsingCoder Encoder coder
{
  if (![coder hasBeenCodedFor [Scroller self]])
    {
      [super (Action) encodeUsingCoder coder];
      [super (Control) encodeUsingCoder coder];
      [coder encode percentage];
      [coder encode value];
      [coder encode continuous];
    }
}

void
  initWithCoder Decoder coder
{
  if (![coder hasBeenCodedFor [Scroller self]])
    {
      [super (Action) initWithCoder coder];
      [super (Control) initWithCoder coder];
      percentage = [coder decode];
      value = [coder decode];
      continuous = [coder decode];
    }
}

end;
