<copyright> TAG SliderCell class.
    Written by <a href="mailto:tiggr@es.ele.tue.nl">Pieter J. Schoenmakers</a>

    Copyright &copy; 1996, 1997 Pieter J. Schoenmakers.
    All rights reserved.

    This file is part of TAG.  TAG is distributed under the terms of the
    TAG License, a copy of which can be found in the TAG distribution; see
    the file LICENSE.

    <id>$Id: SliderCell.t,v 1.14 1998/01/26 13:05:48 tiggr Exp $</id>
    </copyright>

implementation class
SliderCell: ActionCell

end;

implementation instance
SliderCell
{
  <doc> Current value.  </doc>
  public double value;

  <doc> Minimum value.  </doc>
  public mutable double min_value;

  <doc> Maximum value.  </doc>
  public mutable double max_value;

  <doc> The horizontal offset from the mouseDown event to the left side of
      the knob.  </doc>
  float offset;

  <doc> Iff {TRUE}, we are vertical.  </doc>
  boolean vertical;

  <doc> Iff {TRUE} we continuous update our status with our target.  </doc>
  public mutable boolean continuous;

  <doc> Iff {TRUE}, we're actually been dragged.  Reset upon mouse-down.
      Consulted upon mouse-up.  </doc>
  boolean dragged;
}

id
  init
{
  is_image = enabled = YES;
  max_value = 1.0;
  value = 0.314159265;
  = [super init];
}

double
  doubleValue
{
  = value;
}

void
  setDoubleValue double v
{
  if (v > max_value)
    value = max_value;
  else if (v < min_value)
    value = min_value;
  else
    value = v;
}

void
  set_vertical boolean v
{
  [self set_image [Image named v ? "slider-vertical" : "slider-horizontal"]];
  vertical = v;
}

redefine (float, float)
  cellSize
{
  if (!info)
    return [super cellSize];

  Image image = info;
  float w, h;

  (w, h) = [image size];
  = (w + 2.0, h + 2.0);
}

redefine void
  drawInside (float, float, float, float) (x, y, w, h)
      inView View view
{
  Display d = [view currentDisplay];

  (x, y, w, h) = [d innerBWRect (x, y, w, h)];

  [d setColor [Color dark]];
  [d fillRect (x, y, w, h)];

  Image knob = info;
  float iw, ih;
  (iw, ih) = [knob size];

  if (vertical)
    {
      float range = h - ih;

      [knob compositeToPoint (x, float (y + (value - min_value)
					/ (max_value - min_value) * range))
	    operation COMPOSITE_COPY];
    }
  else
    {
      float range = w - iw;

      [knob compositeToPoint (float (x + (value - min_value)
				     / (max_value - min_value) * range), y)
	    operation COMPOSITE_COPY];
    }
}

redefine void
  drawSelf (float, float, float, float) (x, y, w, h)
    inView View view
{
  [[view currentDisplay] drawBWRect (x, y, w, h)];
  [self drawInside (x, y, w, h) inView view];
}

/******************** Event handling ********************/

boolean
  mouseDown MouseEvent e
     inRect (float, float, float, float) (x, y, w, h)
     ofView View v
{
  if (enabled)
    {
      Display d = [v currentDisplay];
      [d set_track_rectangle (x, y, w, h)];

      float kx, ky, kw, kh, range, mx, my;
      Image knob = info;

      dragged = FALSE;
      (kw, kh) = [knob size];
      range = vertical ? (h - kh) : (w - kw);

      (mx, my) = [v convertPoint [e location] fromView nil];

      if (vertical)
	{
	  ky = float ((value - min_value) / (max_value - min_value)) * range;

	  if (my < ky || my >= ky + kh)
	    {
	      offset = kh / 2.0;
	      ky = my - offset;
	      if (ky < y)
		ky = y;
	      else if (ky > y + range)
		ky = y + range;

	      value = min_value + (max_value - min_value) * ((ky - y) / range);
	      [v display];
	    }
	  else
	    offset = my - ky;
	}
      else
	{
	  kx = float ((value - min_value) / (max_value - min_value)) * range;

	  if (mx < kx || mx >= kx + kw)
	    {
	      offset = kw / 2.0;
	      kx = mx - offset;
	      if (kx < x)
		kx = x;
	      else if (kx > x + range)
		kx = x + range;

	      value = min_value + (max_value - min_value) * ((kx - x) / range);
	      [v display];
	    }
	  else
	    offset = mx - kx;
	}
    }

  = enabled;
}

redefine void
  mouseDragged MouseEvent e
	    in Control v
{
  float kw, kh, okx, oky, kx, ky, range, x, y, w, h, mx, my;
  double ovalue = value;
  Image knob = info;

  (kw, kh) = [knob size];
  (x, y, w, h) = [[v currentDisplay] track_rectangle];
  range = vertical ? (h - kh) : (w - kw);

  (mx, my) = [v convertPoint [e location] fromView nil];

  if (vertical)
    {
      oky = float ((value - min_value) / (max_value - min_value)) * range;
      ky = my - offset;

      if (ky < y)
	ky = y;
      else if (ky > y + range)
	ky = y + range;

      value = min_value + (max_value - min_value) * ((ky - y) / range);
      if (ky - oky >= 0.5 || ky - oky < -0.5)
	[v display];
    }
  else
    {
      okx = float ((value - min_value) / (max_value - min_value)) * range;
      kx = mx - offset;

      if (kx < x)
	kx = x;
      else if (kx > x + range)
	kx = x + range;

      value = min_value + (max_value - min_value) * ((kx - x) / range);
      if (kx - okx >= 0.5 || kx - okx < -0.5)
	[v display];
    }

  if (continuous && ovalue != value)
    {
      [v sendAction [self action] to [self target]];
      dragged = TRUE;
    }
}

<doc> Send the {action} to the {target} if we haven't already done so.
    </doc>
redefine void
  mouseUp MouseEvent e
       in Control v
{
  if (!continuous || !dragged)
    [v sendAction [self action] to [self target]];
}

/******************** Archiving ********************/
<doc> <h4>Archiving</h4> </doc>

void
  encodeUsingCoder Encoder coder
{
  if (![coder hasBeenCodedFor [SliderCell self]])
    {
      [super encodeUsingCoder coder];
      [coder encode min_value];
      [coder encode value];
      [coder encode max_value];
      [coder encode vertical];
      [coder encode continuous];
    }
}

void
  initWithCoder Decoder coder
{
  if (![coder hasBeenCodedFor [SliderCell self]])
    {
      [super initWithCoder coder];
      min_value = [coder decode];
      value = [coder decode];
      max_value = [coder decode];
      vertical = [coder decode];
      continuous = [coder decode];
    }
}

<doc> Before passing to super, invoke {set_vertial} to get the image.
    </doc>
id
  awakeAfterUsingCoder Decoder coder
{
  [self set_vertical vertical];
  = [super awakeAfterUsingCoder coder];
}

end;
