<copyright> TAG View class.
    Written by <a href="mailto:tiggr@es.ele.tue.nl">Pieter J. Schoenmakers</a>

    Copyright &copy; 1996, 1997 Pieter J. Schoenmakers.
    All rights reserved.

    This file is part of TAG.  TAG is distributed under the terms of the
    TAG License, a copy of which can be found in the TAG distribution; see
    the file LICENSE.

    <id>$Id: View.t,v 1.29 1998/07/21 15:07:55 tiggr Exp $</id>
    </copyright>

implementation class
View: Responder

end;

implementation instance
View
{
  <doc> Our window.  </doc>
  public mutable Window window;

  <doc> Our containing view.  </doc>
  public id superview;

  <doc> Our subviews.  </doc>
  public MutableIndexed subviews;

  <doc> The way we move and size with our superview being resized.  </doc>
  public mutable int auto_size;

  <doc> Our frame in our parent.  </doc>
  float f_x, f_y, f_w, f_h;

  <doc> Our bounds.  </doc>
  float b_x, b_y, b_w, b_h;

  <doc> Iff !0, this view should be redrawn.  </doc>
  public mutable boolean needs_display;
}

OutputStream
  writeFields OutputStream s
{
  = [s print (" frame=(", f_x, ", ", f_y, ", ", f_w, ", ", f_h, ") sub=",
	      subviews)];
}

Screen
  screen
{
  Display p = printer_display;

  = !p ? [window screen] : [p defaultScreen];
}

<doc> The only legitimate way to ask a locked {View} for the {Display}
    onto which the output will appear.  </doc>
Display
  currentDisplay
{
  Display p = printer_display;

  = !p ? [window display] : p;
}

(float, float)
  convertPoint (float, float) (x, y)
      fromView id v
{
  if (self == v)
    = (x, y);
  else if (!superview)
    {
      if (!v)
	= (x, y);
      else
	{
	  float s_x, s_y;
	  (s_x, s_y) = [v convertPoint (0.0, 0.0) toView self];
	  = (x + s_x, y + s_y);
	}
    }
  else
    = [self convertPointFromSuperview [superview convertPoint (x, y)
						 fromView v]];
}

(float, float)
  convertPoint (float, float) (x, y)
	toView id v
{
  if (self == v)
    = (x, y);
  else if (!superview)
    {
      /* We're the window's content view, I guess.   */
      if (!v)
	= (x, y);
      else
	{
	  float o_x, o_y;

	  (o_x, o_y) = [v convertPoint (0.0, 0.0) toView self];
	  = (x - o_x, y - o_y);
	}
    }
  else
    = [superview convertPoint [self convertPointToSuperview (x, y)] toView v];
}

<doc> Return the point {(x, y)}, given in the coordinates of the
    superview, in the coordinates of the receiving view.  This does not
    query the superview, since the computations only involve the frame and
    bounds.  </doc>
(float, float)
  convertPointFromSuperview (float, float) (x, y)
{
  = (x - f_x + b_x, y - f_y + b_y);
}

<doc> Return the point {(x, y)}, given in the coordinates of the receiving
    view, in the coordinates of the superview.  This does not query the
    superview, since the computations only involve the frame and bounds.
    </doc>
(float, float)
  convertPointToSuperview (float, float) (x, y)
{
  = (x - b_x + f_x, y - b_y + f_y);
}

<doc> Return the rectangle {(x, y, w, h)}, given in the coordinates of the
    superview to the coordinates of the receiving view.  </doc>
(float, float, float, float)
  convertRectFromSuperview (float, float, float, float) (x, y, w, h)
{
  float bx = x - f_x + b_x;
  float by = y - f_y + b_y;

  // Scaling, tilting, ...
  // Mon Dec 15 21:44:28 1997, tiggr@tricky.ics.ele.tue.nl
  = (bx, by, w, h);
}

<doc> Return the rectangle {(x, y, w, h)}, given in the coordinates of the
    receiving view, to the coordinates of the super view.  </doc>
(float, float, float, float)
  convertRectToSuperview (float, float, float, float) (x, y, w, h)
{
  float bx = x - b_x + f_x;
  float by = y - b_y + f_y;

  // Scaling, tilting, ...
  // Mon Mar  2 16:27:40 1998, tiggr@gerbil.org
  = (bx, by, w, h);
}

<doc> Be informed that the {descendant} has changed its frame.  The
    default implementation passed this on to its {superview}, if it has
    one.  </doc>
void
  descendantFrameChanged View descendant
{
  if (superview != nil)
    [superview descendantFrameChanged descendant];
}

<doc> Designated initializer.  </doc>
id
  initFrame (float, float, float, float) (x, y, w, h)
pre
  system_display != nil
{
  (f_x, f_y, f_w, f_h) = (x, y, w, h);
  (b_w, b_h) = (w, h);

  = self;
}

<doc> Return the bounds of the receiving view, in its own coordinates.
    </doc>
(float, float, float, float)
  bounds
{
  = (b_x, b_y, b_w, b_h);
}

<doc> Return the frame of the receiving view, in the coordinates of the
    superview.  </doc>
(float, float, float, float)
  frame
{
  = (f_x, f_y, f_w, f_h);
}

<doc> Set the frame of the receiving view to the arguments, expressed in
    the coordinates of the superview.  </doc>
void
  setFrame (float, float, float, float) (x, y, w, h)
{
  (f_x, f_y, f_w, f_h) = (x, y, w, h);
  (b_w, b_h) = (w, h);
}

<doc> Move the view by the specified amount, in the coordinates of the
    superview.  Invokes {moveTo}.  </doc>
void
  moveBy (float, float) (dx, dy)
{
  [self moveTo (f_x + dx, f_y + dy)];
}

<doc> Move the view to the indicated position, in the coordinates of the
    superview.  Notify the superview that a {descendantFrameChanged}.
    </doc>
void
  moveTo (float, float) (x, y)
{
  (f_x, f_y) = (x, y);

  if (superview != nil)
    [superview descendantFrameChanged self];
}

<doc> Inform the {subviews} that their {superviewSizeChanged}.  </doc>
void
  resizeSubviews (float, float) (old_w, old_h)
{
  if (subviews != nil)
    {
      int i, n = [subviews length];

      for (i = 0; i < n; i++)
	[subviews[i] superviewSizeChanged (old_w, old_h)];
    }
}

<doc> Adjust the size of the receiving view by {(dw, dh)}, in the
    coordinates of its superview.  Implemented by invoking {sizeTo}.
    </doc>
void
  sizeBy (float, float) (dw, dh)
{
  [self sizeTo (f_w + dw, f_h + dh)];
}

<doc> Set the size of the receiving view to {(w, h)}, in the coordinates
    of its superview.  Resize the subviews as indicated by their
    {auto_size}, and inform the superview that a {descendantFrameChanged}.
    </doc>
void
  sizeTo (float, float) (w, h)
{
  float o_w = b_w, o_h = b_h;

  (b_w, b_h) = (f_w, f_h) = (w, h);
  [self resizeSubviews (o_w, o_h)];

  if (superview != nil)
    [superview descendantFrameChanged self];
}

<doc> Return the new values for {x} and {w}, given their old values and
    the old and new values for the superview's {w}.  {how} defines how to
    resize.  </doc>
private (float, float)
  autoResize int how
	  of (float, float, float, float) (x, w, sow, snw)
{
  float inc = snw - sow;

  // CCC switch...
  // Wed Oct  2 11:12:18 1996, tiggr@jaguar.ics.ele.tue.nl
  if (!how || !inc)
    void;
  else if (how == VIEW_SIZE_MIN)
    x += inc;
  else if (how == VIEW_SIZE_SPAN)
    w += inc;
  else if (how == VIEW_SIZE_MAX)
    void;
  else if (how == VIEW_SIZE_MAX | VIEW_SIZE_MIN)
    {
      /* Prevent rounding errors showing up in our display.  */
      if (int (inc) & 1 != 0)
	inc += 1.0;
      x += inc / 2.0;
    }
  else if (how == VIEW_SIZE_MIN | VIEW_SIZE_SPAN)
    {
      if (int (inc) & 1 != 0)
	inc += 1.0;
      x += inc / 2.0;
      w += inc / 2.0;
    }
  else if (how == VIEW_SIZE_MAX | VIEW_SIZE_SPAN)
    {
      if (int (inc) & 1 != 0)
	inc += 1.0;
      w += inc / 2.0;
    }
  else if (how == VIEW_SIZE_MAX | VIEW_SIZE_SPAN | VIEW_SIZE_MIN)
    {
      while (int (inc) % 3 != 0)
	inc += 1.0;
      x += inc / 3.0;
      w += inc / 3.0;
    }

  if (w < 0.0)
    w = 0.0;

  = (x, w);
}

<doc> Autoresize the receiving view, according to {auto_size}.  </doc>
void
  superviewSizeChanged (float, float) (so_w, so_h)
{
  if (!auto_size)
    return;

  float n_x = f_x, n_y = f_y, n_w = f_w, n_h = f_h;
  float sn_w, sn_h;

  (, , sn_w, sn_h) = [superview bounds];

  if (so_w != sn_w && (auto_size >> VIEW_SIZE_X_SHIFT) & VIEW_SIZE_MASK != 0)
    (n_x, n_w) = [self autoResize ((auto_size >> VIEW_SIZE_X_SHIFT)
				   & VIEW_SIZE_MASK)
		       of (n_x, n_w, so_w, sn_w)];
  if (so_h != sn_h && (auto_size >> VIEW_SIZE_Y_SHIFT) & VIEW_SIZE_MASK != 0)
    (n_y, n_h) = [self autoResize ((auto_size >> VIEW_SIZE_Y_SHIFT)
				   & VIEW_SIZE_MASK)
		       of (n_y, n_h, so_h, sn_h)];

  boolean size_changed = n_w != f_w || n_h != f_h;
  float o_w = b_w, o_h = b_h;
  [self setFrame (n_x, n_y, n_w, n_h)];
  if (size_changed)
    {
      [self resizeSubviews (o_w, o_h)];
      // [superview descendantFrameChanged]?
      // Mon Dec 15 21:36:09 1997, tiggr@tricky.ics.ele.tue.nl
    }
}

/******************** drawing ********************/
<doc> <h4>Drawing</h4> </doc>

void
  clipToFrame
{
  if (superview != nil)
    [superview clipToFrame (f_x, f_y, f_w, f_h)];
}

void
  clipToFrame (float, float, float, float) (sv_x, sv_y, sv_w, sv_h)
{
  (sv_x, sv_y, sv_w, sv_h)
    = [self convertRectToSuperview (sv_x, sv_y, sv_w, sv_h)];

  if (!superview)
    {
      Display d = [self currentDisplay];

      // Scaling, tilting...
      // Mon Mar  2 16:34:26 1998, tiggr@gerbil.org
      (sv_x, sv_y) = [[d locked_view] convertPoint (sv_x, sv_y) fromView nil];
      [d clipToRect (sv_x, sv_y, sv_w, sv_h)];
    }
  else
    [superview clipToFrame [tag clipRect (f_x, f_y, f_w, f_h)
				to  (sv_x, sv_y, sv_w, sv_h)]];
}

void
  display
{
  [self display (0.0, 0.0, b_w, b_h)];
}

<doc> Display the {View} and it subviews.  </doc>
void
  display (float, float, float, float) (x, y, w, h)
{
  needs_display = NO;

  [self lockFocus];
  [self drawSelf (x, y, w, h)];
  [self unlockFocus];

  if (subviews != nil)
    {
      int i, n = [subviews length];

      for (i = 0; i < n; i++)
	{
	  View sub = subviews[i];
	  float s_x, s_y, s_w, s_h;

	  (s_x, s_y, s_w, s_h) = [tag clipRect [sub frame] to (x, y, w, h)];
	  if (s_w > 0.0 && s_h > 0.0)
	    [sub display [sub convertRectFromSuperview (s_x, s_y, s_w, s_h)]];
	}
    }
}

<doc> {display} the receiving {View}, if needed according to its
    {needs_display} flag.  Otherwise,if displaying is not needed,
    propagate this message to this view's subviews.  </doc>
void
  displayIfNeeded
{
  if (needs_display)
    [self display];
  else if (subviews != nil)
    {
      int i, n = [subviews length];

      for (i = 0; i < n; i++)
	{
	  View sub = subviews[i];
	  [sub displayIfNeeded];
	}
    }
}

<doc> Draw the contents of the receiving view, which already has the focus
    locked.  The default implementation does nothing.  </doc>
void
  drawSelf (float, float, float, float) (x, y, w, h)
{
}

<doc> Lock focus on the receiving view with the {currentDisplay}.  </doc>
void
  lockFocus
{
  [[self currentDisplay] lockView self];
  [self clipToFrame];
}

<doc> Unlock focus on the receiving view with the {currentDisplay}.  The
    receiving view should be the one that is currently locked.  </doc>
void
  unlockFocus
{
  Display d = [self currentDisplay];
  [d unlockView self];
  View v = [d locked_view];
  if (!v)
    [d unclip];
  // Not needed...
  // Mon Mar  2 17:31:36 1998, tiggr@gerbil.org
  //  else
  //    [v clipToFrame];
}

/******************** hierarchy management ********************/
<doc> <h4>Hierarchy management</h4> </doc>

<doc> Add the {view} to the {subviews} of the receiving view.  This will
    set the superview of the {view} to the receiver.  </doc>
void
  addSubview View view
{
  if (!subviews)
    subviews = [MutableObjectArray new];
  [subviews add view];
  [view setSuperview self];
}

<doc> Remove the receiving view from its {superview}.  This also removes
    the receiving view and all subviews from the {window}.  </doc>
void
  removeFromSuperview
{
  if (superview != nil)
    [superview removeSubview self];
  superview = nil;

  [self viewWillMoveToWindow nil];
}

<doc> Remove the {view} from the {subviews}.  </doc>
protected void
  removeSubview View view
{
  if (subviews != nil)
    [subviews removeIdenticalElement view];
}

<doc> Accept a new superview.  Invoke {removeFromSuperview} if {v} is
    {nil}.  </doc>
protected void
  setSuperview View v
{
  if (!v)
    [self removeFromSuperview];
  else
    {
      if (superview != nil && superview != v)
	[[Condition for self class program-condition
		    message "already have super a superview"] signal];

      next_responder = superview = v;

      Window w = [superview window];
      if (window != w)
	[self viewWillMoveToWindow w];
    }
}

<doc> Be informed that this view will be moved to the window {w}.  In
    fact, this method actually performs the move, passing it on to
    subviews.  </doc>
protected void
  viewWillMoveToWindow Window w
{
  window = w;

  if (subviews != nil)
    [subviews makeElementsPerform selector (void viewWillMoveToWindow Window)
	      with w];
}

/******************** responder management ********************/
<doc><h4>Responder management</h4></doc>

<doc> Return {TRUE} iff the receiving view accepts the mouse {event}, even
    though the same event is making the window key window.  The default
    implementation returns {FALSE}.

    This is currently unused because of the mouse-over-to-focus of many
    X11 window managers.  </doc>
boolean
  acceptsFirstMouse MouseEvent event
pre
  [event type] == EVENT_MOUSE_DOWN
{
}

/******************** Archiving ********************/
<doc> <h4>Archiving</h4> </doc>

void
  encodeUsingCoder Encoder coder
{
  if (![coder hasBeenCodedFor [View self]])
    {
      [super encodeUsingCoder coder];
      [coder encodeConditionally window];
      [coder encodeConditionally superview];
      [coder encode subviews];
      [coder encode f_x];
      [coder encode f_y];
      [coder encode f_w];
      [coder encode f_h];
      [coder encode b_x];
      [coder encode b_y];
      [coder encode b_w];
      [coder encode b_h];
      [coder encode auto_size];
    }
}

void
  initWithCoder Decoder coder
{
  if (![coder hasBeenCodedFor [View self]])
    {
      [super initWithCoder coder];
      window = [coder decode];
      superview = [coder decode];
      subviews = [coder decode];
      f_x = [coder decode];
      f_y = [coder decode];
      f_w = [coder decode];
      f_h = [coder decode];
      b_x = [coder decode];
      b_y = [coder decode];
      b_w = [coder decode];
      b_h = [coder decode];
      auto_size = [coder decode];
    }
}

/******************** Aiding event handling ********************/
<doc> <h4>Aiding event handling</h4> </doc>

<doc> Return the {View} in the receiving view's hierarchy, containing the
    point {(x, y)}.  The coordinates are in the receiving view's superview
    coordinate system.  </doc>
id
  hitTest (float, float) (x, y)
{
  (x, y) = (x - f_x, y - f_y);

  if (!(x >= 0.0 && y >= 0.0 && x < f_w && y < f_h))
    return nil;

  if (subviews != nil)
    {
      int i, n = [subviews length];

      for (i = 0; i < n; i++)
	{
	  View sub = subviews[i];
	  View hit = [sub hitTest (x, y)];

	  if (hit != nil)
	    return hit;
	}
    }

  = self;
}

boolean
   mouse (float, float) (m_x, m_y)
  inRect (float, float, float, float) (x, y, w, h)
{
  = [tag point (m_x, m_y) inRect (x, y, w, h)];
}

end;
