<copyright> TAG Window class.
    Written by <a href="mailto:tiggr@es.ele.tue.nl">Pieter J. Schoenmakers</a>

    Copyright &copy; 1996, 1997 Pieter J. Schoenmakers.
    All rights reserved.

    This file is part of TAG.  TAG is distributed under the terms of the
    TAG License, a copy of which can be found in the TAG distribution; see
    the file LICENSE.

    <id>$Id: Window.t,v 1.20 1998/04/15 22:38:37 tiggr Exp $</id>
    </copyright>

/******************** WindowDelegate ********************/

implementation class
WindowDelegate: instance (All)

end;

implementation instance
WindowDelegate: instance (All)

end;

/******************** Window ********************/

implementation class
Window: Responder

<doc> Return the preferred {View} class.  </doc>
class (View)
  viewClass
{
  = [View self];
}

end;

implementation instance
Window
{
  <doc> The title, i.e. the string shown in the title bar.  </doc>
  public String title;

  <doc> Our content view.  </doc>
  public mutable View content_view;

  <doc> Our frame coordinates.  </doc>
  public float f_x, f_y, f_w, f_h;

  <doc> The background color.  </doc>
  Color background;

  <doc> The view for mouse down and subsequent events.  </doc>
  View mouse_view;

  <doc> The first responder within this window.  </doc>
  public Responder first_responder;

  <doc> The initial first responder, i.e. the first responder the first
      time the window is put on the screen.  </doc>
  public mutable View initial_responder;

  <doc> The delegate.  </doc>
  public mutable WindowDelegate delegate;

  <doc> This window's style.  </doc>
  int style;
}

<doc> Display the receiving window.  </doc>
deferred void
  display;

<doc> Return the {Display} of the receiving window.  </doc>
Display
  display
{
  = [[self screen] display];
}

<doc> Return the screen on which this window (mostly) resides.  </doc>
deferred Screen
  screen;

<doc> Return the {Device} description of this {Window}.  This normally
    returns the description of the {screen}.  </doc>
Device
  deviceDescription
{
  = [[self screen] deviceDescription];
}

<doc> Designated initializer.  New windows should be created through the
    screen on which it is to be displayed though.  </doc>
id (self)
  initWithContentRect (float, float, float, float) (x, y, w, h)
	    styleMask int s
	      backing int backing
	       screen Screen screen
{
  (f_x, f_y, f_w, f_h) = (x, y, w, h);
  style = s;
  content_view = [[[isa viewClass] alloc] initFrame (0.0, 0.0, w, h)];
  [content_view set_window self];
  background = [Color light];
}

Color
  backgroundColor
{
  = background;
}

<doc> Be informed of the new status.  Never invoke this method directly.
    </doc>
void
  becomeKeyWindow
{
  /* This will be a no-op for a panel, and will make a normal window both
     the key window and the main window.  */
  [self makeMainWindow];
}

<doc> Be informed of the new status.  Never invoke this method directly.
    </doc>
void
  becomeMainWindow
{
  // Do nothing...
  // Wed Apr 15 21:42:18 1998, tiggr@gerbil.org
}

boolean
  canBecomeKeyWindow
{
  = TRUE;
}

boolean
  canBecomeMainWindow
{
  = TRUE;
}

(float, float)
  convertBaseToScreen (float, float) (x, y)
{
  = (x + f_x, y + f_y);
}

(float, float)
  convertScreenToBase (float, float) (x, y)
{
  = (x - f_x, x - f_y);
}

(float, float, float, float)
  frame
{
  = (f_x, f_y, f_w, f_h);
}

<doc> Order this window to the front, and make it the key window.  </doc>
void
  makeKeyAndOrderFront All sender
{
  [self orderFront sender];
  [self makeKeyWindow];
}

<doc> Make this window the key window, if it can.  </doc>
void
  makeKeyWindow
{
  if ([self canBecomeKeyWindow])
    {
      if (!first_responder)
	first_responder = initial_responder;
      [[Application shared] set_key_window self];
    }
}

<doc> Make this window the main window, if it can.  </doc>
void
  makeMainWindow
{
  if ([self canBecomeMainWindow])
    [[Application shared] set_main_window self];
}

<doc> Accept the new frame coordinates, which have either been induced by
    the user or by the program itself.  </doc>
void
  noteFrame (float, float, float, float) (x, y, w, h)
{
  boolean moved = f_x != x || f_y != y;
  boolean sized = f_w != w || f_h != h;

  (f_x, f_y, f_w, f_h) = (x, y, w, h);
  // [[[self error] print ("new frame: ", x, ' ', y, ' ', w, ' ', h)] nl];

  if (sized)
    [content_view sizeTo (f_w, f_h)];
}

<doc> Raise the window to the front, putting it onto the screen if it
    wasn't already.  </doc>
deferred void
  orderFront All sender;

<doc> Remove the window from the screen, i.e. cause it to not be visible
    anymore.  </doc>
deferred void
  orderOut All sender;

void
  resignKeyWindow
{
}

void
  resignMainWindow
{
}

void
  setBackgroundColor Color c
{
  background = c;
}

void
  setFrame (float, float, float, float) (x, y, w, h)
{
  [self noteFrame (x, y, w, h)];
}

void
  setTitle String t
{
  title = t;
}

OutputStream (s)
  writeFields OutputStream s
{
  = [[super writeFields s] print (" title=", title)];
}

/******************** Events ********************/
<doc><h4>Events</h4></doc>

<doc> Attempt to make the {responder} the {first_responder} of this
    window.  </doc>
boolean (result)
  makeFirstResponder Responder responder
post
  ((result -> first_responder == responder)
   && (!result -> first_responder == old (first_responder)))
{
  if (responder == first_responder)
    return TRUE;
  if (first_responder != nil && ![first_responder resignFirstResponder])
    return FALSE;
  if (responder != nil && [responder becomeFirstResponder])
    {
      first_responder = responder;
      return TRUE;
    }
  first_responder = self;
}

<doc> Send the {event} to the {responder} using the selector {sel}.  </doc>
void
  sendEvent Event event
	 to Responder responder
      using selector sel
{
  [responder perform sel with event];
}

<doc> Feed the action up the responder chain of the window.  </doc>
boolean (result)
  tryToDispatch selector action
	   from All sender
{
  if (first_responder != nil)
    = [first_responder tryToSendAction action from sender];
}

<doc> Try the {delegate} of this window, if the usual {Responder} response
    fails.  </doc>
boolean (result)
  tryToSendAction selector action
	     from All sender
{
  result = [super tryToSendAction action from sender];
  if (!result && delegate != nil && [delegate respondsTo action])
    {
      [delegate perform action with sender];
      = TRUE;
    }
}

/******************** Archiving ********************/
<doc><h4>Archiving</h4></doc>

void
  encodeUsingCoder Encoder coder
{
  // This hasBeenCodedFor stuff probably causes problems with posing
  // classes also adding state to the coding.
  // Thu May  1 15:16:55 1997, tiggr@akebono.ics.ele.tue.nl
  if (![coder hasBeenCodedFor [Window self]])
    {
      [super encodeUsingCoder coder];
      [coder encode content_view];
      [coder encode initial_responder];
      [coder encode background];
      [coder encode style];
      [coder encode f_x];
      [coder encode f_y];
      [coder encode f_w];
      [coder encode f_h];
    }
}

void
  initWithCoder Decoder coder
{
  if (![coder hasBeenCodedFor [Window self]])
    {
      [super initWithCoder coder];
      content_view = [coder decode];
      initial_responder = [coder decode];
      background = [coder decode];
      style = [coder decode];
      f_x = [coder decode];
      f_y = [coder decode];
      f_w = [coder decode];
      f_h = [coder decode];
    }
}

end;
