/* TAG (temporary) image code.
   Written by Pieter J. Schoenmakers <tiggr@es.ele.tue.nl>

   Copyright (C) 1996 Pieter J. Schoenmakers.
   All rights reserved.

   This file is part of TAG.  TAG is distributed under the terms of the
   TAG License, a copy of which can be found in the TAG distribution; see
   the file LICENSE.

   $Id: images.c,v 1.6 1997/12/18 13:55:03 tiggr Exp $  */

#include "tag-r.h"
#include <tom/util.h>
#include <stdio.h>

static tom_object
mkimagerep (char *plane, int w, int h)
{
  char **planes = xcalloc (5, sizeof (*planes));
  tom_object rep = TRT_SEND (_PI_, CREF (tag_BitmapImageRep),
			     SEL (r_alloc));
  int byte_w = (w * 2 + 7) / 8;

  planes[0] = memcpy (xmalloc (byte_w * h), plane, byte_w * h);

  rep = TRT_SEND ((void * (*) (void *, void *, void *, tom_float, tom_float,
			       tom_int, tom_byte, tom_byte, tom_int, tom_int)),
		  rep, SEL (r_initWithBitmapDataPlanes_p_size__ff__bitsPerSample_i_hasAlpha_o_isPlanar_o_colorSpace_i_bytesPerRow_i),
		  planes, w, h, 2, 0, 0,
		  c_tag_Constants_COLOR_SPACE_DEVICE_WHITE, byte_w);
  return rep;
}

tom_object
c_tag_Image_r_bitmapReturnSign (tom_object self, selector cmd)
{
  /* A two-bit WhiteColorSpace image.  */
  char plane[] =
  {
    0xaa, 0xaa, 0xa0, 0x06,	/* 0 */
    0xaa, 0x0a, 0xa2, 0xae,
    0xa8, 0x5a, 0xa2, 0xae,
    0xa1, 0x90, 0x02, 0xae,
    0x86, 0xaa, 0xaa, 0xae,	/* 4 */
    0x1a, 0xaa, 0xaa, 0xae,
    0x86, 0xaa, 0xaa, 0xae,
    0xa2, 0xbf, 0xff, 0xfe,
    0xa8, 0xba, 0xaa, 0xaa,	/* 8 */
    0xaa, 0x3a, 0xaa, 0xaa,
  };

  return mkimagerep (plane, 16, 10);
}

tom_object
c_tag_Image_r_bitmapHighlightedReturnSign (tom_object self, selector cmd)
{
  /* A two-bit WhiteColorSpace image.  */
  char plane[] =
  {
    0xff, 0xff, 0xf0, 0x07,	/* 0 */
    0xff, 0x0f, 0xf3, 0xfb,
    0xfc, 0x5f, 0xf3, 0xfb,
    0xf1, 0xd0, 0x03, 0xfb,
    0xc7, 0xff, 0xff, 0xfb,	/* 4 */
    0x1f, 0xff, 0xff, 0xfb,
    0xcf, 0xff, 0xff, 0xfb,
    0xf3, 0xea, 0xaa, 0xab,
    0xfc, 0xef, 0xff, 0xff,	/* 8 */
    0xff, 0x2f, 0xff, 0xff,
  };

  return mkimagerep (plane, 16, 10);
}

tom_object
c_tag_Image_r_bitmapHorSlider (tom_object self, selector cmd)
{
  /* A two-bit WhiteColorSpace image.  */
  char plane[] =
  {
    0xff, 0xff, 0xff, 0xff, 0x44, /* 0 */
    0xfa, 0xaa, 0x7a, 0xaa, 0x44,
    0xfa, 0xaa, 0x7a, 0xaa, 0x44,
    0xfa, 0xaa, 0x7a, 0xaa, 0x44,
    0xfa, 0xaa, 0x7a, 0xaa, 0x44, /* 4 */
    0xfa, 0xaa, 0x7a, 0xaa, 0x44,
    0xfa, 0xaa, 0x7a, 0xaa, 0x44,
    0xfa, 0xaa, 0x7a, 0xaa, 0x44,
    0xfa, 0xaa, 0x7a, 0xaa, 0x44, /* 8 */
    0xfa, 0xaa, 0x7a, 0xaa, 0x44,
    0xfa, 0xaa, 0x7a, 0xaa, 0x44,
    0xfa, 0xaa, 0x7a, 0xaa, 0x44,
    0xd5, 0x55, 0x65, 0x55, 0x44, /* 12 */
    0x00, 0x00, 0x00, 0x00, 0x05,
  };

  return mkimagerep (plane, 19, 14);
}

tom_object
c_tag_Image_r_bitmapVerSlider (tom_object self, selector cmd)
{
  /* A two-bit WhiteColorSpace image.  */
  char plane[] =
  {
    0xff, 0xff, 0xff, 0xfc, /* 0 */
    0xff, 0xff, 0xff, 0xcc,
    0xfa, 0xaa, 0xaa, 0x4c,
    0xfa, 0xaa, 0xaa, 0x4c,
    0xfa, 0xaa, 0xaa, 0x4c, /* 4 */
    0xfa, 0xaa, 0xaa, 0x4c,
    0xfa, 0xaa, 0xaa, 0x4c,
    0xfa, 0xaa, 0xaa, 0x4c,
    0xfa, 0xaa, 0xaa, 0x4c, /* 8 */
    0xd5, 0x55, 0x55, 0x4c,
    0xff, 0xff, 0xff, 0x8c,
    0xfa, 0xaa, 0xaa, 0x4c,
    0xfa, 0xaa, 0xaa, 0x4c, /* 12 */
    0xfa, 0xaa, 0xaa, 0x4c,
    0xfa, 0xaa, 0xaa, 0x4c,
    0xfa, 0xaa, 0xaa, 0x4c,
    0xfa, 0xaa, 0xaa, 0x4c, /* 16 */
    0xfa, 0xaa, 0xaa, 0x4c,
    0x55, 0x55, 0x55, 0x4c,
    0x00, 0x00, 0x00, 0x0c,
    0x55, 0x55, 0x55, 0x5c
  };

  return mkimagerep (plane, 14, 21);
}
