<copyright> TAG X Colormap class.
    Written by <a href="mailto:tiggr@es.ele.tue.nl">Pieter J. Schoenmakers</a>

    Copyright &copy; 1996, 1997 Pieter J. Schoenmakers.
    All rights reserved.

    This file is part of TAG.  TAG is distributed under the terms of the
    TAG License, a copy of which can be found in the TAG distribution; see
    the file LICENSE.

    <id>$Id: Colormap.t,v 1.5 1997/05/01 16:31:10 tiggr Exp $</id>
    </copyright>

implementation class
Colormap: State

end;

implementation instance
Colormap
{
  <doc> The screen on which this colormap resides.  </doc>
  public Screen screen;

  <doc> The underlying X colormap.  </doc>
  public int colormap;

  <doc> Mapping from color to representation on the display.  </doc>
  MutableMapped colors;

  <doc> The color space on this map.  Is always RGB or WHITE.  </doc>
  public int color_space;

  <doc> The base pixel, i.e. black.  Other colors are relative to this
      pixel value.  </doc>
  int base_pixel;

  <doc> Maximum values of the primary colors.  Grayscale info is stored in
      the red component.  </doc>
  int red_max, green_max, blue_max;

  <doc> Multiplication factors for the binary primary components.  </doc>
  int red_mul, green_mul, blue_mul;
}

<doc> Designated initializer.  </doc>
private id
	init int x_id
	  on Screen s
  colorSpace int cs
	muls (int, int, int) (rm, gm, bm)
       maxes (int, int, int) (rx, gx, bx)
	base int bp	
{
  (colormap, screen, base_pixel, color_space) = (x_id, s, bp, cs);
  (red_max, green_max, blue_max) = (rx, gx, bx);
  (red_mul, green_mul, blue_mul) = (rm, gm, bm);

  colors = [MutableDictionary new];

  Color c = [Color white];
  colors[c] = [XColor for c inMap self];
  c = [Color black];
  colors[c] = [XColor for c inMap self];
  c = [Color light];
  colors[c] = [XColor for c inMap self];
  c = [Color dark];
  colors[c] = [XColor for c inMap self];

  = self;
}

<doc> Return the vital information (base, maxes, muls) in WHITE space.  </doc>
(int, int, int)
  whiteVitals
{
  = (base_pixel, red_max, red_mul);
}

<doc> Return the vital information (base, maxes, muls) in RGB space.  </doc>
(int, int, int, int, int, int, int)
  rgbVitals
{
  = (base_pixel, red_max, green_max, blue_max, red_mul, green_mul, blue_mul);
}

<doc> Get the pixel values to use for 2-bit WHITE.  </doc>
(int, int, int, int)
  whitePixels
{
  = ([[self xcolorFor [Color black]] pixelValue],
     [[self xcolorFor [Color dark]] pixelValue],
     [[self xcolorFor [Color light]] pixelValue],
     [[self xcolorFor [Color white]] pixelValue]);
}

<doc> Get a color.  </doc>
XColor
  xcolorFor Color c
{
  XColor xc = colors[c];

  if (!xc)
    {
      xc = [XColor for c inMap self];

      if (!xc)
	[[[tag error] print ("unable to alloc color ", c)] nl];
      else
	colors[c] = xc;
    }

  = xc;
}

end;
