<copyright> X TAG Display class.
    Written by <a href="mailto:tiggr@es.ele.tue.nl">Pieter J. Schoenmakers</a>

    Copyright &copy; 1996, 1997 Pieter J. Schoenmakers.
    All rights reserved.

    This file is part of TAG.  TAG is distributed under the terms of the
    TAG License, a copy of which can be found in the TAG distribution; see
    the file LICENSE.

    <id>$Id: Display.t,v 1.31 1998/04/15 22:40:10 tiggr Exp $</id>
    </copyright>

<c>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <tom/util.h>
</c>

implementation class
Display: tag.Display,
	Descriptor, DescriptorReadDelegate, Constants

OutputStream
  help OutputStream s
  done MutableKeyed done
{
  class (State) cls = [Display self];

  if (!done[cls])
    {
      [done add cls];

      [[s print "x.Display
  -display <d>          alternative for `:display x:<d>'"] nl];
    }
  = s;
}

<doc> Register this CIT to TAG, and try to find one or more `-display'
    arguments on the command line.  </doc>
void
  load MutableArray arguments
{
  int i, n = [arguments length], in_n = n;
  String s;

  [self registerCIT self withName CIT_NAME];

  for (i = 0; i < n;)
    {
      ByteString a = arguments[i];

      if (["-display" equal a] || [":x-display" equal a])
	{
	  String s = arguments[i + 1];
	  [arguments removeElements (i, 2)];
	  n -= 2;
	  [self registerDisplayRequest
		[[MutableByteString new] print ("x:", s)]];
	}
      else
	i++;
    }

  if (n == in_n)
    {
      s = [Runtime environment]["DISPLAY"];
      if (s != nil)
	[self registerDisplayRequest [[MutableByteString new] print ("x:", s)]];
    }
}

<doc> Open the display known by its {name}, and return it.  Upon success,
    the display has been registered with the {tag.Display} class.  </doc>
instance (id) (display)
  openDisplay String name
{
  display = [[self alloc] init name];

  if (!display)
    [[[tag error] print ("Unable to open X display `", name, '\'')] nl];
  else
    {
      [self registerDisplay display];
      [[RunLoop current] addDescriptorForRead display delegate display];
    }

  if (!preferred_classes)
    {
      preferred_classes = [MutableEqDictionary new];
      // There should be a nicer mechanism to make X11 variants for every
      // window.  Maybe the display-specific functionality should simply
      // be delegated...
      // Wed Apr 15 21:36:07 1998, tiggr@gerbil.org
      preferred_classes[[tag.Window self]] = [x.Window self];
      preferred_classes[[tag.Panel self]] = [x.Panel self];
    }
}

/******************** low-level X manipulation ********************/

<doc> Close the X Display {d}.  </doc>
private extern void
  closeDisplay pointer d;

end;

/****************************** ******************************/

implementation instance
Display
{
  <doc> The underlying X display structure.  </doc>
  public pointer display;

  <doc> The name of this display.  </doc>
  public String name;

  <doc> The screens on this display.  </doc>
  MutableIndexed screens;

  <doc> Windows on this display, mapped on their X window id.  </doc>
  MutableIntDictionary windows;

  <doc> The loaded fonts, keyed on their name (which includes the size).
      </doc>
  MutableDictionary fonts;

  <doc> The current font.  </doc>
  public Font current_font;

  <doc> The X screen of the locked view.  </doc>
  pointer locked_screen;

  <doc> The X window of the locked view.  </doc>
  public int locked_window;

  <doc> The bounds origin of the locked view, in window coordinates.  </doc>
  float bl_x, bl_y;

  <doc> The current path.  This is actually re-used.  </doc>
  Subpath current_path;
}

<doc> Return the {CachedImageRep} class to be used to display on this
    {Display}.  </doc>
class (CachedImageRep)
  cachedImageRepClass
{
  = [CachedImageRep self];
}

<doc> Close this display.  </doc>
void
  close
{
  [[RunLoop current] removeReadDescriptor self];
  descriptor = -1;

  [isa closeDisplay display];
  [isa unregisterDisplay self];
}

<doc> Designated initializer.  </doc>
id
  init String display_name
{
  [self do_init display_name];
  = [super (tag.Display) init];
}

<doc> Open the connection to the display named {display_name}.  Return
    {nil} if opening the display failed.  </doc>
private extern id
  do_init String display_name;

void
  doLockView View v
{
  [super doLockView v];

  if (locked_view != nil)
    {
      [self updateLockedView];
<0>
      [[err print ("doLockView x-origin=(", bl_x, ",", bl_y, ") view=",
		   locked_view)] nl];
</0>
    }
}

void
  doUnlockView View v
{
  [super doUnlockView v];

  if (locked_view != nil)
    [self updateLockedView];
}

void
  updateLockedView
{
  // CCC ...
  // Mon Nov 25 22:14:42 1996, tiggr@jaguar.ics.ele.tue.nl
  Window w = Window ([locked_view window]);

  // CCC ...
  // Fri Oct  4 13:50:56 1996, tiggr@jaguar.ics.ele.tue.nl
  locked_screen = [x.Screen ([w screen]) screen];
  locked_window = [w window];

  (bl_x, bl_y, , ) = [locked_view bounds];
  (bl_x, bl_y) = [locked_view convertPoint (bl_x, bl_y) toView nil];
  bl_y = [w f_h] - bl_y;
}

/******************** drawing ********************/
<doc> <h4>Drawing</h4> </doc>

void
  beep
{
  pointer d = display;
<c>
  XBell (d, 100);
</c>
}

extern void
  drawImage CachedImageRep rep
	 at (float, float) (x, y);

extern void
  drawLine (float, float, float, float) (x1, y1, x2, y2);

extern void
  fillRect (float, float, float, float) (x, y, w, h);

void
  setColor Color c
{
  // CCC ...
  // Mon Nov 25 22:15:38 1996, tiggr@jaguar.ics.ele.tue.nl
  Window w = Window ([locked_view window]);
  Colormap cm = [w colormap];
  [self setXColor [cm xcolorFor c]];
}

extern void
  setFont Font f;

extern void
  setXColor XColor c;

extern void
  show String text
    at (float, float) (x, y);

/******************** blitting ********************/
<doc><h4>Blitting</h4></doc>

<doc> Copy the indicated rectange in the currently {locked_view} to the
    destination point.  </doc>
extern void
  copyRect (float, float, float, float) (rx, ry, rw, rh)
	to (float, float) (x, y);

/******************** clipping ********************/

<doc> Start clipping to the indicated rectangle on the currently
    {locked_window}.  </doc>
extern void
  clipToRect (float, float, float, float) (x, y, w, h);

// Nested clipping...
// Sat Dec 13 01:23:23 1997, tiggr@tricky.ics.ele.tue.nl
<doc> Cancel a previous {clipToRect}.  </doc>
extern void
  unclip;

/******************** screens ********************/

<doc> Return the screens attached to this display.  </doc>
extern Indexed
  screens;

/******************** windows ********************/

<doc> Be informed by the {window} that it now exists on the receiving
    dislay.  </doc>
void
  addWindow Window window
	key int x_win_id
{
  if (!windows)
    windows = [MutableIntDictionary new];

  windows[x_win_id] = window;
}

<doc> Invoke XClearWindow on the window {w}.  This fills the window with
    its background colour (with whatever attributes).  </doc>
void
  clearWindow int w
{
  pointer d = display;
<c>
  XClearWindow (d, w);
</c>
}

<doc> Move and/or resize the {window}.  </doc>
void
  frameWindow int window
	   to (float, float, float, float) (x, y, w, h)
{
  pointer d = display;

<c>
  XMoveResizeWindow (d, window, (int) x, (int) y, (int) w, (int) h);
</c>
}

void
  moveWindow int window
	   to (float, float) (x, y)
{
  pointer d = display;

<c>
  XMoveWindow (d, window, (int) x, (int) y);
</c>
}

<doc> Invoke XMapWindow or XMapRaised on the {window}.  </doc>
void
  mapWindow int window
     raised boolean raised
{
  pointer d = display;
<c>
  if (raised)
    XMapWindow (d, window);
  else
    XMapRaised (d, window);
</c>
}

void
  setTitle String title
  ofWindow int w
{
  pointer d = display;
<c>
  {
    DECL_USEL (tom, _pi__byteStringContents);
    Display *display = d;
    XTextProperty prop;
    tom_int len;
    char *s;

    C_STRING_WITH_TOM_STRING (s, len, title);

    XStringListToTextProperty (&s, 1, &prop);
    XSetWMName (display, w, &prop);
    XFree (prop.value);
  }
</c>
}

<doc> Invoke XUnmapWindow on the {window}.  </doc>
void
  unmapWindow int w
{
  pointer d = display;
<c>
  XUnmapWindow (d, w);
</c>
}

/******************** Dispatching events ********************/
<doc> <h4>{Dispatching events}</h4> </doc>

<doc> Dispatch, while catching problems (can't do that from C sensibly
    yet).  </doc>
void
  dispatch
{
  Condition c;

  if (catch (self)
	bind ((error, {c = condition; [self throw TRUE]; nil;}))
	  ({
	    [self internalDispatch];
	    FALSE;
	  }))
    {
      [[[tag error] print ("error in X dispatching: ", c)] nl];
      [self close];
    }
}

<doc> Return the number of events queued in some way.  </doc>
extern int
  eventsQueued int when;

<doc> See if we can dispatch more events.  </doc>
id
  flushOutput
{
  Condition c;

  if (lock != nil)
    [lock lock];

  if (catch (self)
	bind ((error, {c = condition; [self throw TRUE]; nil;}))
	  ({
	    if ([self eventsQueued QUEUED_AFTER_FLUSH] != 0)
	      [self internalDispatch];
	    FALSE;
	  }))
    {
      [[[tag error] print ("error in X dispatching: ", c)] nl];
      [self close];
    }

  if (lock != nil)
    [lock unlock];

  = self;
}

<doc> Dispatch events while there are any.  </doc>
private extern void
  internalDispatch;

/******************** DescriptorReadDelegate ********************/
<doc> <h4>{DescriptorReadDelegate}</h4> </doc>

void
  readEventOnDescriptor Descriptor d
{
  [self dispatch];
}

/******************** Fonts ********************/
<doc> <h4>Fonts</h4> </doc>

extern pointer
  loadFont String name;

<doc> Return the font with with {name} and {size}.  It is possible that
    the {size} is ignored.  </doc>
Font
  fontNamed String name
       size int size
{
  name = [UniqueString with name];

  Font f;

  if (!fonts)
    fonts = [MutableDictionary new];
  else
    f = fonts[name];

  if (!f)
    {
      pointer font_info = [self loadFont name];

      if (!!font_info)
	{
	  f = [[Font alloc] init name font font_info on self];
	  fonts[name] = f;
	}
    }

  = f;
}

Font
  systemFont
{
  Font f = [self fontNamed "-adobe-helvetica-medium-r-normal*12*" size 12];

  = !f ? [self fontNamed "fixed" size 12] : f;
}

/******************** paths ********************/

<doc> Close the current path.  </doc>
void
  closepath
{
  [self unimplemented cmd];
}

void
  curveto (float, float, float, float, float, float)
	  (x1, y1, x2, y2, x3, y3)
{
  [current_path curveto (x1, y1, x2, y2, x3, y3)];
}

void
  eofill
{
  [self unimplemented cmd];
}

void
  fill
{
  [self unimplemented cmd];
}

<doc> Replace the {current_path} with a {flattened} version.  </doc>
void
  flattenpath
{
  Subpath new_path = [current_path flattened];
  if (new_path != current_path)
    {
      [current_path dispose];
      current_path = new_path;
    }
}

void
  lineto (float, float) (x, y)
{
  [current_path lineto (x, y)];
}

void
  newpath
{
  if (!current_path)
    current_path = [Subpath alloc];
  else
    [current_path newpath];
}

void
  moveto (float, float) (x, y)
{
  [current_path moveto (x, y)];
}

<doc> Forward to the {current_path}.  </doc>
(float, float, float, float) (x1, y1, x2, y2)
  pathbbox
{
  = [current_path pathbbox];
}

void
  stroke
{
  // Re-use the flattened destination...
  // Sun Mar 15 01:27:17 1998, tiggr@gerbil.org
  Subpath path = [current_path flattened];

  [path stroke (display, locked_window) offsets (bl_x, bl_y)];
  if (path != current_path)
    [path dispose];
  [current_path newpath];
}

end;
