<copyright> X TAG Subpath class.
    Written by <a href="mailto:tiggr@gerbil.org">Pieter J. Schoenmakers</a>

    Copyright &copy; 1998 Pieter J. Schoenmakers.
    All rights reserved.

    This file is part of TAG.  TAG is distributed under the terms of the
    TAG License, a copy of which can be found in the TAG distribution; see
    the file LICENSE.

    The flattening code was insprired by
    GNU Ghostscript 3.33 by Aladdin Enterprises.

    <id>$Id: Subpath.t,v 1.3 1998/03/20 16:37:19 tiggr Exp $</id>
    </copyright>

// This TOM/C hybrid file sort-of sucks.
// Sat Mar 14 23:50:40 1998, tiggr@gerbil.org
<c>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <tom/util.h>

#define ACTIONS  ((char *) this->actions)
#define COORDS  ((float *) this->coordinates)

#define OPCODE_MOVETO  1
#define OPCODE_LINETO  3
#define OPCODE_RLINETO  4
#define OPCODE_CURVETO  5
#define OPCODE_RCURVETO  6
</c>

implementation class
Subpath: State
{
  <doc> Subpath opcodes.  The numbers are identical to PostScript encoded
      user paths.  </doc>
  const OPCODE_MOVETO = 1;
  const OPCODE_LINETO = 3;
  const OPCODE_RLINETO = 4;
  const OPCODE_CURVETO = 5;
  const OPCODE_RCURVETO = 6;
}

end;

/****************************** ******************************/

implementation instance
Subpath
{
  // User space, device space, ...
  // Sat Mar 14 23:14:15 1998, tiggr@gerbil.org
  <doc> The coordinates in this path.  Elements are (C) floats.  </doc>
  pointer coordinates;

  <doc> Number of coordinates (2 elements) in and the capacity (in
      coordinates) of the {coordinates} array.  </doc>
  int coords_num, coords_cap;

  <doc> The actions.  Elements are (C) chars.  </doc>
  pointer actions;

  <doc> The number of actions and the maximum capacity of the {actions}
      array.  </doc>
  int actions_num, actions_cap;

  <doc> The number of curves in this subpath.  </doc>
  int num_curves;

  <doc> Iff {TRUE}, this path is closed.  </doc>
  public boolean closed;
}

<doc> Designated initializer.  The given point is the first (and only and
    mandatory) moveto of this subpath.  </doc>
id
  init (float, float) (x, y)
{
  = [super init];
  [self moveto (x, y)];
}

void
  curveto (float, float, float, float, float, float) (x1, y1, x2, y2, x3, y3)
{
  [self add OPCODE_CURVETO];
  [self add (x1, y1)];
  [self add (x2, y2)];
  [self add (x3, y3)];
  num_curves++;
}

id (result)
  flattened
{
  if (!num_curves)
    return self;

  result = [[isa alloc] init [self coordinate 0]];

  float x1, y1, x2, y2, x3, y3;
  int a, c;

  for (a = c = 1; a < actions_num; a++)
    {
      int opc = [self action a];
      if (opc == OPCODE_LINETO)
	{
	  [result add OPCODE_LINETO];
	  [result add [self coordinate c++]];
	}
      else if (opc == OPCODE_CURVETO)
	{
	  (x1, y1) = [self coordinate c++];
	  (x2, y2) = [self coordinate c++];
	  (x3, y3) = [self coordinate c++];

	  // Flatness is a parameter, not a constant...
	  // Sun Mar 15 00:20:21 1998,tiggr@gerbil.org
	  // This is the same as the default dps currentflat
	  [result flatten 1.0 to (x1, y1, x2, y2, x3, y3)];
	}
      else
	[self unimplemented cmd
	      message: [[MutableByteString new] print ("opcode=", opc)]];
    }
}

void
  lineto (float, float) (x, y)
{
  [self add OPCODE_LINETO];
  [self add (x, y)];
}

void
  moveto (float, float) (x, y)
pre
  !actions_num
{
  [self add OPCODE_MOVETO];
  [self add (x, y)];
}

<doc> Convenience method to re-used the storage offered by this path.  </doc>
void
  newpath
{
  actions_num = coords_num = 0;
}

<doc> Return the bounding box of all coordinates in the receiving path.
    </doc>
(float, float, float, float) (x1, y1, x2, y2)
  pathbbox
{
  if (coords_num > 0)
    {
      float x, y;
      int i;

      (x1, y1) = [self coordinate 0];
      (x2, y2) = (x1, y1);
      for (i = 1; i < coords_num; i++)
	{
	  (x, y) = [self coordinate i];
	  if (x < x1)
	    x1 = x;
	  else if (x > x2)
	    x2 = x;
	  if (y < y1)
	    y1 = y;
	  else if (y > y2)
	    y2 = y;
	}
    }
}

<doc> Stroke the receiving path to the {display}.  </doc>
extern void
   stroke (pointer, int) (display, drawable)
  offsets (float, float) (blx, bly);

protected int (action)
  action int i
pre
  0 <= i && i < actions_num
{
<c>
  {
    struct _es_i_x_Subpath *this
      = trt_ext_address ((void *) self, _ei_i_x_Subpath);
    action = ACTIONS[i];
  }
</c>
}

protected void
  add int action
{
<c>
  {
    struct _es_i_x_Subpath *this
      = trt_ext_address ((void *) self, _ei_i_x_Subpath);

    if (this->actions_num == this->actions_cap)
      {
	this->actions_cap = this->actions_cap ? 2 * this->actions_cap : 8;
	this->actions = xrealloc (ACTIONS,
				  this->actions_cap * sizeof (*ACTIONS));
      }
    ACTIONS[this->actions_num++] = action;
  }
</c>
}

protected void
  add (float, float) (x, y)
{
<c>
  {
    struct _es_i_x_Subpath *this
      = trt_ext_address ((void *) self, _ei_i_x_Subpath);

    if (this->coords_cap == this->coords_num)
      {
	this->coords_cap = this->coords_cap ? 2 * this->coords_cap : 8;
	this->coordinates = xrealloc (COORDS,
				      this->coords_cap * 2 * sizeof (*COORDS));
      }

    COORDS[2 * this->coords_num] = x;
    COORDS[2 * this->coords_num + 1] = y;
    this->coords_num++;
  }
</c>
}

protected (float, float) (x, y)
  coordinate int i
pre
  0 <= i && i < coords_num
{
<c>
  {
    struct _es_i_x_Subpath *this
      = trt_ext_address ((void *) self, _ei_i_x_Subpath);
    x = COORDS[2 * i];
    y = COORDS[2 * i + 1];
  }
</c>
}

void
  dealloc
{
  [self dispose];
}

void
  dispose
{
<c>
   struct _es_i_x_Subpath *this
     = trt_ext_address ((void *) self, _ei_i_x_Subpath);
   xfree (ACTIONS);
   this->actions = 0;
   xfree (COORDS);
   this->coordinates = 0;
</c>
}

<doc> Flatten a curve with the {flatness} to the indicated coordinates.
    </doc>
protected void
  flatten float flatness
       to (float, float, float, float, float, float) (x1, y1, x2, y2, x3, y3)
pre
  coords_num > 0
{
  int k;
  float x0, y0;
  (x0, y0) = [self coordinate coords_num - 1];

  float x03 = x0 - x3;
  float y03 = y0 - y3;
  if (x03 < 0.0)
    x03 = -x03;
  if (y03 < 0.0)
    y03 = -y03;
  if (x03 < 16.0 && y03 < 16.0)
    flatness /= 2.0;

  if (flatness < 0.5)
    {
      float m = x03 > y03 ? x03 : y03;
      for (k = 1; m > 1.0; void)
	{
	  m /= 2.0;
	  k++;
	}
    }
  else
    {
      float x12 = x1 - x2;
      float y12 = y1 - y2;
      float dx0 = x0 - x1 - x12;
      float dy0 = y0 - y1 - y12;
      float dx1 = x12 - x2 + x3;
      float dy1 = y12 - y2 + y3;
      float adx0 = dx0 < 0.0 ? -dx0 : dx0;
      float ady0 = dy0 < 0.0 ? -dy0 : dy0;
      float adx1 = dx1 < 0.0 ? -dx1 : dx1;
      float ady1 = dy1 < 0.0 ? -dy1 : dy1;
      float d = 0.75 * ((adx0 > adx1 ? adx0 : adx1)
			+ (ady0 > ady1 ? ady0 : ady1));
      float q = d / flatness;

      for (k = 0; q > 1.0; void)
	{
	  q /= 4.0;
	  k++;
	}
    }
  [self flatten k to (x1, y1, x2, y2, x3, y3)];
}

<doc> Flatten a curve to the receiver using {1 << k} line segments, modulo
    weird circumstances.  </doc>
protected void
  flatten int k
       to (float, float, float, float, float, float) (x1, y1, x2, y2, x3, y3)
{
  float x0, y0;
  (x0, y0) = [self coordinate coords_num - 1];
  float cx = 3.0 * (x1 - x0);
  float bx = 3.0 * (x2 - x1) - cx;
  float ax = x3 - x0 - cx - bx;
  float cy = 3.0 * (y1 - y0);
  float by = 3.0 * (y2 - y1) - cy;
  float ay = y3 - y0 - cy - by;

  if (k == 1)
    {
      float rx = 0.0, ry = 0.0;
      float x = x0 + 0.5 * (cx + 0.5 * (bx + 0.5 * ax));
      float y = y0 + 0.5 * (cy + 0.5 * (by + 0.5 * ay));

      if ((x - x0 > 0.0 ? x - x0 : x0 - x) > 0.5
	  || (y - y0 > 0.0 ? y - y0 : y0 - y) > 0.5)
	{
	  [self add OPCODE_LINETO];
	  [self add (x, y)];
	}
    }
  else if (k > 0)
    {
      // Do it the classic way.  Probably full of rounding errors, but
      // it's type-able fast and easy for now...
      // Sun Mar 15 01:02:31 1998, tiggr@gerbil.org
      int i = 1, n = 1 << k;
      float t, dt = 1.0 / float (n);
      float px = x0, py = y0;

      for (t = dt; i < n; {i++; t += dt;})
	{
	  float t2 = t * t;
	  float t3 = t2 * t;
	  float x = ax * t3 + bx * t2 + cx * t + x0;
	  float y = ay * t3 + by * t2 + cy * t + y0;

	  if ((x - px > 0.0 ? x - px : px - x) > 0.5
	      || (y - py > 0.0 ? y - py : py - y) > 0.5)
	    {
	      [self add OPCODE_LINETO];
	      [self add (x, y)];
	      (px, py) = (x, y);
	    }
	}
    }

  [self add OPCODE_LINETO];
  [self add (x3, y3)];
}

end;
