<copyright> X TAG Window class.
    Written by <a href="mailto:tiggr@es.ele.tue.nl">Pieter J. Schoenmakers</a>

    Copyright &copy; 1996-1998 Pieter J. Schoenmakers.
    All rights reserved.

    This file is part of TAG.  TAG is distributed under the terms of the
    TAG License, a copy of which can be found in the TAG distribution; see
    the file LICENSE.

    <id>$Id: Window.t,v 1.26 1998/07/21 15:07:27 tiggr Exp $</id>
    </copyright>

implementation class
Panel: Window, tag.Panel

end;

implementation instance
Panel

end;

implementation class
Window: tag.Window

<doc> Return the X handle of a newly created X window with the indicated
    dimensions and features on the {screen}.  </doc>
protected extern int
  createWindow (float, float, float, float) (x, y, w, h)
	  with (int, int) (style, backing)
	    on Screen screen;

<doc> Set the background {xcolor} of the X {window} on the {display}.  </doc>
protected extern void
  setBackground XColor xcolor
       ofWindow int window
      onDisplay pointer display;

end;

implementation instance
Window
{
  <doc> Our {Screen}.  </doc>
  public x.Screen screen;

  <doc> The underlying X window.  </doc>
  public int window;
}

<doc> For now, we only use the default screen colormap.  </doc>
Colormap
  colormap
{
  = [screen colormap];
}

<doc> Display the receiving window.  This does nothing if the window is
    currently off-screen.  </doc>
void
  display
{
  if (window != 0)
    {
      [x.Display ([screen display]) clearWindow window];
      [content_view display];
    }
}

<doc> Our {display} is an {x.Display}.  </doc>
redeclare Display
  display;

<doc> Designated initializer.  </doc>
id
  initWithContentRect (float, float, float, float) (x, y, w, h)
	    styleMask int style
	      backing int backing
	       screen Screen scr
{
  screen = scr;

  = [super initWithContentRect (x, y, w, h) styleMask style backing backing
	   screen screen];
}

void
  orderFront All sender
{
  [self putOnServer];
  [[self display] mapWindow window raised YES];
}

void
  orderOut All sender
{
  [[self display] unmapWindow window];
}

void
  setBackgroundColor Color c
{
  [super setBackgroundColor c];

  if (window != 0)
    [isa setBackground [[self colormap] xcolorFor background]
	 ofWindow window onDisplay [[self display] display]];
}

void
  setFrame (float, float, float, float) (x, y, w, h)
{
  float old_w = f_w, old_h = f_h;

  [super setFrame (x, y, w, h)];

  if (window != 0)
    {
      float s_h;

      // CCC 
      // Sun Jul 27 12:18:16 1997, tiggr@tricky.es.ele.tue.nl
      (, s_h) = [Screen ([self screen]) size];

      if (old_w == f_w && old_h == f_h)
	[[self display] moveWindow window to (f_x, s_h - f_y - f_h)];
      else
	[[self display] frameWindow window to (f_x, s_h - f_y - f_h, f_w, f_h)];
    }
}

void
  setTitle String t
{
  if (window != 0)
    [[self display] setTitle t ofWindow window];
  [super setTitle t];
}

<doc> Set this window's {screen} from the {system_display}'s default.
    </doc>
id (self)
  awakeAfterUsingCoder Decoder coder
{
  screen = Screen ([system_display defaultScreen]);

  = [super awakeAfterUsingCoder coder];
}

/******************** low-level event handling ********************/
<doc> <h4>Low-level event handling</h4> </doc>

<doc> Handle a {ConfigureNotify} event.  </doc>
extern void
  configureNotify pointer event;

<doc> Handle an expose event.  </doc>
void
  expose (float, float, float, float) (x, y, w, h)
    left int count
{
  /* Translate the coordinates to TAG Standard Orientation.  */
  y = f_h - y - h;

  [content_view display (x, y, w + 1.0, h + 1.0)];
}

<doc> Handle a mouse event.  </doc>
void
  mouseEvent MouseEvent e
{
  int type = [e type];
  float x, y;

  (x, y) = [e location];
  [e set_location (x, y = f_h - y)];

  if (type == EVENT_MOUSE_DOWN)
    {
      mouse_view = [content_view hitTest (x, y)];
      if (mouse_view != nil)
	{
	  if (mouse_view == first_responder)
	    [self becomeKeyWindow];
	  else if (mouse_view != first_responder
	      && [mouse_view acceptsFirstResponder e])
	    {
	      if ([self makeFirstResponder mouse_view])
		[self becomeKeyWindow];
	    }
	  else
	    [self becomeKeyWindow];
	  [self sendEvent e to mouse_view
		using selector (void mouseDown Event)];
	}
    }
  else if (type == EVENT_MOUSE_UP || type == EVENT_MOUSE_DRAG)
    {
      if (mouse_view == nil)
	void;
      else if (type == EVENT_MOUSE_DRAG)
	[self sendEvent e to mouse_view
	      using selector (void mouseDragged Event)];
      else
	{
	  [self sendEvent e to mouse_view using selector (void mouseUp Event)];
	  mouse_view = nil;
	}
    }
}

/******************** other low-level ********************/

<doc> Make sure the window has a valid X window.  </doc>
void
  putOnServer
{
  // All the casts to x.Display were fixed, I thought...
  // Fri Jul 11 00:04:04 1997, tiggr@tricky.es.ele.tue.nl
  if (!window)
    {
      window = [isa createWindow (f_x, f_y, f_w, f_h)
		    with (style, 0) on screen];
      if (title != nil)
	[self setTitle title];
      [isa setBackground [[self colormap] xcolorFor background]
	   ofWindow window onDisplay [x.Display ([screen display]) display]];
      [x.Display ([screen display]) addWindow self key window];
    }
}

end;
